unit BezDefs;

interface

uses WinTypes, WinProcs;

const
    gt_GraphicObject   = 1;
    gt_LineClass       = 1+2;
    gt_BezierClass     = 1+2+4;
    gt_GroupClass      = 1    +8;
    gt_SelectionClass  = 1    +8+16;
    gt_WorldClass      = 1    +8+16+32;
    gt_ConvertClass    = 1    +8+16+32+64;
    gt_EllipseClass    = 1    +8         +128;
    gt_MultiGroupClass = 1    +8             +256;

    fm_Bezier = 1;
    fm_Clear  = 2;

    AngToRad = Pi/180;
    Rad0=0;
    Rad90=Pi/2;
    Rad180=Pi;
    Rad270=3*Pi/2;
    Rad360=2*Pi;

type
  PPunkt = ^Punkt;
  Punkt = record
              X, Y: Real;
          end;

  PPolyPoint = ^TPolyPoint;
  TPolyPoint = array[0..1001] of TPoint;

  PPolyAnz = ^TPolyAnz;
  TPolyAnz = array[0..1000] of Integer;

 var
    AiFile: Text;
    OutFile: Text;

    VisMin, VisMax: Punkt;
    ZoomX1, ZoomY1, ZoomFaktor: Real;

    BreakConv: Boolean;
    SkipMode:      Integer;
    LastPoint: Punkt;
    FrameMode: Word;
    TheWindow: HWnd;

    PolyPtr: PPolyPoint;

    MinX, MaxX, MinY, MaxY: Real;
    ClipX1, ClipY1, ClipX2, ClipY2: Real;
    DeltaTest,
    MinF: Real;
    ShowConv: Boolean;


function zx(X: Real): Integer;
function zy(Y: Real): Integer;
procedure Trans(var X, Y: Real; NX1, NY1, NX2, NY2, MX, MY: Real);
function PunktKleiner(P1, P2: Punkt): Boolean;
function ATan2(B,A:real):real;
function MinLowerThan(X: Real): Real;
function BlendColor(C1, C2: LongInt; t: Real): LongInt;
procedure BlendPoint(var B: Punkt; P1, P2: Punkt; t: Real);

implementation


function zx(X: Real): Integer;
 var
    r: Real;
begin
  r := (X - ZoomX1)*ZoomFaktor;
  if r>MaxInt then
    r := MaxInt;
  if r<-MaxInt then
    r := -MaxInt;
  zx := trunc(r);
end;

function zy(Y: Real): Integer;
 var
    r: Real;
begin
  r := (ZoomY1 - Y)*ZoomFaktor;
  if r>MaxInt then
    r := MaxInt;
  if r<-MaxInt then
    r := -MaxInt;
  zy := trunc(r);
end;

procedure Trans(var X, Y: Real; NX1, NY1, NX2, NY2, MX, MY: Real);
 var
    r: Real;
begin
  r := MX+NX1*(X-MX)+NX2*(Y-MY);
  Y := MY+NY1*(X-MX)+NY2*(Y-MY);
  X := r;
end;


function PunktKleiner(P1, P2: Punkt): Boolean;
begin
  PunktKleiner := (P1.Y>P2.Y) or
                  (P1.Y=P2.Y) and (P1.X<P2.X);
end;



function ATan2(B,A:real):real;
 var Q,Atan:real;
begin
  if A=0 then
    if B>0 then
      Atan2:=Rad90
    else
      Atan2:=Rad270
  else
    begin
      Q:=B/A;
      if A>0 then
        Atan:=arctan(Q)
      else
        Atan:=Rad180+arctan(Q);
      if Atan<0 then
        Atan2:=Atan+Rad360
      else
        Atan2:=Atan
    end
end;

function MinLowerThan(X: Real): Real;
 var
    dX, Y: Real;
begin
    dX:=1;
    repeat
        dX := dX/2;
        Y:=X-dX/2;
    until X=Y;
    Y := X-dX;
    MinLowerThan := Y;
end;

function BlendColor(C1, C2: LongInt; t: Real): LongInt;
 var
    r1, g1, b1, r2, g2, b2: Real;
begin
  if C1= -1 then
    C1:=rgb(255,255,255);
  if C2=-1 then
    C2:=rgb(255,255,255);
  b1 := (C1 shr 16) and $FF;
  g1 := (C1 shr  8) and $FF;
  r1 := C1 and $FF;
  b2 := (C2 shr 16) and $FF;
  g2 := (C2 shr  8) and $FF;
  r2 := C2 and $FF;
  BlendColor := rgb(trunc(r1+(r2-r1)*t),
                        trunc(g1+(g2-g1)*t),
                        trunc(b1+(b2-b1)*t));
end;

procedure BlendPoint(var B: Punkt; P1, P2: Punkt; t: Real);
begin
  B.X := P1.X+(P2.X-P1.X)*t;
  B.Y := P1.Y+(P2.Y-P1.Y)*t;
end;

end.