unit Bezier1;

interface

uses {WObjects}Objects, WinTypes, WinProcs,
     Defs, BezDefs, Polygone, Poly, GrTools;


type
    IArray = array[0..8000] of Integer;

    PGraphicObject = ^GraphicObject;
    PLine = ^LineClass;
    PBezier = ^BezierClass;
    PGroup = ^GroupClass;
    PEllipse = ^EllipseClass;
    PMultiGroup = ^MultiGroupClass;

    GraphicObject = Object(TObject)
        P1, P2, P3, P4: Punkt;
        PMin, PMax: Punkt;
        Group: PGroup;
        Color: LongInt;
        function    CreateCopy: PGraphicObject;                      virtual;
        function    GetType: Word;                                   virtual;
        function    Compare(GrObPtr: PGraphicObject): Boolean;       virtual;
        procedure   Move(dx, dy: Real);                              virtual;
        procedure   Stretch(xs, ys: Real);                           virtual;
        procedure   Rot(NX1, NY1, NX2, NY2, MX, MY: Real);      virtual;
        procedure   Draw;                                            virtual;
        procedure   Erase;                                           virtual;
        function    Cuts(TestMin, TestMax: Punkt): Boolean;          virtual;
        procedure   WriteOut;                                        virtual;
    end;

    LineClass = Object(GraphicObject)
        constructor Init(X1, Y1, X4, Y4: Real);
        function    CreateCopy: PGraphicObject;                      virtual;
        function    GetType: Word;                                   virtual;
        function    Compare(GrObPtr: PGraphicObject): Boolean;       virtual;
        function    GetTouches(LinePtr: PLine): Integer;             virtual;
        function    Matters(t: Real): Boolean;                       virtual;
        function    ShrinkL: Real;                                   virtual;
        function    ShrinkR: Real;                                   virtual;
        function    Qx(t: Real): Real;                               virtual;
        function    Qy(t: Real): Real;                               virtual;
        function    dQx(t: Real): Real;                              virtual;
        function    dQy(t: Real): Real;                              virtual;
        procedure   Assign(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Real);    virtual;
        procedure   Draw;                                            virtual;
        procedure   GetPoints(PolyPtr: PPolyPoint; var Anz: Word; Revers: Boolean);   virtual;
        procedure   GetPunkte(PolygonPtr: PPolygon);                 virtual;
        procedure   ShowHull;                                        virtual;
        procedure   ShowCut(t: Real);                                virtual;
        procedure   Transform(t1, t2: Real);                         virtual;
        function    GetCut(LinePtr: PLine; var tSelf, tLin: real): Boolean; virtual;
        procedure   WriteOut;                                        virtual;
        procedure   Reverse;                                         virtual;
        destructor  Done;                                            virtual;
    end;

    BezierClass = Object(LineClass)
        constructor Init(X1, Y1,  X2, Y2,  X3, Y3,  X4, Y4: Real);
        function    CreateCopy: PGraphicObject;                      virtual;
        function    GetType: Word;                                   virtual;
        function    Qx(t: Real): Real;                               virtual;
        function    Qy(t: Real): Real;                               virtual;
        function    dQx(t: Real): Real;                              virtual;
        function    dQy(t: Real): Real;                              virtual;
        procedure   Draw;                                            virtual;
        procedure   GetPoints(PolyPtr: PPolyPoint; var Anz: Word; Revers: Boolean);   virtual;
        procedure   GetPunkte(PolygonPtr: PPolygon);                 virtual;
        procedure   Transform(t1, t2: Real);                         virtual;
        function    GetCut(LinePtr: PLine; var tSelf, tLin: real): Boolean; virtual;
        function    GetBezCut(BezPtr: PBezier; var tSelf, tBez: real): Boolean; virtual;
        destructor  Done;                                            virtual;
    end;
    
    GroupClass = Object(GraphicObject)
        GObjects: TCollection; {of PGraphicObject}
        Closed: Boolean;
        FillColor: LongInt;
        PolygonPtr: PPolygon;
        constructor Init(Col, FCol: LongInt);
        function    CreateCopy: PGraphicObject;                      virtual;
        function    GetType: Word;                                   virtual;
        procedure   CalcFrame;                                       virtual;
        procedure   ShowFrame;                                       virtual;
        function    HitsFrameControlPoints(X, Y: Integer): Integer;  virtual;
        procedure   AddObject(GrObPtr: PGraphicObject);              virtual;
        procedure   RemoveObject(GrObPtr: PGraphicObject);           virtual;
        procedure   RemoveAll;                                       virtual;
        function    GetObject(i: Integer):PGraphicObject;            virtual;
        function    Count: Integer;                                  virtual;
        procedure   Move(dx, dy: Real);                              virtual;
        procedure   Stretch(xs, ys: Real);                           virtual;
        procedure   Rot(NX1, NY1, NX2, NY2, MX, MY: Real);           virtual;
        procedure   SetLineColor;                                    virtual;
        function    Snap(Mi, Ma: Punkt): PGraphicObject;             virtual;
        procedure   GetPunkte(APolygonPtr: PPolygon);                 virtual;
        procedure   Draw;                                            virtual;
        procedure   WriteOut;                                        virtual;
        function    TestUnique(GrObPtr: PGraphicObject): Boolean;    virtual;
        procedure   MakeLine;                                        virtual;
        procedure   MakeBezier;                                      virtual;
        function    Sort:Boolean;                                    virtual;
        procedure   Connect;                                         virtual;
        destructor  Done;                                            virtual;
    end;

    EllipseClass = Object(GroupClass)
        IsEllipse: Boolean;
        constructor Init(X1, Y1, X2, Y2: Real; Col, FCol: LongInt);
        function    CreateCopy: PGraphicObject;                      virtual;
        function    GetType: Word;                                   virtual;
        procedure   ShowFrame;                                       virtual;
        function    HitsFrameControlPoints(X, Y: Integer): Integer;  virtual;
        procedure   Rot(NX1, NY1, NX2, NY2, MX, MY: Real);      virtual;
        procedure   AddObject(GrObPtr: PGraphicObject);              virtual;
        procedure   RemoveObject(GrObPtr: PGraphicObject);           virtual;
        procedure   RemoveAll;                                       virtual;
        procedure   Draw;                                            virtual;
        function    Snap(Mi, Ma: Punkt): PGraphicObject;             virtual;
    end;

    MultiGroupClass = Object(GroupClass)
        constructor Init(Col, FCol: LongInt);
        function    CreateCopy: PGraphicObject;                      virtual;
        function    GetType: Word;                                   virtual;
        procedure   Draw;                                            virtual;
        function    Snap(Mi, Ma: Punkt): PGraphicObject;             virtual;
    end;

var
    HitObjectPtr:  PGraphicObject;

function LineToBezier(LinePtr: PLine): PBezier;
procedure Ausgabe(S: String);
procedure AusgabeM(LinePtr: PLine; Start: Boolean);
procedure AusgabeC(LinePtr: PLine; Start: Boolean);
procedure GrClear;
procedure GrRectangle(X1, Y1, X2, Y2: Real; Brush: HBrush );
procedure GrEllipse(X1, Y1, X2, Y2: Real);

implementation

{---------------------------------------------------------}
{-------------------- Hilfsfunktionen --------------------}
{---------------------------------------------------------}

function LineToBezier(LinePtr: PLine): PBezier;
 var
    BezPtr: PBezier;
begin
  LinePtr^.P2.X := (2*LinePtr^.P1.X + LinePtr^.P4.X)/3;
  LinePtr^.P2.Y := (2*LinePtr^.P1.Y + LinePtr^.P4.Y)/3;
  LinePtr^.P3.X := (LinePtr^.P1.X + 2*LinePtr^.P4.X)/3;
  LinePtr^.P3.Y := (LinePtr^.P1.Y + 2*LinePtr^.P4.Y)/3;
  BezPtr := New(PBezier, Init( LinePtr^.P1.X, LinePtr^.P1.Y,
                               LinePtr^.P2.X, LinePtr^.P2.Y,
                               LinePtr^.P3.X, LinePtr^.P3.Y,
                               LinePtr^.P4.X, LinePtr^.P4.Y));
  BezPtr^.Color := LinePtr^.Color;
  LineToBezier := BezPtr;
end;

{---------------------------------------------------------}
{-------------------- Daten Ausgeben ---------------------}
{---------------------------------------------------------}


procedure Aus(S: String);
begin
    Write(OutFile,S);
end;

procedure Ausgabe(S: String);
begin
    WriteLn(OutFile,S);
end;

procedure PAus(P: Punkt);
 var
    Sx, Sy: String;
begin
    str(P.X:7:3,Sx);
    str(P.Y:7:3,Sy);
    Aus(Sx+' '+Sy+' ');
end;

procedure AusgabeM(LinePtr: PLine; Start: Boolean);
begin
    if Start then
        PAus(LinePtr^.P1)
    else
        PAus(LinePtr^.P4);
    Aus('m'+chr(13)+chr(10));
end;

procedure AusgabeC(LinePtr: PLine; Start: Boolean);
begin
    if LinePtr^.GetType=gt_LineClass then
    begin
        if Start then
            PAus(LinePtr^.P4)
        else
            PAus(LinePtr^.P1);
        Aus('l'+chr(13)+chr(10));
    end
    else
    begin
        if Start then
        begin
            PAus(LinePtr^.P2);
            PAus(LinePtr^.P3);
            PAus(LinePtr^.P4);
        end
        else
        begin
            PAus(LinePtr^.P3);
            PAus(LinePtr^.P2);
            PAus(LinePtr^.P1);
        end;
        Aus('c'+chr(13)+chr(10));
    end;
end;

{---------------------------------------------------------}
{-------------------- Graphikfunktionen ------------------}
{---------------------------------------------------------}

procedure GrClear;
begin
    GiveWindowTime;
    SendMessage(TheWindow, wm_Command, cm_ClearBitmap, 0);
end;

procedure GrLine(X1, Y1, X2, Y2: Real);
begin
    GiveWindowTime;
    MoveTo(vhDC, zx(X1), zy(Y1));
    LineTo(vhDC, zx(X2), zy(Y2));
end;

procedure GrRectangle(X1, Y1, X2, Y2: Real; Brush: HBrush );
 var
    H: Real;
 var
    r: TRect;
begin
    if X1>X2 then
    begin
        H  := Y1;
        X1 := X2;
        X2 :=  H;
    end; 
    if Y1<Y2 then
    begin
        H  := Y1;
        Y1 := Y2;
        Y2 :=  H;
    end; 
    SetRect(r, zx(X1),
               zy(Y1),
               zx(X2),
               zy(Y2));
    GiveWindowTime;
    FrameRect(vhDC, r, Brush);
end;

procedure GrEllipse(X1, Y1, X2, Y2: Real);
begin
  GiveWindowTime;
  Ellipse(vhDC, zx(X1), zy(Y2), zx(X2), zy(Y1));
end;



{---------------------------------------------------------}
{-------------------- GraphicObject ----------------------}
{---------------------------------------------------------}

function GraphicObject.CreateCopy: PGraphicObject;
begin
end;

function GraphicObject.GetType: Word;
begin
  GetType := gt_GraphicObject;
end;

function GraphicObject.Compare(GrObPtr: PGraphicObject): Boolean;
begin
  Compare := False;
end;

procedure GraphicObject.Move(dx, dy: Real);
begin
  P1.X := P1.X + dx;
  P1.Y := P1.Y + dy;
  P2.X := P2.X + dx;
  P2.Y := P2.Y + dy;
  P3.X := P3.X + dx;
  P3.Y := P3.Y + dy;
  P4.X := P4.X + dx;
  P4.Y := P4.Y + dy;
  PMin.X := PMin.X + dx;
  PMin.Y := PMin.Y + dy;
  PMax.X := PMax.X + dx;
  PMax.Y := PMax.Y + dy;
end;

procedure GraphicObject.Stretch(xs, ys: Real);
begin
  P1.X := P1.X * xs;
  P1.Y := P1.Y * ys;
  P2.X := P2.X * xs;
  P2.Y := P2.Y * ys;
  P3.X := P3.X * xs;
  P3.Y := P3.Y * ys;
  P4.X := P4.X * xs;
  P4.Y := P4.Y * ys;
  PMin.X := PMin.X * xs;
  PMin.Y := PMin.Y * ys;
  PMax.X := PMax.X * xs;
  PMax.Y := PMax.Y * ys;
end;

procedure GraphicObject.Rot(NX1, NY1, NX2, NY2, MX, MY: Real);
begin
  Trans(P1.X, P1.Y, NX1, NY1, NX2, NY2, MX, MY);
  Trans(P2.X, P2.Y, NX1, NY1, NX2, NY2, MX, MY);
  Trans(P3.X, P3.Y, NX1, NY1, NX2, NY2, MX, MY);
  Trans(P4.X, P4.Y, NX1, NY1, NX2, NY2, MX, MY);
  PMin.X := Min(Min(Min(P1.X, P2.X), P3.X), P4.X);
  PMax.X := Max(Max(Max(P1.X, P2.X), P3.X), P4.X);
  PMin.Y := Min(Min(Min(P1.Y, P2.Y), P3.Y), P4.Y);
  PMax.Y := Max(Max(Max(P1.Y, P2.Y), P3.Y), P4.Y);
end;

procedure GraphicObject.Draw;
begin
end;

procedure GraphicObject.Erase;
begin
end;

procedure GraphicObject.WriteOut;
begin
end;

function GraphicObject.Cuts(TestMin, TestMax: Punkt): Boolean;
 var D: real;
begin
    D := PMax.X-PMin.X + PMax.Y-PMin.Y +
         TestMax.X-TestMin.X + TestMax.Y-TestMin.Y;
    D := D * DeltaTest;
    Cuts := (PMin.X <= TestMax.X+D) and (PMin.Y <= TestMax.Y+D) and
            (PMax.X >= TestMin.X-D) and (PMax.Y >= TestMin.Y-D);
end;

{---------------------------------------------------------}
{-------------------- LineClass     ----------------------}
{---------------------------------------------------------}

constructor LineClass.Init(X1, Y1, X4, Y4: Real);
begin
    Assign(X1, Y1, X4, Y4, X1, Y1, X4, Y4);
end;

function LineClass.CreateCopy: PGraphicObject;
 var
    LinPtr: PLine;
begin
  LinPtr := New(PLine, Init(P1.X, P1.Y, P4.X, P4.Y));
  LinPtr^.Color := Color;
  CreateCopy := LinPtr;
end;

function LineClass.GetType: Word;
begin
  GetType := gt_LineClass;
end;

function LineClass.Compare(GrObPtr: PGraphicObject): Boolean;
begin
    if GrObPtr^.GetType <> GetType then
      Compare := False
    else
      Compare := ((P1.X = GrObPtr^.P1.X) and (P1.Y = GrObPtr^.P1.Y) and
                  (P2.X = GrObPtr^.P2.X) and (P2.Y = GrObPtr^.P2.Y) and
                  (P3.X = GrObPtr^.P3.X) and (P3.Y = GrObPtr^.P3.Y) and
                  (P4.X = GrObPtr^.P4.X) and (P4.Y = GrObPtr^.P4.Y))
                 or
                 ((P1.X = GrObPtr^.P4.X) and (P1.Y = GrObPtr^.P4.Y) and
                  (P2.X = GrObPtr^.P3.X) and (P2.Y = GrObPtr^.P3.Y) and
                  (P3.X = GrObPtr^.P2.X) and (P3.Y = GrObPtr^.P2.Y) and
                  (P4.X = GrObPtr^.P1.X) and (P4.Y = GrObPtr^.P1.Y))
end;

function LineClass.GetTouches(LinePtr: PLine): Integer;            
 var
    F1, F2: Real;
    Result: Integer;
begin
  F1 := min( abs(P1.X-LinePtr^.P1.X) + abs(P1.Y-LinePtr^.P1.Y),
             abs(P1.X-LinePtr^.P4.X) + abs(P1.Y-LinePtr^.P4.Y));
  F2 := min( abs(P4.X-LinePtr^.P4.X) + abs(P4.Y-LinePtr^.P4.Y),
             abs(P4.X-LinePtr^.P1.X) + abs(P4.Y-LinePtr^.P1.Y));
  Result := 0;

  if F1<MinF then
      Result := 1
  else
      Result := 0;

  if F2<MinF then
      Result := Result + 2;

  GetTouches := Result;
end;

function LineClass.Matters(t: Real): Boolean;
 var
    F: Real;
begin
  F := min( abs(P1.X-Qx(t)) + abs(P1.Y-Qy(t)),
            abs(P4.X-Qx(t)) + abs(P4.Y-Qy(t)));

  Matters := (F>=MinF);
end;

function LineClass.ShrinkL: Real;
 var
    r: real;
begin
  r := 0.001;
  while (r<0.5) and (abs(P1.X-Qx(r)) + abs(P1.Y-Qy(r)) < 4*MinF) do
      r := r * 1.5;    
  r := r * 2;
  ShrinkL := r;
  if r<0.5 then
    transform(r,1);
end;

function LineClass.ShrinkR: Real;
 var
    r: real;
begin
  r := 0.001;
  while (r<0.5) and (abs(P4.X-Qx(1-r)) + abs(P4.Y-Qy(1-r)) < 4*MinF) do
      r := r * 1.5;    
  r := r * 2;
  ShrinkR := 1-r;
  if r<0.5 then
    transform(0,1-r);
end;

function LineClass.Qx(t: Real): Real;
begin
    Qx := P1.X+(P4.X-P1.X)*T;
end;

function LineClass.Qy(t: Real): Real;
begin
    Qy := P1.Y+(P4.Y-P1.Y)*T;
end;

function LineClass.dQx(t: Real): Real;
begin
    dQx := (P4.X-P1.X);
end;

function LineClass.dQy(t: Real): Real;
begin
    dQy := (P4.Y-P1.Y);
end;

procedure LineClass.Assign(X1, Y1,  X2, Y2,  X3, Y3,  X4, Y4: Real);
begin
    P1.X := X1;    P1.Y := Y1;
    P2.X := X2;    P2.Y := Y2;
    P3.X := X3;    P3.Y := Y3;
    P4.X := X4;    P4.Y := Y4;
    PMin.X := Min(Min(Min(P1.X, P2.X), P3.X), P4.X);
    PMax.X := Max(Max(Max(P1.X, P2.X), P3.X), P4.X);
    PMin.Y := Min(Min(Min(P1.Y, P2.Y), P3.Y), P4.Y);
    PMax.Y := Max(Max(Max(P1.Y, P2.Y), P3.Y), P4.Y);
end;

procedure LineClass.Draw;
 var
    SaveColor: ColorData;
begin
    SetColor(SaveColor, vhDC, Color, -1);
    GrLine(P1.X, P1.Y, P4.X, P4.Y);
    ResetColor(SaveColor);
end;



procedure LineClass.GetPoints(PolyPtr: PPolyPoint; var Anz: Word; Revers: Boolean);

  procedure AddPoint(X,Y: Real);
  begin
    if Anz<=1000 then
    begin
      PolyPtr^[Anz].X := zx(X);
      PolyPtr^[Anz].Y := zy(Y);
      Anz := Anz + 1;
    end;
  end;

begin
    if revers then
    begin
      AddPoint(P4.X,P4.Y);
      AddPoint(P1.X,P1.Y);
    end
    else
    begin
      AddPoint(P1.X,P1.Y);
      AddPoint(P4.X,P4.Y);
    end;
end;

procedure LineClass.GetPunkte(PolygonPtr: PPolygon);
begin
  PolygonPtr^.AddPoint(P1.X,P1.Y);
  PolygonPtr^.AddPoint(P4.X,P4.Y);
end;

procedure LineClass.ShowHull;
begin
    if ShowConv then
    begin
        GrRectangle(PMin.X, PMin.Y, PMax.X, PMax.Y,
                    GetStockObject(DkGray_Brush));
        GrRectangle(PMin.X, PMin.Y, PMax.X, PMax.Y,
                    GetStockObject(LtGray_Brush));
    end
    else
        GiveWindowTime;
end;

procedure LineClass.ShowCut(t: Real);
 var
    X, Y: Real;
    OldPen: HPen;
begin
    if ShowConv then
    begin
        X := Qx(t);
        Y := Qy(t);
        OldPen := SelectObject(vhDC, TheRedPen);
        GrLine(X-3,Y-3,X+3,Y+3);
        GrLine(X-3,Y+3,X+3,Y-3);
        SelectObject(vhDC, OldPen);
    end
    else
        GiveWindowTime;
end;


procedure LineClass.Transform(t1, t2: Real);
 var NewP1, NewP2, NewP3, NewP4: Punkt;
begin
    NewP1.X := Qx(t1);
    NewP1.Y := Qy(t1);
    NewP4.X := Qx(t2);
    NewP4.Y := Qy(t2);
    Assign( NewP1.X, NewP1.Y,
            NewP4.X, NewP4.Y,
            NewP1.X, NewP1.Y,
            NewP4.X, NewP4.Y);
end;

function LineClass.GetCut(LinePtr: PLine; var tSelf, tLin: Real): Boolean;
 var
    a, b, c, d, e, f, g, h, n: Real;

  function GetLin(a, b, c, d: Real; var t1, t2: Real): Boolean;
   var
      Found: Boolean;
  begin
    Found := False;
    if (a<>0) then
    begin
      t1 := (d-b)/a;
      t2 := 0;
      if (t1>0) and (t1<1) then
        Found := True
      else
      begin
        t1 := (c+d-b)/a;
        t2 := 1;
        if (t1>0) and (t1<1) then
          Found := True;
      end;
    end;
    if (c<>0) and not Found then
    begin
      t1 := 0;
      t2 := (b-d)/c;
      if (t2>0) and (t2<1) then
        Found := True
      else
      begin
        t1 := 1;
        t2 := (a+b-d)/c;
        if (t2>0) and (t2<1) then
          Found := True;
      end;
    end;
    GetLin := Found;
  end;

  function GetParallel(var t1, t2: Real): Boolean;
   var
      Found: Boolean;
  begin
    Found := False;
    if (a<>0) and (c<>0) and ((h-d)/c=(f-b)/a) then
    begin
      t1 := (h-d)/c;
      t2 := 0;
      if (t1>0) and (t1<1) then
        Found := True
      else
      begin
        t1 := (g+h-d)/c;
        t2 := 1;
        if (t1>0) and (t1<1) then
          Found := True
        else
        begin
          t1 := 0;
          t2 := (d-h)/g;
          if (t2>0) and (t2<1) then
            Found := True
          else
          begin
            t1 := 1;
            t2 := (c+d-h)/g;
            if (t2>0) and (t2<1) then
              Found := True;
          end;
        end;
      end;
    end;
    GetParallel := Found;
  end;

begin
    if BreakConv or not Cuts(LinePtr^.PMin, LinePtr^.PMax) or (SkipMode>0) then
        GetCut := false
    else
    begin
        if LinePtr^.GetType <> gt_LineClass then
            GetCut := LinePtr^.GetCut(@Self, tLin, tSelf)
        else
        begin 
            ShowHull;
            LinePtr^.ShowHull;

            GetCut := False;
            a := dQx(0);
            b := P1.X;
            c := dQy(0);
            d := P1.Y;
            e := LinePtr^.dQx(0);
            f := LinePtr^.P1.X;
            g := LinePtr^.dQy(0);
            h := LinePtr^.P1.Y;

            if (a=0) and (e=0) then
              GetCut := GetLin(c,d,g,h, tSelf, tLin)
            else if (c=0) and (g=0) then
              GetCut := GetLin(a,b,e,f, tSelf, tLin)
            else if (c<>0) and (g<>0) and (a/c=e/g) then
              GetCut := GetParallel(tSelf, tLin)
            else if (e=0) then
            begin
                if (g<>0) and (a<>0) then
                begin
                    tSelf := (f-b)/a;
                    tLin := (c*tSelf+d-h)/g;

                    if (tLin<0) and (tLin>0-MinF) then
                        tLin := 0;
                    if (tLin>1) and (tLin<1+MinF) then
                        tLin := 1;
                    if (tSelf<0) and (tSelf>0-MinF) then
                        tSelf := 0;
                    if (tSelf>1) and (tSelf<1+MinF) then
                        tSelf := 1;

                    if (tSelf>=0) and (tSelf<=1) and
                       (tLin>=0) and (tLin<=1) then
                    begin
                        GetCut := True;
                        ShowCut(tSelf);
                    end;
                end
            end
            else
            begin
                n := (c-g*a/e);
                if (n<>0) then
                begin
                    tSelf := (g*(b-f)/e+h-d)/n;
                    tLin  := a/e*tSelf+(b-f)/e;

                    if (tLin<0) and (tLin>0-MinF) then
                        tLin := 0;
                    if (tLin>1) and (tLin<1+MinF) then
                        tLin := 1;
                    if (tSelf<0) and (tSelf>0-MinF) then
                        tSelf := 0;
                    if (tSelf>1) and (tSelf<1+MinF) then
                        tSelf := 1;

                    if (tSelf>=0) and (tSelf<=1) and
                       (tLin>=0) and (tLin<=1) then
                    begin
                        GetCut := True;
                        ShowCut(tSelf);
                    end;
                end;
            end;
        end;
    end;
end; 

procedure LineClass.WriteOut;
 var
    Start: Boolean;
begin
  Start := True;
  if (LastPoint.X=P4.X) and (LastPoint.Y=P4.Y) then
    Start := False
  else if (LastPoint.X<>P1.X) or (LastPoint.Y<>P1.Y) then
    AusgabeM(@Self, Start);
  AusgabeC(@Self, Start);
  if Start then
    LastPoint := P4
  else
    LastPoint := P1;
end;

procedure LineClass.Reverse;
 var
    P: Punkt;
begin
  P  := P1;
  P1 := P4;
  P4 := P;
  P  := P2;
  P2 := P3;
  P3 := P;
end;

destructor LineClass.Done;
begin
    GraphicObject.Done;
end;


{---------------------------------------------------------}
{-------------------- BezierClass   ----------------------}
{---------------------------------------------------------}

constructor BezierClass.Init(X1, Y1,  X2, Y2,  X3, Y3,  X4, Y4: Real);
begin
    Assign(X1, Y1, X2, Y2, X3, Y3, X4, Y4);
end;

function BezierClass.CreateCopy: PGraphicObject;
 var
    BezPtr: PBezier;
begin
    BezPtr := New(PBezier, Init(P1.X, P1.Y, P2.X, P2.Y,
                                P3.X, P3.Y, P4.X, P4.Y));
    BezPtr^.Color := Color;
    CreateCopy := BezPtr;
end;

function BezierClass.GetType: Word;
begin
    GetType := gt_BezierClass;
end;

function BezierClass.Qx(t: Real): Real;
 var
    T2,T3,iT,iT2,iT3 : Real;
begin
    T2 := Sqr(T);
    T3 := T2*T;
    iT := 1-T;
    iT2 := Sqr(iT);
    iT3 := iT*iT2;
    Qx := iT3*P1.X+3*T*iT2*P2.X+3*T2*iT*P3.X+T3*P4.X;
end;

function BezierClass.Qy(t: Real): Real;
 var
    T2,T3,iT,iT2,iT3 : Real;
begin
    T2 := Sqr(T);
    T3 := T2*T;
    iT := 1-T;
    iT2 := Sqr(iT);
    iT3 := iT*iT2;
    Qy := iT3*P1.Y+3*T*iT2*P2.Y+3*T2*iT*P3.Y+T3*P4.Y;
end;

function BezierClass.dQx(t: Real): Real;
begin
    dQx := 3*(P4.X-3*P3.X+3*P2.X-P1.X)*t*t + 2*(3*P3.X-6*P2.X+3*P1.X)*t +
           3*P2.X-3*P1.X;
end;

function BezierClass.dQy(t: Real): Real;
begin
    dQy := 3*(P4.Y-3*P3.Y+3*P2.Y-P1.Y)*t*t + 2*(3*P3.Y-6*P2.Y+3*P1.Y)*t +
           3*P2.Y-3*P1.Y;
end;


procedure BezierClass.Draw;
 var
    i,n: Integer;
    OldX,OldY,NewX,NewY: Real;
    P: Punkt;
    SaveColor: ColorData;
begin
    SetColor(SaveColor, vhDC, Color, -1);
    OldX := P1.X;
    OldY := P1.Y;
    n := trunc(abs(P4.X-P1.X)+abs(P4.Y-P1.Y)+0.5);

    if n<20 then
      n := 5
    else
      n := n div 4;

    for i := 1 to n do
    begin
        NewX := Qx(i/n);
        NewY := Qy(i/n);
        GrLine(OldX, OldY, NewX, NewY);
        OldX := NewX;
        OldY := NewY;
    end;
    ResetColor(SaveColor);
end;


{
procedure BezierClass.Draw;
 var
    i,n: Integer;
    OldX,OldY: Real;
    a, b, c,
    Delta, Delta2, Delta3,
    x, dx, ddx, dddx,
    y, dy, ddy, dddy: Real;
    P: Punkt;
begin
    n := trunc((PMax.X-PMin.X+PMax.Y-PMin.Y)*ZoomFaktor);

    if n<6 then
      n := 3
    else
      n := n div 2;

    Delta := 1.0/n;
    Delta2 := Sqr(Delta);
    Delta3 := Delta*Delta2;

    a := P4.X-3*(P3.X-P2.X)-P1.X;
    b := 3*(P3.X-2*P2.X+P1.X);
    c := 3*(P2.X-P1.X);

    x := P1.X;
    dx := Delta3*a+Delta2*b+Delta*c;
    dddx := 6*Delta3*a;
    ddx := dddx+2*Delta2*b;

    a := P4.Y-3*(P3.Y-P2.Y)-P1.Y;
    b := 3*(P3.Y-2*P2.Y+P1.Y);
    c := 3*(P2.Y-P1.Y);

    y := P1.Y;
    dy := Delta3*a+Delta2*b+Delta*c;
    dddy := 6*Delta3*a;
    ddy := dddx+2*Delta2*b;

    OldX := x;
    OldY := y;

    for i := 1 to n do
    begin
        x := x+dx; dx := dx+ddx; ddx := ddx+dddx;
        y := y+dy; dy := dy+ddy; ddy := ddy+dddy;
        GrLine(OldX, OldY, x,y);
        OldX := x;
        OldY := y;
    end;
end;
}


procedure BezierClass.GetPoints(PolyPtr: PPolyPoint; var Anz: Word; Revers: Boolean);
 var
    VisMinZX, VisMinZY, VisMaxZX, VisMaxZY: Integer;

  procedure AddPoint(n: Integer; X, Y: Real);
   var
      i: Integer;
  begin
    if Anz<=1000 then
    begin
      for i := Anz downto n+1 do
        PolyPtr^[i] := PolyPtr^[i-1];
      PolyPtr^[n].X := zx(X);
      PolyPtr^[n].Y := zy(Y);
      Anz := Anz + 1;
    end;
  end;

  procedure SubDiv(n: Integer; t1, dt: Real);
   var
      m: Integer;
      nx, ny, nl, X, Y, d, dx, dy: Real;
  begin
    AddPoint(n+1, QX(t1+dt), QY(t1+dt));
    dx := PolyPtr^[n+2].X-PolyPtr^[n].X;
    dy := PolyPtr^[n+2].Y-PolyPtr^[n].Y;
    X  := abs(PolyPtr^[n+1].X-PolyPtr^[n].X);
    Y  := abs(PolyPtr^[n+1].Y-PolyPtr^[n].Y);
    if (X>10) or (Y>10) then
      d := 10
    else
    if (dx=0) and (dy=0) then
      d := 0
    else if (abs(dx)>abs(dy)) then
      d := Y-X*dy/dx
    else
      d := X-Y*dx/dy;
{    nx := -dy;
    ny := dx;
    nl := Sqrt(Sqr(nx)+Sqr(ny));
    nx := nx/nl;
    ny := ny/nl;
    X  := nx*(PolyPtr^[n+1].X-PolyPtr^[n].X);
    Y  := ny*(PolyPtr^[n+1].Y-PolyPtr^[n].Y);
    X := Sqr(X);
    Y := Sqr(Y);
}
    if abs(d)>4 then
    begin
      m := Anz;
      if not (((PolyPtr^[n].X<VisMinZX) and (PolyPtr^[n+1].X<VisMinZX)) or
              ((PolyPtr^[n].Y<VisMinZY) and (PolyPtr^[n+1].Y<VisMinZY)) or
              ((PolyPtr^[n].X>VisMaxZX) and (PolyPtr^[n+1].X>VisMaxZX)) or
              ((PolyPtr^[n].Y>VisMaxZY) and (PolyPtr^[n+1].Y>VisMaxZY))) then
        SubDiv(n, t1, 0.5*dt);
      n := n+1+Anz-m;
      if not (((PolyPtr^[n].X<VisMinZX) and (PolyPtr^[n+1].X<VisMinZX)) or
              ((PolyPtr^[n].Y<VisMinZY) and (PolyPtr^[n+1].Y<VisMinZY)) or
              ((PolyPtr^[n].X>VisMaxZX) and (PolyPtr^[n+1].X>VisMaxZX)) or
              ((PolyPtr^[n].Y>VisMaxZY) and (PolyPtr^[n+1].Y>VisMaxZY))) then
        SubDiv(n, t1+dt, 0.5*dt);
    end;
  end;

begin
  if Revers then
  begin
    AddPoint(Anz, P4.X, P4.Y);
    AddPoint(Anz, P1.X, P1.Y);
  end
  else
  begin
    AddPoint(Anz, P1.X, P1.Y);
    AddPoint(Anz, P4.X, P4.Y);
  end;

  if Cuts(VisMin, VisMax) then
  begin
    VisMinZX := zx(VisMin.X);
    VisMinZY := zy(VisMax.Y);
    VisMaxZX := zx(VisMax.X);
    VisMaxZY := zy(VisMin.Y);
    if Revers then
      SubDiv(Anz-2, 1, -0.5)
    else
      SubDiv(Anz-2, 0, 0.5);
  end;
end;
{
procedure BezierClass.GetPoints(PolyPtr: PPolyPoint; var Anz: Word; Revers: Boolean);
 var
    i,n: Integer;

  procedure AddPoint(X,Y: Real);
  begin
    if Anz<=1000 then
    begin
      PolyPtr^[Anz].X := zx(X);
      PolyPtr^[Anz].Y := zy(Y);
      Anz := Anz + 1;
    end;
  end;

begin
    n := trunc(abs(P4.X-P1.X)+abs(P4.Y-P1.Y)+0.5);
    if n<20 then n := 10
            else n := n div 2;
    n := 5;
    if revers then
    begin
      for i := n downto 0 do
        AddPoint(Qx(i/n),QY(i/n));
    end
    else
    begin
      for i := 0 to n do
          AddPoint(QX(i/n),QY(i/n));
    end;
end;
}

procedure BezierClass.GetPunkte(PolygonPtr: PPolygon);
 var
    PktPtr, Pkt1Ptr, Pkt2Ptr: PPunkt;

  procedure SubDiv(n: Integer; t1, dt: Real);
   var
      m: Integer;
      nx, ny, nl, X, Y, d, dx, dy: Real;
  begin
    PolygonPtr^.AddAtPoint(n+1, QX(t1+dt), QY(t1+dt));
    Pkt2Ptr := PolygonPtr^.At(n+2);
    Pkt1Ptr := PolygonPtr^.At(n+1);
    PktPtr := PolygonPtr^.At(n);
    dx := zx(Pkt2Ptr^.X)-zx(PktPtr^.X);
    dy := zy(Pkt2Ptr^.Y)-zy(PktPtr^.Y);
    X  := abs(zx(Pkt1Ptr^.X)-zx(PktPtr^.X));
    Y  := abs(zy(Pkt1Ptr^.Y)-zy(PktPtr^.Y));
    if (X>10) or (Y>10) then
      d := 10
    else
    if (dx=0) and (dy=0) then
      d := 0
    else if (abs(dx)>abs(dy)) then
      d := Y-X*dy/dx
    else
      d := X-Y*dx/dy;
{    nx := -dy;
    ny := dx;
    nl := Sqrt(Sqr(nx)+Sqr(ny));
    nx := nx/nl;
    ny := ny/nl;
    X  := nx*(PolyPtr^[n+1].X-PolyPtr^[n].X);
    Y  := ny*(PolyPtr^[n+1].Y-PolyPtr^[n].Y);
    X := Sqr(X);
    Y := Sqr(Y);
}
    if abs(d)>4 then
    begin
      m := PolygonPtr^.Count;
      if not (((PktPtr^.X<VisMin.X) and (Pkt1Ptr^.X<VisMin.X)) or
              ((PktPtr^.Y<VisMin.Y) and (Pkt1Ptr^.Y<VisMin.Y)) or
              ((PktPtr^.X>VisMax.X) and (Pkt1Ptr^.X>VisMax.X)) or
              ((PktPtr^.Y>VisMax.Y) and (Pkt1Ptr^.Y>VisMax.Y))) then
        SubDiv(n, t1, 0.5*dt)
      else
        PolygonPtr^.MaxZoom := 0;
      n := n+1+PolygonPtr^.Count-m;
      Pkt1Ptr := PolygonPtr^.At(n+1);
      PktPtr := PolygonPtr^.At(n);
      if not (((PktPtr^.X<VisMin.X) and (Pkt1Ptr^.X<VisMin.X)) or
              ((PktPtr^.Y<VisMin.Y) and (Pkt1Ptr^.Y<VisMin.Y)) or
              ((PktPtr^.X>VisMax.X) and (Pkt1Ptr^.X>VisMax.X)) or
              ((PktPtr^.Y>VisMax.Y) and (Pkt1Ptr^.Y>VisMax.Y))) then
        SubDiv(n, t1+dt, 0.5*dt)
      else
        PolygonPtr^.MaxZoom := 0;
    end;
  end;

begin
  if PolygonPtr^.MaxZoom<>0 then
   PolygonPtr^.MaxZoom := ZoomFaktor;
  PolygonPtr^.AddPoint(P1.X, P1.Y);
  PolygonPtr^.AddPoint(P4.X, P4.Y);

  if Cuts(VisMin, VisMax) then
    SubDiv(PolygonPtr^.Count-2, 0, 0.5)
  else
    PolygonPtr^.MaxZoom := 0;
end;


procedure BezierClass.Transform(t1, t2: Real);
 var NewP1, NewP2, NewP3, NewP4: Punkt;
begin
    NewP1.X := Qx(t1);
    NewP1.Y := Qy(t1);
    NewP4.X := Qx(t2);
    NewP4.Y := Qy(t2);
    NewP2.X := (t2-t1)*dQx(t1)/3+NewP1.X;
    NewP2.Y := (t2-t1)*dQy(t1)/3+NewP1.Y;
    NewP3.X := NewP4.X-(t2-t1)*dQx(t2)/3;
    NewP3.Y := NewP4.Y-(t2-t1)*dQy(t2)/3;
    Assign( NewP1.X, NewP1.Y,
            NewP2.X, NewP2.Y,
            NewP3.X, NewP3.Y,
            NewP4.X, NewP4.Y);
end;

function BezierClass.GetCut(LinePtr: PLine; var tSelf, tLin: Real): Boolean;
 var
    PolyX, PolyY, LineX, LineY,
    HelpPoly1, HelpPoly2: PolynomClass;
    Anz, i: Integer;
    n: Koeff;
    t1, t2: Real;

begin
    if LinePtr^.GetType <> gt_LineCLass then
        GetCut := GetBezCut(PBezier(LinePtr), tSelf, tLin)
    else if BreakConv or not Cuts(LinePtr^.PMin, LinePtr^.PMax) or (SkipMode>0) then
        GetCut := false
    else
    begin
        ShowHull;
        LinePtr^.ShowHull;


        PolyX.Init( -P1.X+3.0*P2.X-3.0*P3.X+P4.X, 3.0*P1.X-6.0*P2.X+3.0*P3.X, -3.0*P1.X+3.0*P2.X, P1.X);
        PolyY.Init( -P1.Y+3.0*P2.Y-3.0*P3.Y+P4.Y, 3.0*P1.Y-6.0*P2.Y+3.0*P3.Y, -3.0*P1.Y+3.0*P2.Y, P1.Y);
        LineX.Init( 0,0, LinePtr^.dQx(0), LinePtr^.P1.X);
        LineY.Init( 0,0, LinePtr^.dQy(0), LinePtr^.P1.Y);

        if LinePtr^.dQx(0)=0 then
        begin
            HelpPoly1.CreateCopy(@PolyX);
            HelpPoly1.Sub(@LineX);
            HelpPoly2.CreateCopy(@LineY);
            HelpPoly2.Invert1;
        end
        else if LinePtr^.dQy(0)=0 then
        begin
            HelpPoly1.CreateCopy(@PolyY);
            HelpPoly1.Sub(@LineY);
            HelpPoly2.CreateCopy(@LineX);
            HelpPoly2.Invert1;
        end
        else
        begin
            HelpPoly1.CreateCopy(@PolyX);
            HelpPoly2.CreateCopy(@LineX);
            HelpPoly2.Invert1;
            HelpPoly1.Combine1(@HelpPoly2);
            HelpPoly1.Combine1(@LineY);
            HelpPoly1.Sub(@PolyY);
        end;

        Anz := HelpPoly1.NullStellen(n, 0-MinF, 1+MinF);

        GetCut := False;
        for i := 0 to Anz-1 do
        if Anz<>-1 then begin
            t1 := n[i];
            if LinePtr^.dQx(0)<>0 then
                t2 := HelpPoly2.f(PolyX.f(t1))
            else
                t2 := HelpPoly2.f(PolyY.f(t1));

            if (t1<0) and (t1>0-MinF) then
                t1 := 0;
            if (t1>1) and (t1<1+MinF) then
                t1 := 1;
            if (t2<0) and (t2>0-MinF) then
                t2 := 0;
            if (t2>1) and (t2<1+MinF) then
                t2 := 1;

            if (t2>=0) and (t2<=1) then
            begin
                tSelf := t1;
                tLin  := t2;
                GetCut := True;
                ShowCut(tSelf);
                Anz := -1;
            end;
        end;

        PolyX.Done;
        PolyY.Done;
        LineX.Done;
        LineY.Done;
        HelpPoly1.Done;
        HelpPoly2.Done;

    end;
end;

function BezierClass.GetBezCut(BezPtr: PBezier; var tSelf, tBez: Real): Boolean;
 var LocalBezPtr: PBezier;
     FB, FSelf: Real;
     i: Integer;
begin
    if BreakConv or not Cuts(BezPtr^.PMin, BezPtr^.PMax) or (SkipMode>0) then
        GetBezCut := false
    else
    begin
        FB := abs(BezPtr^.P1.X-BezPtr^.P4.X)+
              abs(BezPtr^.P1.Y-BezPtr^.P4.Y);
        FSelf := abs(P1.X-P4.X)+abs(P1.Y-P4.Y);
        if FB+FSelf<=MinF then  {MinF=0.001}
        begin
            tSelf := 0.5;
            tBez  := 0.5;
            GetBezCut := True;
            showCut(tSelf);
        end
        else if FB>FSelf then
        begin
            for i := 0 to 1 do
            begin
                LocalBezPtr := PBezier(BezPtr^.CreateCopy);
                LocalBezPtr^.Transform(0.5*i, 0.5*i+0.5);
                LocalBezPtr^.ShowHull;
                if LocalBezPtr^.GetBezCut(@Self, tBez, tSelf) then
                begin
                    tBez := 0.5*(tBez+i);
                    GetBezCut := True;
                    exit;
                end;
                Dispose(LocalBezPtr, Done);
            end;
            GetBezCut := False;
        end
        else
        begin
            for i := 0 to 1 do
            begin
                LocalBezPtr := PBezier(CreateCopy);
                LocalBezPtr^.Transform(0.5*i, 0.5*i+0.5);
                LocalBezPtr^.ShowHull;
                if LocalBezPtr^.GetBezCut(BezPtr, tSelf, tBez) then
                begin
                    tSelf := 0.5*(tSelf+i);
                    GetBezCut := True;
                    exit;
                end;
                Dispose(LocalBezPtr, Done);
            end;
            GetBezCut := False;
        end;
    end;
end; 


destructor BezierClass.Done;
begin
    LineClass.Done;
end;

{---------------------------------------------------------}
{-------------------- GroupClass    ----------------------}
{---------------------------------------------------------}


constructor GroupClass.Init(Col, FCol: LongInt);
begin
  GObjects.Init(5,0);
  Color := Col;
  FillColor := FCol;
  Closed := False;
  PolygonPtr := nil;
end;


function GroupClass.CreateCopy: PGraphicObject;
 var
    GroupPtr: PGroup;
    i: Integer;
begin
  new(GroupPtr, Init(Color, FillColor));
  for i := 0 to Count-1 do
    GroupPtr^.AddObject(GetObject(i)^.CreateCopy);
  GroupPtr^.Closed := Closed;
  GroupPtr^.PMin := PMin;
  GroupPtr^.PMax := PMax;
  CreateCopy := GroupPtr;
end;

function GroupClass.GetType: Word;
begin
  GetType := gt_GroupClass;
end;

procedure GroupClass.CalcFrame;
 var
    i: Integer;
    GrObPtr: PGraphicObject;
begin
  if Count=0 then
  begin
    PMin.X := 0;
    PMin.Y := 0;
    PMax.X := 0;
    PMax.Y := 0;
  end
  else
  begin
    GrObPtr := GetObject(0);
    PMin := GrObPtr^.PMin;
    PMax := GrObPtr^.PMax;
    for i := 1 to Count-1 do
    begin
      GrObPtr := GetObject(i);
      PMin.X := min(PMin.X,GrObPtr^.PMin.X);
      PMin.Y := min(PMin.Y,GrObPtr^.PMin.Y);
      PMax.X := max(PMax.X,GrObPtr^.PMax.X);
      PMax.Y := max(PMax.Y,GrObPtr^.PMax.Y);
    end;
  end;
  P1 := PMin;
  P2 := PMax;
  P3 := PMin;
  P4 := PMax;
end;

procedure GroupClass.AddObject(GrObPtr: PGraphicObject);        
begin
  if GObjects.Limit<=Count then
    GObjects.SetLimit(Count+5);
  GObjects.Insert(GrObPtr);
  GrObPtr^.Group := @Self;
{  CalcFrame;}
end;

procedure GroupClass.RemoveObject(GrObPtr: PGraphicObject);
begin
  GObjects.Delete(GrObPtr);
{  CalcFrame;}
end;

procedure GroupClass.RemoveAll;
begin
  GObjects.DeleteAll;
end;

function GroupClass.GetObject(i: Integer):PGraphicObject;
begin
  if (i<0) or (i>=Count) then
    GetObject := nil
  else
    GetObject := GObjects.At(i);
end;

function GroupClass.Count: Integer;                                  
begin
  Count := GObjects.Count;
end;

procedure GroupClass.Move(dx, dy: Real);                              
 var
    i: Integer;
begin
  if (PolygonPtr<>nil) and (PolygonPtr^.MaxZoom>=ZoomFaktor) then
    PolygonPtr^.Move(dx,dy);
  for i := 0 to Count-1 do
    GetObject(i)^.Move(dx,dy);
  CalcFrame;
end;

procedure GroupClass.Stretch(xs, ys: Real);                           
 var
    i: Integer;
begin
  if (PolygonPtr<>nil) and (PolygonPtr^.MaxZoom>=ZoomFaktor) then
    PolygonPtr^.Stretch(xs, ys);
  for i := 0 to Count-1 do
    GetObject(i)^.Stretch(xs,ys);
  CalcFrame;
end;

procedure GroupClass.Rot(NX1, NY1, NX2, NY2, MX, MY: Real);      
 var
    i: Integer;
begin
  if (PolygonPtr<>nil) and (PolygonPtr^.MaxZoom>=ZoomFaktor) then
    PolygonPtr^.Rot(NX1, NY1, NX2, NY2, MX, MY);
  for i := 0 to Count-1 do
    GetObject(i)^.Rot(NX1, NY1, NX2, NY2, MX, MY);
  CalcFrame;
end;

procedure GroupClass.SetLineColor;
 var
    i: Integer;
begin
  Color := FillColor;
  if not closed then
  begin
    for i := 0 to Count-1 do
      if (GetObject(i)^.GetType and gt_GroupClass) = gt_GroupClass then
        PGroup(GetObject(i))^.SetLineColor;
  end;
end;

procedure GroupClass.GetPunkte(APolygonPtr: PPolygon);                 
 var
    i: Integer;
begin
  for i := 0 to Count-1 do
    PLine(GetObject(i))^.GetPunkte(APolygonPtr);
end;

procedure GroupClass.Draw;                                            
 var
    i: Integer;
    GrObPtr: PGraphicObject;
    Anz: Word;
    SaveColor: ColorData;
begin
  if Cuts(VisMin, VisMax) then
  begin
    if not Closed then
    begin
      for i := 0 to Count-1 do
      begin
        GrObPtr := GetObject(i);
        if GrObPtr^.Cuts(VisMin, VisMax) then
          GrObPtr^.Draw;
      end;
    end
    else
    begin
      i := 0;

{
      while (i<Count) and ((GetObject(i)^.GetType and gt_LineClass)<>gt_LineClass) do
      begin
        GrObPtr := GetObject(i);
        if GrObPtr^.Cuts(VisMin, VisMax) then
          GrObPtr^.Draw;
        i := i+1;
      end;
}
      if (PolygonPtr=nil) or (PolygonPtr^.MaxZoom<ZoomFaktor) then
      begin
        if PolygonPtr<>nil then
          Dispose(PolygonPtr, Done);

        New(PolygonPtr, Init);
        GetPunkte(PolygonPtr);
      end;
      Anz := PolygonPtr^.GetPoints(PolyPtr);

      if Anz>1 then
      begin
        SetColor(SaveColor, vhDC, Color, FillColor);
        Polygon(vhDC,PolyPtr^,Anz);
        ResetColor(SaveColor);
      end;

    end;
  end;
end;



procedure GroupClass.ShowFrame;
 var
    X1, Y1, X2, Y2: Integer;
    XM, YM: Integer;
    i: Integer;
    GrObPtr: PGraphicObject;

 procedure ShowPoint(X, Y: Integer);
  var
     r: TRect;
 begin
   SetRect(r, X-3, Y-3, X+3, Y+3);
   InvertRect(vhDC,r);
 end;

 procedure ShowSmallPoint(X, Y: Integer);
  var
     r: TRect;
 begin
   SetRect(r, X-2, Y-2, X+2, Y+2);
   InvertRect(vhDC,r);
 end;

begin
  if Count>0 then
  begin
    if (FrameMode and fm_Bezier)<>0 then
    begin
      for i := 0 to Count-1 do
      begin
        GrObPtr := GetObject(i);
        if (GrObPtr^.GetType and gt_GroupClass) = gt_GroupClass then
          PGroup(GrObPtr)^.ShowFrame
        else
        begin
          ShowPoint(zx(GrObPtr^.P1.X), zy(GrObPtr^.P1.Y));
          ShowPoint(zx(GrObPtr^.P4.X), zy(GrObPtr^.P4.Y));
          ShowSmallPoint(zx(GrObPtr^.P4.X), zy(GrObPtr^.P4.Y));
          if GrObPtr^.GetType = gt_BezierClass then
          begin
            ShowSmallPoint(zx(GrObPtr^.P2.X), zy(GrObPtr^.P2.Y));
            ShowSmallPoint(zx(GrObPtr^.P3.X), zy(GrObPtr^.P3.Y));

            InvertLine(vhDC, zx(GrObPtr^.P1.X), zy(GrObPtr^.P1.Y),
                              zx(GrObPtr^.P2.X), zy(GrObPtr^.P2.Y), $3333);
            InvertLine(vhDC, zx(GrObPtr^.P4.X), zy(GrObPtr^.P4.Y),
                              zx(GrObPtr^.P3.X), zy(GrObPtr^.P3.Y), $3333);
          end;
        end;
      end;
    end
    else
    begin
      X1 := zx(PMin.X);
      Y2 := zy(PMin.Y);
      X2 := zx(PMax.X);
      Y1 := zy(PMax.Y);
      XM := (X1+X2) div 2;
      YM := (Y1+Y2) div 2;
      X1 := X1-4;
      Y1 := Y1-4;
      X2 := X2+4;
      Y2 := Y2+4;
      ShowPoint(X1, Y1);
      ShowPoint(XM, Y1);
      ShowPoint(X2, Y1);
      ShowPoint(X2, YM);
      ShowPoint(X2, Y2);
      ShowPoint(XM, Y2);
      ShowPoint(X1, Y2);
      ShowPoint(X1, YM);
    end;
  end;
end;

function GroupClass.HitsFrameControlPoints(X, Y: Integer): Integer;
 var
    X1, Y1, X2, Y2, XM, YM: Integer;
    i: Integer;
    GrObPtr: PGraphicObject;
    HFCP: Integer;

 procedure TestPoint(pX, pY, Nr: Integer);
 begin
   if (X>=px-3) and (X<=px+3) and
      (Y>=py-3) and (Y<=py+3) then
     HFCP := Nr
 end;

 procedure TestSmallPoint(pX, pY, Nr: Integer);
 begin
   if (X>=px-2) and (X<=px+2) and
      (Y>=py-2) and (Y<=py+2) then
     HFCP := Nr
 end;

begin
  HFCP := -1;
  if Count>0 then
  begin
    if (FrameMode and fm_Bezier)<>0 then
    begin
      for i := 0 to Count-1 do
      begin
        GrObPtr := GetObject(i);
        if (GrObPtr^.GetType and gt_GroupClass) = gt_GroupClass then
          HFCP := PGroup(GrObPtr)^.HitsFrameControlPoints(X,Y)
        else
        begin
          TestPoint(zx(GrObPtr^.P1.X), zy(GrObPtr^.P1.Y), 0);
          TestPoint(zx(GrObPtr^.P4.X), zy(GrObPtr^.P4.Y), 3);
          if GrObPtr^.GetType=gt_BezierClass then
          begin
            TestSmallPoint(zx(GrObPtr^.P2.X), zy(GrObPtr^.P2.Y), 1);
            TestSmallPoint(zx(GrObPtr^.P3.X), zy(GrObPtr^.P3.Y), 2);
          end;
          if HFCP<>-1 then
            HitObjectPtr := GrObPtr;
        end;
        if HFCP<>-1 then
        begin
          HitsFrameControlPoints := HFCP;
          exit;
        end;
      end;
    end
    else
    begin
      X1 := zx(PMin.X);
      Y2 := zy(PMin.Y);
      X2 := zx(PMax.X);
      Y1 := zy(PMax.Y);
      XM := (X1+X2) div 2;
      YM := (Y1+Y2) div 2;
      X1 := X1-4;
      Y1 := Y1-4;
      X2 := X2+4;
      Y2 := Y2+4;
      TestPoint(X1, Y1, 3);
      TestPoint(XM, Y1, 1);
      TestPoint(X2, Y1, 5);
      TestPoint(X2, YM, 4);
      TestPoint(X2, Y2,12);
      TestPoint(XM, Y2, 8);
      TestPoint(X1, Y2,10);
      TestPoint(X1, YM, 2);
    end;
  end;
  HitsFrameControlPoints := HFCP;
end;

function GroupClass.Snap(Mi, Ma: Punkt): PGraphicObject;
 var
    LocalLine: LineClass;
    i, j: Integer;
    Dummy1, Dummy2: Real;
    Found, OldShowConv: Boolean;
    sn, GrObPtr: PGraphicObject;
    TempRgn: HRgn;
    r: TRect;
begin
  snap := nil;
  if Cuts(Mi, Ma) then
  begin
    Sn := nil;

    if Closed and (PolygonPtr<>nil) and (FillColor<>-1) then
    begin
      SetRect(r, zx(Mi.X), zy(Ma.Y), zx(Ma.X), zy(Mi.Y));
      TempRgn := PolygonPtr^.GetRgn;
      if RectInRegion(TempRgn, r) then
        Sn := @Self;
      DeleteObject(TempRgn);
    end
    else
    begin
      OldShowConv := ShowConv;
      ShowConv := False;
      LocalLine.Init(Mi.X, Mi.Y, Ma.X, Ma.Y);
      i := Count-1;
      while (i>=0) and (sn=nil) do
      begin
        GrObPtr := GetObject(i);
        if (GrObPtr^.GetType and gt_GroupClass) = gt_GroupClass then
          sn := PGroup(GrObPtr)^.Snap(Mi, Ma)
        else
          if PLine(GrObPtr)^.GetCut(@LocalLine, Dummy1, Dummy2) then
            sn := GrObPtr;
        i := i-1;
      end;
      LocalLine.Done;
      if sn=nil then
      begin
        LocalLine.Init(Ma.X, Mi.Y, Mi.X, Ma.Y);
        i := Count-1;
        while (i>=0) and (sn=nil) do
        begin
          GrObPtr := GetObject(i);
          if (GrObPtr^.GetType and gt_GroupClass) <> gt_GroupClass then
            if PLine(GrObPtr)^.GetCut(@LocalLine, Dummy1, Dummy2) then
              sn := GrObPtr;
          i := i-1;
        end;
        LocalLine.Done;
      end;
      ShowConv := OldShowConv;
    end;

    if sn<>nil then
    begin
      HitObjectPtr := sn;
      snap := @Self;
    end;
  end;
end;

procedure GroupClass.WriteOut;
 var
    i: Integer;
    s: String;

 function CMYKOUT(Col: LongInt): String;
  var
     r, g, b, k: Real;
 begin
   b := (Col shr 16) and $FF;
   g := (Col shr  8) and $FF;
   r := Col and $FF;
   r := r/255.0;
   g := g/255.0;
   b := b/255.0;
   k := max(r, max(g, b));
   CMYKOUT := NToS(k-r)+' '+NToS(k-g)+' '+NToS(k-b)+' '+NToS(1.0-k);
 end;

begin
  LastPoint.X := -1e10;
  LastPoint.Y := -1e10;
  if Count>1 then
    Ausgabe('u');
  for i := 0 to Count-1 do
    GetObject(i)^.WriteOut;
  if Count>1 then
  begin
    s := 's';
    if Closed then
    begin
      if FillColor<>-1 then
      begin
        Ausgabe(CMYKOUT(FillColor)+' k');
        s := 'f';
      end;
      if Color<>-1 then
      begin 
        Ausgabe(CMYKOUT(Color)+' K');
        if s='f' then
          s := 'b'
        else
          s := 's';
      end;
    end;
    Ausgabe(s);
    Ausgabe('U');
  end;
end;

function GroupClass.TestUnique(GrObPtr: PGraphicObject): Boolean;
 var
    i: Integer;
    Found: Boolean;
begin
  Found := False;
  for i := 0 to Count-1 do
    if not found then
      Found := GetObject(i)^.Compare(GrObPtr);
  TestUnique := not Found;
end;

procedure GroupClass.MakeLine;
 var
    i: Integer;
    GrObPtr: PGraphicObject;
    LinPtr: PLine;
begin
  for i := Count-1 downto 0 do
  begin
    GrObPtr := GetObject(i);
    if (GrObPtr^.GetType and gt_GroupClass) = gt_GroupClass then
      PGroup(GrObPtr)^.MakeLine
    else if (GrObPtr^.GetType = gt_BezierClass) then
    begin
      LinPtr := New(PLine, Init(GrObPtr^.P1.X, GrObPtr^.P1.Y, GrObPtr^.P4.X, GrObPtr^.P4.Y));
      LinPtr^.Color := GrObPtr^.Color;
      RemoveObject(GrObPtr);
      GObjects.AtInsert(i, LinPtr);
      LinPtr^.Group := @Self;
      Dispose(GrObPtr, Done);
    end;
  end;
  CalcFrame;
end;

procedure GroupClass.MakeBezier;
 var
    i: Integer;
    GrObPtr: PGraphicObject;
    BezPtr: PBezier;
begin
  for i := Count-1 downto 0 do
  begin
    GrObPtr := GetObject(i);
    if (GrObPtr^.GetType and gt_GroupClass) = gt_GroupClass then
      PGroup(GrObPtr)^.MakeBezier
    else if (GrObPtr^.GetType = gt_LineClass) then
    begin
      BezPtr := LineToBezier(PLine(GrObPtr));
      RemoveObject(GrObPtr);
      GObjects.AtInsert(i,BezPtr);
      BezPtr^.Group := @Self;
      Dispose(GrObPtr, Done);
    end;
  end;
  CalcFrame;
end;


{-- PUNKTINFOCLASS --}
type PPunktInfo = ^PunktInfoClass;
     PunktInfoClass = Object(TObject)
       P: Punkt;
       PInPtr,
       POutPtr: PPunktInfo;
       GrObIn,
       GrObOut: PGraphicObject;
       Constructor Init1(GrObPtr: PGraphicObject);
       Constructor Init2(GrObPtr: PGraphicObject);
       Function    Connect1(GrObPtr: PGraphicObject): Boolean;   virtual;
       Function    Connect2(GrObPtr: PGraphicObject): Boolean;   virtual;
       Procedure   ConnectTo(PInfPtr: PPunktInfo; GrObPtr: PGraphicObject); virtual;
     end;

Constructor PunktInfoClass.Init1(GrObPtr: PGraphicObject);
begin
  P := GrObPtr^.P1;
  GrObOut := GrObPtr;
  GrObIn := nil;
end;

Constructor PunktInfoClass.Init2(GrObPtr: PGraphicObject);
begin
  P := GrObPtr^.P4;
  GrObOut := GrObPtr;
  GrObIn := nil;
end;

Function PunktInfoClass.Connect1(GrObPtr: PGraphicObject): Boolean;
begin
  if (GrObPtr^.P1.X=P.X) and (GrObPtr^.P1.Y=P.Y) then
  begin
    GrObIn := GrObPtr;
    Connect1 := True;
  end
  else
    Connect1 := False;
end;

Function PunktInfoClass.Connect2(GrObPtr: PGraphicObject): Boolean;
begin
  if (GrObPtr^.P4.X=P.X) and (GrObPtr^.P4.Y=P.Y) then
  begin
    GrObIn := GrObPtr;
    Connect2 := True;
  end
  else
    Connect2 := False;
end;

Procedure PunktInfoClass.ConnectTo(PInfPtr: PPunktInfo; GrObPtr: PGraphicObject);
begin
  if GrObPtr = GrObIn then
    PInPtr := PInfPtr
  else
    POutPtr := PInfPtr;
  if GrObPtr = PInfPtr^.GrObIn then
    PInfPtr^.PInPtr := @Self
  else
    PInfPtr^.PoutPtr := @Self;
end;

{-- Ende PUNKTINFOCLASS --}



Function GroupClass.Sort: Boolean;
 var
    Anz, i, j: Integer;
    Open1, Open2: Boolean;
    GrObPtr: PGraphicObject;
    SubGroupPtr: PGroup;
    Pkte: TCollection;   {of PPunktInfo}
    PInfPtr, PInfPtr1, PInfPtr2: PPunktInfo;
    PT: Punkt;
begin
  Sort := False;
  Open1 := True;
  Closed := False;
  if Count<2 then
    exit;
  PT := GetObject(Count-1)^.P4;
  for i := 0 to Count-1 do
  begin
    GrObPtr := GetObject(i);
    if ((GrObPtr^.GetType and gt_LineClass) <> gt_LineClass) then
      exit;
    if Open1 then
    begin
      if (PT.X = GrObPtr^.P1.X) and (PT.Y = GrObPtr^.P1.Y) then
        PT := GrObPtr^.P4
      else
        Open1 := False;
    end;
  end;
  if Open1 then
  begin
    Closed := True;
    Sort := True;
    exit;
  end;
  Pkte.Init(2*Count,1);
  Anz := 0;
  for i := 0 to Count-1 do
  begin
    GrObPtr := GetObject(i);
    Open1 := True;
    Open2 := True;
    for j := Anz to Pkte.Count-1 do
    begin
      if Open1 or Open2 then
      begin
        PInfPtr := Pkte.At(j);
        if Open1 and PInfPtr^.Connect1(GrObPtr) then
        begin
          PInfPtr1 := PInfPtr;
          Pkte.AtDelete(j);
          Pkte.AtInsert(0,PInfPtr);
          Anz := Anz+1;
          Open1 := False;
        end
        else
        if Open2 and PInfPtr^.Connect2(GrObPtr) then
        begin
          PInfPtr2 := PInfPtr;
          Pkte.AtDelete(j);
          Pkte.AtInsert(0,PInfPtr);
          Anz := Anz+1;
          Open2 := False;
        end;
      end;
    end;
    if Open1 then
    begin
      PInfPtr1 := New(PPunktInfo, Init1(GrObPtr));
      Pkte.Insert(PInfPtr1);
    end;
    if Open2 then
    begin
      PInfPtr2 := New(PPunktInfo, Init2(GrObPtr));
      Pkte.Insert(PInfPtr2);
    end;
    PInfPtr1^.ConnectTo(PInfPtr2, GrObPtr);
  end;
  if Anz=Pkte.Count then
  begin
    Sort := True;
    while Pkte.Count>0 do
    begin
      New(SubGroupPtr, Init(Color, FillColor));
      i := 0;
      PT := PPunktInfo(Pkte.At(i))^.P;
      for j := 1 to Pkte.Count-1 do
      begin
        if PunktKleiner(PT, PPunktInfo(Pkte.At(j))^.P) then
        begin
          i := j;
          PT := PPunktInfo(Pkte.At(i))^.P;
        end;
      end;
      PInfPtr1 := Pkte.At(i);
      PInfPtr := PInfPtr1;
      if PunktKleiner(PInfPtr^.PInPtr^.P, PInfPtr^.POutPtr^.P) then
        GrObPtr := PInfPtr^.GrObIn
      else
        GrObPtr := PInfPtr^.GrObOut;

      repeat
        if (GrObPtr^.P1.X<>PInfPtr^.P.X) or (GrObPtr^.P1.Y<>PInfPtr^.P.Y) then
          PLine(GrObPtr)^.Reverse;
        SubGroupPtr^.AddObject(GrObPtr);
        RemoveObject(GrObPtr);
        PInfPtr2 := PInfPtr;
        if GrObPtr=PInfPtr^.GrObOut then
          PInfPtr := PInfPtr^.POutPtr
        else
          PInfPtr := PInfPtr^.PInPtr;
        if PInfPtr<>PInfPtr1 then
        begin
          if GrObPtr = PInfPtr^.GrObIn then
            GrObPtr := PInfPtr^.GrObOut
          else
            GrObPtr := PInfPtr^.GrObIn;
        end;
        Pkte.Free(PInfPtr2);
      until PInfPtr=PInfPtr1;
      SubGroupPtr^.CalcFrame;
      SubGroupPtr^.Sort;
      AddObject(SubGroupPtr);
    end;
    if Count=1 then
    begin
      SubGroupPtr := PGroup(GetObject(0));
      for i := 0 to SubGroupPtr^.Count do
        AddObject(SubGroupPtr^.GetObject(i));
      RemoveObject(SubGroupPtr);
      SubGroupPtr^.RemoveAll;
      Dispose(SubGroupPtr, Done);
      Closed := True;
    end;
  end;
  Pkte.Done;
end;

procedure GroupClass.Connect;
 var
    Anz, i, j: Integer;
    Open1, Open2: Boolean;
    GrObPtr: PGraphicObject;
    Pkte: TCollection;   {of PPunktInfo}
    PInfPtr, PInfPtr1, PInfPtr2: PPunktInfo;
    PT, PT2: Punkt;
    D, MinD: Real;
begin
  Closed := False;
  if Count<2 then
    exit;
  PT := GetObject(Count-1)^.P4;
  for i := 0 to Count-1 do
  begin
    GrObPtr := GetObject(i);
    if ((GrObPtr^.GetType and gt_LineClass) <> gt_LineClass) then
      exit;
    if Open1 then
    begin
      if (PT.X = GrObPtr^.P1.X) and (PT.Y = GrObPtr^.P1.Y) then
        PT := GrObPtr^.P4
      else
        Open1 := False;
    end;
  end;
  if Open1 then
  begin
    Closed := True;
    exit;
  end;
  Pkte.Init(2*Count,1);
  Anz := 0;
  for i := 0 to Count-1 do
  begin
    GrObPtr := GetObject(i);
    Open1 := True;
    Open2 := True;
    for j := Anz to Pkte.Count-1 do
    begin
      if Open1 or Open2 then
      begin
        PInfPtr := Pkte.At(j);
        if Open1 and PInfPtr^.Connect1(GrObPtr) then
        begin
          PInfPtr1 := PInfPtr;
          Pkte.AtDelete(j);
          Pkte.AtInsert(0,PInfPtr);
          Anz := Anz+1;
          Open1 := False;
        end
        else
        if Open2 and PInfPtr^.Connect2(GrObPtr) then
        begin
          PInfPtr2 := PInfPtr;
          Pkte.AtDelete(j);
          Pkte.AtInsert(0,PInfPtr);
          Anz := Anz+1;
          Open2 := False;
        end;
      end;
    end;
    if Open1 then
    begin
      PInfPtr1 := New(PPunktInfo, Init1(GrObPtr));
      Pkte.Insert(PInfPtr1);
    end;
    if Open2 then
    begin
      PInfPtr2 := New(PPunktInfo, Init2(GrObPtr));
      Pkte.Insert(PInfPtr2);
    end;
    PInfPtr1^.ConnectTo(PInfPtr2, GrObPtr);
  end;

  while Anz<Pkte.Count do
  begin
    PInfPtr1 := Pkte.At(Anz);
    PT := PInfPtr1^.P;
    j := Anz+1;
    MinD := 9999999;
    for i := Anz+1 to Pkte.Count-1 do
    begin
      PInfPtr2 := Pkte.At(i);
      PT2 := PInfPtr2^.P;
      d := abs(PT2.X-PT.X)+abs(PT2.Y-PT.Y);
      if (d<MinD) and (PInfPtr2^.PInPtr<>PInfPtr1) then
      begin
        MinD := d;
        j := i;
      end;
    end;
    PInfPtr2 := Pkte.At(j);
    PInfPtr1^.GrObIn := PInfPtr2^.GrObOut;
    if (PInfPtr1^.GrObIn^.P1.X=PInfPtr2^.P.X) and
       (PInfPtr1^.GrObIn^.P1.Y=PInfPtr2^.P.Y) then
      PInfPtr1^.GrObIn^.P1 := PInfPtr1^.P
    else
      PInfPtr1^.GrObIn^.P4 := PInfPtr1^.P;
    PInfPtr := PInfPtr2^.POutPtr;
    PInfPtr1^.PInPtr := PInfPtr;
    if PInfPtr^.PInPtr = PInfPtr2 then
      PInfPtr^.PInPtr := PInfPtr1
    else
      PInfPtr^.POutPtr := PInfPtr1;
    Anz := Anz+1;
    Pkte.Free(PInfPtr2);
  end;


  if Anz=Pkte.Count then
  begin
    while Pkte.Count>0 do
    begin
      i := 0;
      PT := PPunktInfo(Pkte.At(i))^.P;
      for j := 1 to Pkte.Count-1 do
      begin
        if PunktKleiner(PT, PPunktInfo(Pkte.At(j))^.P) then
        begin
          i := j;
          PT := PPunktInfo(Pkte.At(i))^.P;
        end;
      end;
      PInfPtr1 := Pkte.At(i);
      PInfPtr := PInfPtr1;
      if PunktKleiner(PInfPtr^.PInPtr^.P, PInfPtr^.POutPtr^.P) then
        GrObPtr := PInfPtr^.GrObIn
      else
        GrObPtr := PInfPtr^.GrObOut;

      repeat
        if (GrObPtr^.P1.X<>PInfPtr^.P.X) or (GrObPtr^.P1.Y<>PInfPtr^.P.Y) then
          PLine(GrObPtr)^.Reverse;
        GObjects.Delete(GrObPtr);
        GObjects.Insert(GrObPtr);
        PInfPtr2 := PInfPtr;
        if GrObPtr=PInfPtr^.GrObOut then
          PInfPtr := PInfPtr^.POutPtr
        else
          PInfPtr := PInfPtr^.PInPtr;
        if PInfPtr<>PInfPtr1 then
        begin
          if GrObPtr = PInfPtr^.GrObIn then
            GrObPtr := PInfPtr^.GrObOut
          else
            GrObPtr := PInfPtr^.GrObIn;
        end;
        Pkte.Free(PInfPtr2);
      until PInfPtr=PInfPtr1;
    end;
    Closed := True;
  end;
  Pkte.Done;
end;

{

procedure GroupClass.Sort;
 var
    i, j: Integer;
    PS1, PS2: Punkt;
    GrObPtr: PGraphicObject;
begin
  Closed := False;
  for i := 0 to Count-1 do
    if ((GetObject(i)^.GetType and gt_LineClass) <> gt_LineClass) then
      exit;
  i := 0;
  GrObPtr := GetObject(0);
  PS1 := GrObPtr^.PMin;
  PS2 := GrObPtr^.PMax;
  for j:= 1 to Count - 1 do
  begin
    GrObPtr := GetObject(j);
    if  (GrObPtr^.PMin.Y<PS1.Y) or
       ((GrObPtr^.PMin.Y=PS1.Y) and (GrObPtr^.PMin.X<PS1.X)) or
       ((GrObPtr^.PMin.Y=PS1.Y) and (GrObPtr^.PMin.X=PS1.X) and (GrObPtr^.PMax.Y<PS2.Y)) or
       ((GrObPtr^.PMin.Y=PS1.Y) and (GrObPtr^.PMin.X=PS1.X) and (GrObPtr^.PMax.Y=PS2.Y) and (GrObPtr^.PMax.X<PS2.X)) then
    begin
      i := j;
      PS1 := GrObPtr^.PMin;
      PS2 := GrObPtr^.PMax;
    end;
  end;

  GrObPtr := GetObject(i);
  PS1:=GrObPtr^.P1;
  PS2:=GrObPtr^.P4;
  if (PS1.Y>PS2.Y) or ((PS1.Y=PS2.Y) and (PS1.X>PS2.X)) then
  begin
    PLine(GrObPtr)^.Reverse;
    PS1 := GrObPtr^.P1;
    PS2 := GrObPtr^.P4;
  end;

  RemoveObject(GrObPtr);
  AddObject(GrObPtr);

  i := Count-2;
  while i>=0 do
  begin
    j := i;
    while j>=0 do
    begin
      GrObPtr := GetObject(j);
      if (PS2.X = GrObPtr^.P1.X) and (PS2.Y = GrObPtr^.P1.Y) then
      begin
        PS2 := GrObPtr^.P4;
        j := -1;
      end
      else if (PS2.X = GrObPtr^.P4.X) and (PS2.Y = GrObPtr^.P4.Y) then
      begin
        PLine(GrObPtr)^.Reverse;
        PS2 := GrObPtr^.P4;
        j := -1;
      end;
      j := j-1;
    end;
    if j<>-2 then
      i := -2
    else
    begin
      RemoveObject(GrObPtr);
      AddObject(GrObPtr);
    end;
    i := i-1;
  end;
  if i<>-2 then
    if (PS1.X = PS2.X) and (PS1.Y = PS2.Y) then
    begin
      Closed := True;
    end;
end;
}

destructor GroupClass.Done;
begin
  if PolygonPtr <> nil then
    Dispose(PolygonPtr, Done);
  GObjects.Done;
  GraphicObject.Done;
end;                                            



{---------------------------------------------------------}
{-------------------- EllipseClass  ----------------------}
{---------------------------------------------------------}


constructor EllipseClass.Init(X1, Y1, X2, Y2: Real; Col, FCol: LongInt);
 var
    XM, YM, DX, DY: Real;
    BezPtr: PBezier;
begin
  GroupClass.Init(Col, FCol);
  if X1>X2 then
  begin
    XM := X1;
    X1 := X2;
    X2 := XM;
  end;
  if Y1>Y2 then
  begin
    YM := Y1;
    Y1 := Y2;
    Y2 := YM;
  end;

  XM := (X2+X1) / 2;
  YM := (Y2+Y1) / 2;
  DX := (X2-X1) / 2 * 0.552280842264;
  DY := (Y2-Y1) / 2 * 0.552280842264;
  BezPtr := new(PBezier, Init( XM,    Y1,
                               XM+DX, Y1,
                               X2,    YM-DY,
                               X2,    YM));
  BezPtr^.Color := Color;                        
  AddObject(BezPtr);
  BezPtr := new(PBezier, Init( X2,    YM,
                               X2,    YM+DY,
                               XM+DX, Y2,
                               XM,    Y2));
  BezPtr^.Color := Color;
  AddObject(BezPtr);
  BezPtr := new(PBezier, Init( XM,    Y2,
                               XM-DX, Y2,
                               X1,    YM+DY,
                               X1,    YM));
  BezPtr^.Color := Color;
  AddObject(BezPtr);
  BezPtr := new(PBezier, Init( X1,    YM,
                               X1,    YM-DY,
                               XM-DX, Y1,
                               XM,    Y1));
  BezPtr^.Color := Color;
  AddObject(BezPtr);
  CalcFrame;
  IsEllipse := True;
  Closed := True;
end;

function EllipseClass.CreateCopy: PGraphicObject;
 var
    ElPtr: PEllipse;
begin
  ElPtr := new(PEllipse, Init(PMin.X, PMin.Y, PMax.X, PMax.Y, Color, FillColor));
  CreateCopy := ElPtr;
end;

function EllipseClass.GetType: Word;
begin
  GetType := gt_EllipseClass;
end;

procedure EllipseClass.ShowFrame;
begin
  if ((frameMode and fm_Bezier) = 0) or not IsEllipse then
    GroupClass.ShowFrame;
end;

function EllipseClass.HitsFrameControlPoints(X, Y: Integer): Integer;
begin
  if ((frameMode and fm_Bezier) = 0) or not IsEllipse then
    HitsFrameControlPoints := GroupClass.HitsFrameControlPoints(X, Y)
  else
    HitsFrameControlPoints := -1;
end;

procedure EllipseClass.Rot(NX1, NY1, NX2, NY2, MX, MY: Real);
begin
  if IsEllipse and ((Int(NX1)<>NX1) or (Int(NY1)<>NY1) or
                    (Int(NX2)<>NX2) or (Int(NY2)<>NY2)) then
    IsEllipse := False;
  GroupClass.Rot(NX1, NY1, NX2, NY2, MX, MY);
end;

procedure EllipseClass.AddObject(GrObPtr: PGraphicObject);
begin
  IsEllipse := False;
  GroupClass.AddObject(GrObPtr);
end;

procedure EllipseClass.RemoveObject(GrObPtr: PGraphicObject);
begin
  IsEllipse := False;
  GroupClass.RemoveObject(GrObPtr);
end;

procedure EllipseClass.RemoveAll;
begin
  IsEllipse := False;
  GroupClass.RemoveAll;
end;


procedure EllipseClass.Draw;
 var
    SaveColor: ColorData;
begin
  if IsEllipse then
  begin
    SetColor(SaveColor, vhDC, Color, FillColor);
    GrEllipse(PMin.X, PMin.Y, PMax.X, PMax.Y);
    ResetColor(SaveColor);
  end
  else
    GroupClass.Draw;
end;


function EllipseClass.Snap(Mi, Ma: Punkt): PGraphicObject;
 var
    X, Y: Integer;
    r: TRect;
    TempRgn: HRgn;
begin
  if not IsEllipse then
    Snap := GroupClass.Snap(Mi,Ma)
  else
  begin
    SetRect(r, zx(Mi.X), zy(Ma.Y), zx(Ma.X), zy(Mi.Y));
    TempRgn := CreateEllipticRgn(zx(PMin.X), zy(PMax.Y), zx(PMax.X), zy(PMin.Y));
    if RectInRegion(TempRgn, r) then
    begin
      HitObjectPtr := @Self;
      Snap := @Self;
    end
    else
      Snap := nil;
    DeleteObject(TempRgn);
  end;
end;

{---------------------------------------------------------}
{-------------------- MULTIGROUPCLASS --------------------}
{---------------------------------------------------------}


constructor MultiGroupClass.Init(Col, FCol: LongInt);
begin
  GroupClass.Init(Col, FCol);
  Closed := True;
end;

function MultiGroupClass.CreateCopy: PGraphicObject;                      
 var
    MGroupPtr: PMultiGroup;
    i: Integer;
begin
  new(MGroupPtr, Init(Color, FillColor));
  for i := 0 to Count-1 do
    MGroupPtr^.AddObject(GetObject(i)^.CreateCopy);
  MGroupPtr^.Closed := Closed;
  MGroupPtr^.PMin := PMin;
  MGroupPtr^.PMax := PMax;
  CreateCopy := MGroupPtr;
end;

function MultiGroupClass.GetType: Word;
begin
  GetType := gt_MultiGroupClass;
end;
                                   

procedure MultiGroupClass.Draw;                                            
 var
    i: Integer;
    GrObPtr: PGraphicObject;
    AnzPtr: PPolyAnz;
    Anz: Word;
    SaveColor: ColorData;
begin
  if Cuts(VisMin, VisMax) then
  begin
    i := 0;

    if (PolygonPtr=nil) or (PolygonPtr^.MaxZoom<ZoomFaktor) then
    begin
      if PolygonPtr<>nil then
        Dispose(PolygonPtr, Done);

      PolygonPtr := New(PPolyPolygon, Init(Count));
      Anz := 0;
      for i := 0 to Count-1 do
      begin
        PGroup(GetObject(i))^.GetPunkte(PolygonPtr);
        PPolyPolygon(PolygonPtr)^.NewPoly;
      end;
    end;

    Anz := PPolyPolygon(PolygonPtr)^.GetPolyPoints(PolyPtr, AnzPtr);

    if Anz>0 then
    begin
      SetColor(SaveColor, vhDC, Color, FillColor);
      PolyPolygon(vhDC,PolyPtr^,AnzPtr^,Anz);
      ResetColor(SaveColor);
    end;
  end;
end;


function MultiGroupClass.Snap(Mi, Ma: Punkt): PGraphicObject;             
 var
    r: TRect;
    TempRgn: HRgn;
begin
  Snap := nil;
  if (PolygonPtr<>nil) then
  begin
    SetRect(r, zx(Mi.X), zy(Ma.Y), zx(Ma.X), zy(Mi.Y));
    TempRgn := PolygonPtr^.GetRgn;
    if RectInRegion(TempRgn, r) then
    begin
      HitObjectPtr := @Self;
      Snap := @Self;
    end;
    DeleteObject(TempRgn);
  end;
end;


end.