unit Bezier2;

{ $N+}


interface

uses {WObjects}Objects, WinTypes, WinProcs, WinDos,
     Defs, Polygone, BezDefs, Bezier1, Script;

const
    PersonName = 'Ferenc Kurucz                           ';
    FirmaName  = 'Graphiq                                 ';

var
    AIDIR,
    CDRDIR,
    PASNEWNAME: String;



type

{    PGraphicClass = ^GraphicClass;}
    PSon = ^SonClass;
    PNode = ^NodeClass;
    PSelection = ^SelectionClass;

    SonClass = Object(TObject)
        NodePtr: PNode;
        Ang, Ang2: Real;
        IsStart, Marked: Boolean;
        LinePtr: PLine;
        constructor Init(NodPtr: PNode; Start: Boolean; LinPtr: PLine);
    end;

    NodeClass = Object(TObject)
        P: Punkt;
        Sons: TCollection; { of PSon }
        constructor Init(P1: Punkt);
        procedure   AddSon(SonPtr: PSon);                             virtual;
        function    NextSon(Ang: Real): PSon;                         virtual;
        destructor  Done;                                             virtual;
    end;



    SelectionClass = Object(GroupClass)
        constructor Init;
        function    GetType: Word;                                   virtual;
        procedure   GetFrame(var X1, Y1, X2, Y2: Real);              virtual;
        procedure   Adjust(AdMode: Word);                            virtual;
    end;

    WorldClass = Object(SelectionClass)
        Temp,
        Pool,
        Copy:     SelectionClass;
        Script: ScriptClass;
        CurrentState,
        State2,
        State3: LongInt;
        constructor Init;
        function    GetType: Word;                                   virtual;
        procedure   AddLine(X1, Y1, X2, Y2: Real; Col: Longint);     virtual;
        procedure   AddLineExpanded(X1, Y1, X2, Y2: Real; Col: Longint);        virtual;
        procedure   AddBez(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Real; Col: Longint); virtual;
        procedure   AddEllipse(X1, Y1, X2, Y2: Real; Col, FCol: Longint);       virtual;
        function    GetSnap(X1, Y1, X2, Y2: Real): Integer;          virtual;
        procedure   SnapAll(X1, Y1, X2, Y2: Real);                   virtual;
        procedure   ToggleHitObject;                                 virtual;
        procedure   Rotate(Alpha, MX, MY: Real);                     virtual;
        procedure   HMirror(MX: Real);                               virtual;
        procedure   VMirror(MY: Real);                               virtual;
        procedure   GroupSelection(Col, FCol: LongInt);              virtual;
        procedure   GroupSelected(Col, FCol: LongInt);               virtual;
        procedure   ConnectSelected(Col, FCol: LongInt);               virtual;
        procedure   UngroupSelected;                                 virtual;
        procedure   DrawAll;                                         virtual;
        procedure   Clear(X1, Y1, X2, Y2: Real);                     virtual;
        procedure   Redraw(X1, Y1, X2, Y2: Real);                    virtual;
        procedure   RedrawSelected(X1, Y1, X2, Y2: Real);            virtual;
        procedure   SelectAll;                                       virtual;
        procedure   ClearSelected;                                   virtual;
        procedure   DeleteSelection;                                 virtual;
        procedure   DeleteSelected;                                  virtual;
        procedure   SetSelectedColor(Col, FCol: LongInt);            virtual;
        procedure   GetSelectedFrame(var X1d, Y1d, X2d, Y2d: Integer); virtual;
        procedure   CopySelected;                                    virtual;
        procedure   PasteSelected;                                   virtual;
        procedure   AlignSelected;                                           virtual;
        procedure   MoveSelected(dX, dY: Real);                      virtual;
        procedure   StretchSelected(XOld, YOld, XNew, YNew, XS, YS: Real); virtual;
        procedure   Rot(NX1, NY1, NX2, NY2, MX, MY: Real);           virtual;
        procedure   MakeLine;                                        virtual;
        procedure   MakeBezier;                                      virtual;
        procedure   SetSelectedPosition(Pos: Word);                  virtual;
        procedure   WriteOut;                                        virtual;
        procedure   ReplaySelected;                                  virtual;
        procedure   Replay;                                          virtual;
        procedure   IterateStep(n, FirstCount, LastCount: Integer; SelfIterate: Boolean); virtual;
        procedure   Iterate(n: Integer; SelfIterate: Boolean);       virtual;
        procedure   Blend(Steps: Integer);                           virtual;
        procedure   GetFirstSelectedLinePoints(var X1, Y1, X2, Y2: Real); virtual;
        destructor  Done;                                            virtual;
    end;

    ConvertClass = Object(WorldClass)
        Punkte: TCollection;  { of PNode }
        constructor Init;
        function    GetType: Word;                                   virtual;
        function    AddTest(GrObPtr: PGraphicObject): Boolean;                virtual;
        procedure   Flatten;                                         virtual;
        procedure   Split;                                           virtual;
        function    FindNode(P: Punkt): PNode;                       virtual;
        procedure   MakeGraph;                                       virtual;
        function    TestPoly(NodePtr: PNode; SonPtr: PSon): Boolean; virtual;
        procedure   ShowPoly(NodePtr: PNode; SonPtr: PSon);          virtual;
        procedure   GetPolys(NodePtr: PNode);                        virtual;
        destructor  Done;                                            virtual;
    end;



    PInteger = ^Integer;

var
{    LineWorld: GraphicClass; }
    World: ConvertClass;
    GridInfo: TGridInfo;
    MaxGroup: Word;


procedure ShowAI(AiFileName: String);
procedure Main;
procedure BreakMain;
procedure Skip(m: Integer);

procedure SaveLines(FileName: String);
procedure SaveAreas(FileName: String);

function GetState(Nr: Integer): LongInt;

procedure ResetZoom;
procedure SetZoom(X1, Y1, X2, Y2: Real);

procedure ResetClip;
procedure SetClip(X1, Y1, X2, Y2: Real);

procedure ShowProgress;
procedure HideProgress;

procedure GetSize(BufferPtr: PSizeInfo);
procedure SetSize(BufferPtr: PSizeInfo);
procedure GetMaxSize(BufferPtr: PSizeInfo);

function InvX(X: Real): Real;
function InvY(Y: Real): Real;
function SnapX(X: Real): Real;
function SnapY(Y: Real): Real;

procedure ShowGrid;
procedure HideGrid;

procedure UnitInit(vWindow: HWnd);


implementation

uses Poly,    { Unit zur Handhabung von Polynomen bis Grad 3 }
     GrTools, { Unit zum malen invertierter Linien }
     Strings;




{---------------------------------------------------------}
{-------------------- Hilfsfunktionen --------------------}
{---------------------------------------------------------}



function InvX(X: Real): Real;
begin
  InvX := X/ZoomFaktor+ZoomX1;
end;

function InvY(Y: Real): Real;
begin
  InvY :=ZoomY1-Y/ZoomFaktor;
end;

function SnapX(X: Real): Real;
begin
  X := InvX(X);
  if GridInfo.SnapX then
    X := round(X/GridInfo.GridX)*GridInfo.GridX;
  SnapX := X;
end;

function SnapY(Y: Real): Real;
begin
  Y := InvY(Y);
  if GridInfo.SnapY then
    Y := round(Y/GridInfo.GridY)*GridInfo.GridY;
  SnapY := Y;
end;

procedure ResetClip;
begin
  if ClipX1<>-99999 then
      GrRectangle(ClipX1, ClipY1, ClipX2, ClipY2,
                  GetStockObject(White_Brush));

  ClipX1 := -99999;
  ClipY1 := -99999;
  ClipX2 := 99999;
  ClipY2 := 99999;
end;

procedure SetClip(X1, Y1, X2, Y2: Real);
begin
  if ClipX1<>-99999 then
      GrRectangle(ClipX1, ClipY1, ClipX2, ClipY2,
                  GetStockObject(White_Brush));

  ClipX1 := Min(InvX(X1),InvX(X2));
  ClipY1 := Min(InvY(Y1),InvY(Y2));
  ClipX2 := Max(InvX(X1),InvX(X2));
  ClipY2 := Max(InvY(Y1),InvY(Y2));
  GrRectangle(ClipX1, ClipY1, ClipX2, ClipY2,
              GetStockObject(LtGray_Brush));
end;

procedure SetVis;
begin
  VisMin.X := InvX(-1);
  VisMin.Y := InvY(yw+1);
  VisMax.X := InvX(xw+1);
  VisMax.Y := InvY(-1);
end;

procedure ResetZoom;
 var
    ZF, ZX1, ZY1: Real;
    Buffer: TSizeInfo;
begin
  GetMaxSize(@Buffer);

  ZF := (xw-10)/(Buffer.X2-Buffer.X1);
  ZF := min(ZF,(yw-10)/(Buffer.Y2-Buffer.Y1));
  ZX1 := Buffer.X1-5*(Buffer.X2-Buffer.X1)/(xw-10);
  ZY1 := Buffer.Y2+5*(Buffer.Y2-Buffer.Y1)/(yw-10);

  ZoomFaktor := ZF;
  ZoomX1 := ZX1;
  ZoomY1 := ZY1;

  SetVis;
end;

procedure SetZoom(X1, Y1, X2, Y2: Real);
begin
  if (X1<>X2) and (Y1<>Y2) then
  begin
    X1 := InvX(X1);
    Y1 := InvY(Y1);
    X2 := InvX(X2);
    Y2 := InvY(Y2);
    ZoomX1 := X1;
    ZoomY1 := Y1;
    ZoomFaktor := 1.0*xw/(X2-X1);
    if ZoomFaktor>1.0*yw/(Y1-Y2) then
      ZoomFaktor := 1.0*yw/(Y1-Y2);

    SetVis;
  end;
end;


procedure GetSize(BufferPtr: PSizeInfo);
begin
  BufferPtr^.X1 := MinX;
  BufferPtr^.Y1 := MinY;
  BufferPtr^.X2 := MaxX-MinX;
  BufferPtr^.Y2 := MaxY-MinY;
end;

procedure SetSize(BufferPtr: PSizeInfo);
begin
  if (BufferPtr^.X2>0) and (BufferPtr^.Y2>0) then
  begin
    MinX := BufferPtr^.X1;
    MinY := BufferPtr^.Y1;
    MaxX := MinX+BufferPtr^.X2;
    MaxY := MinY+BufferPtr^.Y2;
  end;
end;

procedure GetMaxSize(BufferPtr: PSizeInfo);
begin
  with BufferPtr^ do
  begin
    X1 := MinX;
    Y1 := MinY;
    X2 := MaxX-MinX;
    Y2 := MaxY-MinY;
    if World.Pool.Count>0 then
    begin
      X1 := min(X1, World.Pool.PMin.X);
      Y1 := min(Y1, World.Pool.PMin.Y);
      X2 := max(X2, World.Pool.PMax.X);
      Y2 := max(Y2, World.Pool.PMax.Y);
    end;
  end;
end;


procedure PaintGrid(Color: LongInt);
 var
    dx, dy, gx, gy, X1, X, Y: Real;

begin
  dx := VisMax.X-VisMin.X;
  dy := Vismax.Y-VisMin.Y;
  gx := 1.0/GridInfo.GridX;
  while (dx*gx)>30 do
    gx := gx / 2.0;
  gy := 1.0/GridInfo.GridY;
  while (dy*gy)>20 do
    gy := gy / 2.0;
  gx := 1.0/gx;
  gy := 1.0/gy;
  X1 := int(VisMin.X/gx)*gx;
  Y := int(VisMin.Y/gy)*gy;
  while (Y<=VisMax.Y) do
  begin
    X := X1;
    while (X<=VisMax.X) do
    begin
      SetPixel(vhDC, zx(X), zy(Y), Color);
      X := X+gx;
    end;
    Y := Y+gy;
  end;
end;

procedure ShowGrid;
begin
  PaintGrid(RGB(0,0,255));
end;

procedure HideGrid;
begin
  PaintGrid(RGB(255,255,255));
end;







{---------------------------------------------------------}
{-------------------- Daten Einlesen ---------------------}
{---------------------------------------------------------}
{

nC nM nY nK   k            -   Fllfarbe als CMYK-Wert
NC nM nY nK   K            -   Rahmenfarbe als CMYK-Wert

b                          -   Object gefllt mit Rahmen
n1 n2 n3 n4 n5 n6 c        -   Bezierkurve nach (n5,n6)
n1 n2 n3 n4 n5 n6 C        -   Bezierkurve nach (n5,n6)
f                          -   Object gefllt ohne Rahmen
nX nY L                    -   Linie nach (X,Y)
nX nY m                    -   Startpunkt des Objects bei (X,Y)
s                          -   Object durchsichtig mit Rahmen und Verbindung der Endpunkte
S                          -   Object druchsichtig mit Rahmen
u                          -   Anfang Gruppierung (auch geschachtelt mglich)
U                          -   Ende Gruppierung

}


procedure WriteHead(FileName: String);
 var
    Day, Month, Year, Hour, Minute, Dummy: Word;
    APm: Char;
    FileOnlyName: String;
begin
  FileOnlyName := FileName;
  while pos('\',FileOnlyName)<>0 do
    FileOnlyName := copy(FileOnlyName,pos('\',FileOnlyName)+1,length(FileOnlyName));
  GetDate(Year, Month, Day, Dummy);
  GetTime(Hour, Minute, Dummy, Dummy);
  APm := 'A';
  if (Hour=0) or (Hour>=13) then
  begin
    Hour := abs(Hour-12);
    APm := 'P';
  end;
  WriteLn(OutFile,'%!PS-Adobe-3.0 EPSF-3.0');
  WriteLn(OutFile,'%%Creator: Adobe Illustrator (TM) 3.0.1 exported from AI2CDR v0.5b');
  WriteLn(OutFile,'%%FOR: (',PersonName,') (',FirmaName,')');
  WriteLn(OutFile,'%%Title: (',FileOnlyName,')');
  WriteLn(OutFile,'%%CreationDate: (',Day,'/',Month,'/',Year,') (',Hour,':',Minute,' ',APm,'M)');
  WriteLn(OutFile,'%%BoundingBox: ',MinX:7:3,' ',MinY:7:3,' ',MaxX:7:3,' ',MaxY:7:3);
  WriteLn(OutFile,'%%DocumentProcessColors: Black');
  WriteLn(OutFile,'%%BeginSetup');
  WriteLn(OutFile,'%%EndSetup');
  Ausgabe('u');
  Ausgabe('0.000 0.000 0.000 1.000 K');
  Ausgabe('1.000 0.000 0.000 0.000 k');
end;




procedure WriteTail;
 var
    S: String;
begin
    Ausgabe('U');
    WriteLn(OutFile,'%%PageTrailer');
    WriteLn(OutFile,'%%EOF');
end;



procedure GetToken(var S1, S: String);
 var i,j,k: Integer;
begin
  i := 1;
  j := length(S);
  while (i<=j) and (s[i]=' ') do
    i := i+1;
  k := i+1;
  while (k<=j) and (s[k]<>' ') do
    k := k+1;
  S1 := copy(S,i,k-i);
  S := copy(S,k,j);
end;

procedure ReadFile(AiFileName: String);
 var
    S, S1: String;
    Command: Char;
    Anz: Integer;
    Dummy: Word;
    LastX, LastY: Real;
    i: Integer;
    Numbers: array[1..80] of Real;
    Datei: file of Byte;
    Color, FillColor: LongInt;
begin

    Assign(Datei, AiFileName);
    Reset(Datei);

    World.State2 := 0;
    World.State3 := fileSize(Datei);
    World.CurrentState := 5;

    close(Datei);

    Assign(AiFile, AiFileName);
    Reset(AiFile);

    MinX := 99999;
    MaxX := -99999;
    MinY := 99999;
    MaxY := -99999;
    Color := rgb(0,0,0);
    FillColor := rgb(255,255,255);
    repeat
        ReadLn(AiFile, S);
        if copy(S,1,14)='%%BoundingBox:' then
        begin
          GetToken(S1, S);
          GetToken(S1, S);
          val(S1,MinX,Dummy);
          GetToken(S1, S);
          val(S1,MinY,Dummy);
          GetToken(S1, S);
          val(S1,MaxX,Dummy);
          GetToken(S1, S);
          val(S1,MaxY,Dummy);
        end;
        World.State2 := World.State2 + Length(S)+1;
    until (S='%%EndSetup') or BreakConv or eof(AiFile);

    if eof(AiFile) then
      BreakMain;

    ReadLn(AiFile, S);
    World.State2 := World.State2 + Length(S)+2;


    repeat

        GiveWindowTime;

        Command := S[Length(S)];

        if Length(S)>=2 then
          if (S[Length(S)-1]<>' ') or (S[1]='%') then
            Command := '?';
        Anz := 0;
        While (S<>'') and (S[1] = ' ') do
            S := copy(S,2,length(S)-1);
        While S[1] in ['-','0'..'9'] do
        begin
            Anz := Anz+1;
            val(copy(S,1,pos(' ',S)-1), Numbers[Anz], Dummy);
            S := copy(S, pos(' ',S+' ')+1,length(s));
            While (S<>'') and (S[1] = ' ') do
                S := copy(S,2,length(S)-1);
        end;

        case Command of
      'c','C' : begin
                    World.AddBez( LastX, LastY,
                                  Numbers[1], Numbers[2],
                                  Numbers[3], Numbers[4],
                                  Numbers[5], Numbers[6],
                                  Color);
                    LastX := Numbers[5];
                    LastY := Numbers[6];
                    MinX := min(MinX,LastX);
                    MaxX := max(MaxX,LastX);
                    MinY := min(MinY,LastY);
                    MaxY := max(MaxY,LastY);
                end;
      'f','F' : begin
                  World.GroupSelection(-1, FillColor);
                  World.ClearSelected;
                end;
      's','S' : begin
                  World.GroupSelection(Color, -1);
                  World.ClearSelected;   
                end;
      'b','B' : begin
                  World.GroupSelection(Color, FillColor);
                  World.ClearSelected;   
                end;
      'k'     : FillColor := cmyk(Numbers[1], Numbers[2], Numbers[3], Numbers[4]);
      'K'     : Color := cmyk(Numbers[1], Numbers[2], Numbers[3], Numbers[4]);
      'l','L' : begin
                    World.AddLineExpanded( LastX, LastY,
                                           Numbers[1], Numbers[2],
                                           Color);
                    LastX := Numbers[1];
                    LastY := Numbers[2];
                    MinX := min(MinX,LastX);
                    MaxX := max(MaxX,LastX);
                    MinY := min(MinY,LastY);
                    MaxY := max(MaxY,LastY);
                end;
      'm','M' : begin
                    LastX := Numbers[1];
                    LastY := Numbers[2];
                    MinX := min(MinX,LastX);
                    MaxX := max(MaxX,LastX);
                    MinY := min(MinY,LastY);
                    MaxY := max(MaxY,LastY);

                end;
          'v' : begin
                    World.AddBez( LastX, LastY,
                                  Numbers[1], Numbers[2],
                                  Numbers[1], Numbers[2],
                                  Numbers[3], Numbers[4],
                                  Color);
                    LastX := Numbers[3];
                    LastY := Numbers[4];
                    MinX := min(MinX,LastX);
                    MaxX := max(MaxX,LastX);
                    MinY := min(MinY,LastY);
                    MaxY := max(MaxY,LastY);
                    Anz :=Anz;
                end;
        end;
        ReadLn(AiFile, S);
        World.State2 := World.State2 + Length(S)+2;
    until (S='%%PageTrailer') or eof(AiFile) or BreakConv;

    Close(AiFile);

    World.State2 := World.State3;

    MinX := Max(ClipX1, MinX);
    MinY := Max(ClipY1, MinY);
    MaxX := Min(ClipX2, MaxX);
    MaxY := Min(ClipY2, MaxY);

{    VisMin.X := MinX;
    VisMin.Y := MinY;
    VisMax.X := MaxX;
    VisMax.Y := MaxY;}

    World.CalcFrame;
    World.Pool.CalcFrame;

    ResetZoom;

    SendMessage(TheWindow, wm_Command, cm_SetScrollbars, 0)

end;

procedure DeletePasNew;
 var
    EraseFile: Text;
begin
  if PasNewName <> '' then
  begin
    assign(EraseFile, PASNEWNAME);
    {$I-}
    Reset(EraseFile);
    {$I+}
    if IOResult = 0 then
    begin
      Close(EraseFile);
      erase(EraseFile);
    end;
  end;
  PASNEWNAME := '';
end;

procedure SaveAreas(FileName: String);
 var
    s: String;
begin
  assign(AiFile, PASNEWNAME);
  {$I-}
  reset(AiFile);
  {$I+}
  if IOResult<>0 then
  begin
    MessageBox(0,'Ergebnis der Konvertierung konnte nicht mehr gefunden werden!','AI -> CDR', mb_OK+mb_iconStop);
    exit;
  end;

  assign(OutFile, FileName);
  rewrite(OutFile);

  WriteHead(FileName);

  while not eof(AiFile) do
  begin
    readln(AiFile,s);
    writeln(OutFile,s);
  end;

  WriteTail;

  close(AiFile);
  close(OutFile);
  DeletePasNew;
end;

procedure SaveLines(FileName: String);
 var
    i: Integer;
begin
  assign(OutFile,FileName);
  {$I-}
  rewrite(OutFile);
  {$I+}
  if IOResult<>0 then
    MessageBox(0,'Fehler beim anlegen der Datei','AI2 -> CDR', mb_ok+mb_iconStop)
  else
  begin
    WriteHead(FileName);
    World.WriteOut;
{    for i := 0 to LineWorld.Linien.Count-1 do
    begin
      AusgabeM(LineWorld.Linien.At(i), True);
      AusgabeC(LineWorld.Linien.At(i), True);
      Aus('S'+chr(13)+chr(10));
    end;
}
    WriteTail;
    close(OutFile);
  end;
end;

{---------------------------------------------------------}
{-------------------- SonClass      ----------------------}
{---------------------------------------------------------}

Constructor SonClass.Init(NodPtr: PNode; Start: Boolean; LinPtr: PLine);
begin
    NodePtr := NodPtr;
    LinePtr := LinPtr;
    if Start then
    begin
        IsStart := True;

{ Gerade Verbindung 
        Ang  := atan2(LinPtr^.P4.X-LinPtr^.P1.X,LinPtr^.P4.Y-LinPtr^.P1.Y);
        Ang2 := atan2(LinPtr^.P1.X-LinPtr^.P4.X,LinPtr^.P1.Y-LinPtr^.P4.Y);
}
{ Anfangssteigung statt gerade Verbindung } 
        Ang := atan2(LinPtr^.dQx(0.01),LinPtr^.dQy(0.01));
        Ang2 := atan2(LinPtr^.dQx(0.99),LinPtr^.dQy(0.99))+rad180;
        if Ang2>=Rad360 then
            Ang2 := Ang2-Rad360;

    end
    else
    begin
        IsStart := False;

{ Gerade Verbindung
        Ang  := atan2(LinPtr^.P1.X-LinPtr^.P4.X,LinPtr^.P1.Y-LinPtr^.P4.Y);
        Ang2 := atan2(LinPtr^.P4.X-LinPtr^.P1.X,LinPtr^.P4.Y-LinPtr^.P1.Y);
}
{ Anfangssteigung statt gerade Verbindung}
        Ang := atan2(LinPtr^.dQx(0.99),LinPtr^.dQy(0.99))+rad180;
        if Ang>=Rad360 then
            Ang := Ang-Rad360;
        Ang2 := atan2(LinPtr^.dQx(0.01),LinPtr^.dQy(0.01));

    end;
    Marked := False;
end;

{---------------------------------------------------------}
{-------------------- NodeClass     ----------------------}     
{---------------------------------------------------------}

constructor NodeClass.Init(P1: Punkt);
begin
    P := P1;
    Sons.Init(2,1);
end;

procedure NodeClass.AddSon(SonPtr: PSon);
 var i: Integer;
begin
    if Sons.Count>=Sons.Limit then
        Sons.SetLimit(Sons.Count+1);
    i := 0;
    while (i<Sons.Count) and (SonPtr^.Ang>PSon(Sons.At(i))^.Ang) do
        i := i+1;
    if (i<Sons.Count) then
        if (SonPtr^.Ang=PSon(Sons.At(i))^.Ang) then
        begin
            SonPtr^.Ang := MinLowerThan(SonPtr^.Ang);
            AddSon(SonPtr);
            exit;
        end;
    Sons.AtInsert(i, SonPtr);
end;

function NodeClass.NextSon(Ang: Real): PSon;
 var
    i: Integer;
    SonPtr: PSon;
begin
    if Sons.Count=1 then
        NextSon := nil
    else
    begin
        SonPtr := nil;
        i := 0;
        while (i<Sons.Count) and (SonPtr=nil) do
            if PSon(Sons.At(i))^.Ang>Ang then
                SonPtr := Sons.At(i)
            else
                i := i+1;
        if SonPtr=nil then
            SonPtr := Sons.At(0);
        if SonPtr^.Marked then
            NextSon := nil
        else
            NextSon := SonPtr;
    end;
end;
                      
destructor NodeClass.Done;
begin
    Sons.Done;
    TObject.Done;
end;



{---------------------------------------------------------}
{-------------------- SelectionClass ---------------------}
{---------------------------------------------------------}

constructor SelectionClass.Init;
begin
  GroupClass.Init(0, 0);
end;

function SelectionClass.GetType: Word;
begin
  GetType := gt_SelectionClass;
end;                                           

procedure SelectionClass.GetFrame(var X1, Y1, X2, Y2: Real);
begin
  X1 := PMin.X;
  Y1 := PMin.Y;
  X2 := PMax.X;
  Y2 := PMax.Y;
end;                                           

procedure SelectionClass.Adjust(AdMode: Word);
 var
    GrObPtr: PGraphicObject;
    i: Integer;
    X1, Y1, X2, Y2: Real;
begin
  GetFrame(X1, Y1, X2, Y2);
  for i := 0 to Count-1 do
  begin
    GrObPtr := GetObject(i);
    if (AdMode and ad_Left)<>0 then
      GrObPtr^.Move(X1-GrObPtr^.PMin.X, 0);
    if (AdMode and ad_Top)<>0 then
      GrObPtr^.Move(0, Y1-GrObPtr^.PMin.Y);
    if (AdMode and ad_Right)<>0 then
      GrObPtr^.Move(X2-GrObPtr^.PMax.X, 0);
    if (AdMode and ad_Bottom)<>0 then
      GrObPtr^.Move(0, Y2-GrObPtr^.PMax.Y);
  end;
  CalcFrame;
end;


{---------------------------------------------------------}
{-------------------- WorldClass    ----------------------}
{---------------------------------------------------------}

constructor WorldClass.Init;
begin
  SelectionClass.Init;
  Pool.Init;
  Copy.Init;
  Temp.Init;
  Script.Init;
  ScriptRecord := False;
  ScriptPlay := False;
  CurrentState := 0;
  Group := nil;
  Pool.Group := @Self;
  Copy.Group := @Self;
  Temp.Group := @Self;
end;

function WorldClass.GetType: Word;
begin
  GetType := gt_WorldClass;
end;

procedure WorldClass.AddLine(X1, Y1, X2, Y2: Real; Col: LongInt);
 var
    LinePtr: PLine;
begin
  LinePtr := New(PLine, Init(X1, Y1, X2, Y2));
  LinePtr^.Color := Col;
  AddObject(LinePtr);
  Pool.AddObject(LinePtr);
end;

procedure WorldClass.AddLineExpanded(X1, Y1, X2, Y2: Real; Col: Longint);
 var
    dx, dy, dMax: Real;
begin
    dx := X2 - X1;   {nderung 1.2.99}
    dy := Y2 - Y1;
    dMax := Max(Abs(dx), Abs(dy));
    if (dMax = 0) then
    begin
         dx := 0;
         dy := 0;
    end
    else
    begin
        dx := 0.5*dx/dMax;
        dy := 0.5*dy/dMax;
    end;
    AddLine(X1-dx, Y1-dy, X2+dx, Y2+dy, Col);
end;


procedure WorldClass.AddBez(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Real; Col: LongInt);
 var
    BezPtr: PLine;
begin
  BezPtr := New(PBezier, Init(X1, Y1,  X2,  Y2,
                              X3,  Y3, X4, Y4));
  BezPtr^.Color := Col;
  AddObject(BezPtr);
  Pool.AddObject(BezPtr);
end;

procedure WorldClass.AddEllipse(X1, Y1, X2, Y2: Real; Col, FCol: LongInt);
 var
    EllipsePtr: PEllipse;
begin
  EllipsePtr := New(PEllipse, Init(X1, Y1, X2, Y2, Col, FCol));
  AddObject(EllipsePtr);
  Pool.AddObject(EllipsePtr);
end;

function  WorldClass.GetSnap(X1, Y1, X2, Y2: Real): Integer;
 var
    sn: PGraphicObject;
    Mi, Ma: Punkt;
begin
  sn := nil;
  Mi.X := InvX(X1);
  Mi.Y := InvY(Y2);
  Ma.X := InvX(X2);
  Ma.Y := InvY(Y1);
  sn := Pool.Snap(Mi, Ma);
  if sn=nil then
    GetSnap := -1
  else if GObjects.IndexOf(HitObjectPtr)<>-1 then
    GetSnap := 1
  else
    getSnap := 0
end;

procedure WorldClass.ToggleHitObject;
begin
  if GObjects.IndexOf(HitObjectPtr)=-1 then
    AddObject(HitObjectPtr)
  else
    RemoveObject(HitObjectPtr);
end;

Procedure WorldClass.SnapAll(X1, Y1, X2, Y2: Real);
 var
    GrObPtr: PGraphicObject;
    i: Integer;
    h: Real;
begin
  X1 := InvX(X1);
  Y1 := InvY(Y1);
  X2 := InvX(X2);
  Y2 := InvY(Y2);

  h := Y1;
  Y1 := Y2;
  Y2 := h;

  i := 0;

  while (i<Pool.Count) do
  begin
    GrObPtr := Pool.GetObject(i);
    if (GrObPtr^.P1.X>=X1) and
       (GrObPtr^.P1.X<=X2) and
       (GrObPtr^.P1.Y>=Y1) and
       (GrObPtr^.P1.Y<=Y2) and
       (GrObPtr^.P4.X>=X1) and
       (GrObPtr^.P4.X<=X2) and
       (GrObPtr^.P4.Y>=Y1) and
       (GrObPtr^.P4.Y<=Y2) then
    begin
      if GObjects.IndexOf(GrObPtr)<>-1 then
        RemoveObject(GrObPtr)
      else
        AddObject(GrObPtr);
    end;
    i := i+1;
  end;
end;

procedure WorldClass.Rotate(Alpha, MX, MY: Real);
 var
    NX1, NY1, NX2, NY2: Real;
begin
  Alpha := Alpha-360*int((Alpha/360));
  if Alpha<0 then
    Alpha := Alpha+360;
  if Alpha<>0 then
  begin
    NX1 := 0;
    NY1 := 0;
    NX2 := 0;
    NY2 := 0;
    if Alpha=90 then
    begin
      NY1 := 1;
      NX2 := -1;
    end else if Alpha=180 then
    begin
      NX1 := -1;
      NY2 := -1;
    end else if Alpha=270 then
    begin
      NY1 := -1;
      NX2 := 1;
    end else
    begin
      Alpha := Alpha*Pi/180;
      NX1 := cos(Alpha);
      NY1 := sin(Alpha);
      NX2 := -sin(Alpha);
      NY2 := cos(Alpha);
    end;
    rot(NX1, NY1, NX2, NY2, MX, MY);
  end;
end;

procedure WorldClass.HMirror(MX: Real);
begin
  rot(-1, 0, 0, 1, MX, 0);
end;

procedure WorldClass.VMirror(MY: Real);
begin
  rot(1, 0, 0, -1, 0, MY);
end;

procedure WorldClass.GroupSelection(Col, FCol: LongInt);
 var
    GroupPtr: PGroup;
    i: Integer;
    X1, Y1, X2, Y2: Real;
begin
  if Count>1 then
  begin
    GroupPtr := nil;
    if GroupPtr = nil then
    begin
      GroupPtr := New(PGroup, Init(Col, FCol));
      for i := 0 to Count-1 do
      begin
        GroupPtr^.AddObject(GetObject(i));
        Pool.RemoveObject(GetObject(i));
      end;
      RemoveAll;
      AddObject(GroupPtr);
      Pool.AddObject(GroupPtr);
    end;
    GroupPtr^.Sort;
    GroupPtr^.CalcFrame;
  end;
end;

procedure WorldClass.GroupSelected(Col, FCol: LongInt);
 var
    GroupPtr: PGroup;
    MGroupPtr: PMultiGroup;
    i: Integer;
    X1, Y1, X2, Y2: Real;
begin
  if ScriptRecord then
    Script.AddScript(scr_Group, Col, FCol, 0,0,0,0);
  if Count>1 then
  begin
    ShowFrame;
    GroupPtr := New(PGroup, Init(Col, FCol));
    for i := 0 to Count-1 do
    begin
      GroupPtr^.AddObject(GetObject(i));
      Pool.RemoveObject(GetObject(i));
    end;
    RemoveAll;
    if GroupPtr^.Sort then
      if (GroupPtr^.Count>1) and not GroupPtr^.Closed then
      begin
        New(MGroupPtr, Init(Col, FCol));
        for i := 0 to GroupPtr^.Count-1 do
          MGroupPtr^.AddObject(GroupPtr^.GetObject(i));
        GroupPtr^.RemoveAll;
        Dispose(GroupPtr, Done);
        GroupPtr := MGroupPtr;
      end;
    GroupPtr^.CalcFrame;
    AddObject(GroupPtr);
    Pool.AddObject(GroupPtr);
{    if GroupPtr^.Closed then}
    begin
      GetFrame(X1, Y1, X2, Y2);
      RedrawSelected(X1, Y1, X2, Y2);
    end;
    ShowFrame;
  end;
end;

procedure WorldClass.ConnectSelected(Col, FCol: LongInt);
 var
    GroupPtr: PGroup;
    i: Integer;
    X1, Y1, X2, Y2: Real;
begin
  if Count>1 then
  begin
    ShowFrame;
    GroupPtr := New(PGroup, Init(Col, FCol));
    for i := 0 to Count-1 do
    begin
      GroupPtr^.AddObject(GetObject(i));
      Pool.RemoveObject(GetObject(i));
    end;
    RemoveAll;
    AddObject(GroupPtr);
    Pool.AddObject(GroupPtr);
    GroupPtr^.Connect;
    GroupPtr^.CalcFrame;
    if GroupPtr^.Closed then
    begin
      GetFrame(X1, Y1, X2, Y2);
      CalcFrame;
      RedrawSelected(X1, Y1, X2, Y2);
    end;
    ShowFrame;
  end;
end;

procedure WorldClass.UngroupSelected;
 var
    GroupPtr: PGroup;
    i: Integer;
    X1, Y1, X2, Y2: Real;
begin
  if ScriptRecord then
    Script.AddScript(scr_Ungroup, 0,0,0,0,0,0);
  ShowFrame;
  if (Count=1) and ((GetObject(0)^.GetType and gt_GroupClass) = gt_GroupClass) then
  begin
    GroupPtr := PGroup(GetObject(0));
    RemoveAll;
    Pool.RemoveObject(GroupPtr);
    for i := 0 to GroupPtr^.Count-1 do
    begin
      AddObject(GroupPtr^.GetObject(i));
      Pool.AddObject(GroupPtr^.GetObject(i));
    end;
    GroupPtr^.RemoveAll;
    if GroupPtr^.Closed then
    begin
      GetFrame(X1, Y1, X2, Y2);
      RedrawSelected(X1, Y1, X2, Y2);
    end;
    Dispose(GroupPtr,Done);
    CalcFrame;
    ShowFrame;
  end;
end;

procedure WorldClass.DrawAll;
begin
 if GridInfo.Show then
    ShowGrid;
  if ClipX1<>-99999 then
  begin
    GrRectangle(ClipX1, ClipY1, ClipX2, ClipY2,
                GetStockObject(LtGray_Brush));
  end;
  Pool.Draw;
end;


procedure WorldClass.Clear(X1, Y1, X2, Y2: Real);
 var
    r: TRect;
begin
  SetRect(r, zx(X1), zy(Y2), zx(X2)+1, zy(Y1)+1);
  FillRect(vhDC, r, TheBkBrush);
end;

procedure WorldClass.Redraw(X1, Y1, X2, Y2: Real);                    
 var
    h: Real;
    OldMin, OldMax: Punkt;
    TempRgn: HRgn;
begin
  if X2<X1 then
  begin
    h := X1;
    X1 := X2;
    X2 := h;
  end;
  if Y2<Y1 then
  begin
    h := Y1;
    Y1 := Y2;
    Y2 := h;
  end;

  OldMin := VisMin;
  OldMax := VisMax;

  VisMin.X := max(VisMin.X,X1);
  VisMin.Y := max(VisMin.Y,Y1);
  VisMax.X := min(VisMax.X,X2);
  VisMax.Y := min(VisMax.Y,Y2);

  TempRgn := CreateRectRgn(zx(VisMin.X), zy(VisMax.Y), zx(VisMax.X)+1, zy(VisMin.Y)+1);
  SelectClipRgn(vhDC, TempRgn);
  DeleteObject(TempRgn);

  Clear(VisMin.X, VisMin.Y, VisMax.X, VisMax.Y);


  if GridInfo.Show then
  begin
    GridInfo.Show := False;
    DrawAll;
    GridInfo.Show := True;
  end
  else
    DrawAll;

  VisMin := OldMin;
  VisMax := OldMax;

  if GridInfo.Show then
    ShowGrid;

  TempRgn := CreateRectRgn(0, 0, xw, yw);
  SelectClipRgn(vhDC, TempRgn);
  DeleteObject(TempRgn);

end;

procedure WorldClass.RedrawSelected(X1, Y1, X2, Y2: Real);
 var
    h: Real;
    OldMin, OldMax: Punkt;
    SX1, SY1, SX2, SY2: Real;
    Rgn1, Rgn2, Rgn3: HRgn;
begin
  if X2<X1 then
  begin
    h := X1;
    X1 := X2;
    X2 := h;
  end;
  if Y2<Y1 then
  begin
    h := Y1;
    Y1 := Y2;
    Y2 := h;
  end;

  GetFrame(SX1, SY1, SX2, SY2);

  if (SX1>X2) or (SY1>Y2) or (SX2<X1) or (SY2<Y1) then
  begin
    Redraw(X1, Y1, X2, Y2);
    Redraw(SX1, SY1, SX2, SY2);
  end
  else
  begin
    SX1 := min(X1, SX1);
    SY1 := min(Y1, SY1);
    SX2 := max(X2, SX2);
    SY2 := max(Y2, SY2);
    Redraw(SX1, SY1, SX2, SY2);
  end;  
end;

procedure WorldClass.SelectAll;
 var
    i: Integer;
begin
  if ScriptRecord then
    Script.AddScript(scr_SelectAll, 0,0,0,0,0,0);
  RemoveAll;
  for i := 0 to Pool.Count-1 do
    AddObject(Pool.GetObject(i));
end;

procedure WorldClass.ClearSelected;
 var
    i: Integer;
begin
  if ScriptRecord then
    Script.AddScript(scr_DeselectAll, 0,0,0,0,0,0);
  RemoveAll;
end;

procedure WorldClass.DeleteSelection;
 var
    i : Integer;
    GrObPtr: PGraphicObject;
begin
  for i := 0 to Count-1 do
  begin
    GrObPtr := GetObject(i);
    Pool.RemoveObject(GrObPtr);
    Dispose(GrObPtr, Done);
  end;
  RemoveAll;
end;

procedure WorldClass.DeleteSelected;
 var
    X1, Y1, X2, Y2: Real;
begin
  if ScriptRecord then
    Script.AddScript(scr_Cut, 0,0,0,0,0,0);
  GetFrame(X1, Y1, X2, Y2);
  DeleteSelection;
  Clear(X1, Y1, X2, Y2);
  Redraw(X1, Y1, X2, Y2);
end;

procedure WorldClass.SetSelectedColor(Col, FCol: LongInt);
 var
    i: Integer;
    GrObPtr: PGraphicObject;
begin
  for i := 0 to Count-1 do
  begin
    GrObPtr := GetObject(i);
    GrObPtr^.Color := Col;
    if (GrObPtr^.GetType and gt_GroupClass) = gt_GroupClass then
      PGroup(GrObPtr)^.FillColor := FCol;
  end;
end;

procedure WorldClass.GetSelectedFrame(var X1d, Y1d, X2d, Y2d: Integer);
 var
    X1, Y1, X2, Y2: Real;
begin
  if Count=0 then
  begin
    X1d := 0;
    Y1d := 0;
    X2d := 0;
    Y2d := 0;
  end
  else
  begin
    GetFrame(X1, Y1, X2, Y2);
    X1d := zx(X1);
    Y1d := zy(Y2);
    X2d := zx(X2);
    Y2d := zy(Y1);
  end;
end;

procedure WorldClass.CopySelected;                                    
 var
    i: Integer;
begin
  if ScriptRecord then
    Script.AddScript(scr_Copy, 0,0,0,0,0,0);
  Copy.Done;
  Copy.Init;
  for i := 0 to Count-1 do
    Copy.AddObject(GetObject(i)^.CreateCopy);
end;

procedure WorldClass.PasteSelected;
 var
    i: Integer;
    GrObPtr: PGraphicObject;
    X1, Y1, X2, Y2: Real;
begin
  ClearSelected;
  if ScriptRecord then
    Script.AddScript(scr_Paste, 0,0,0,0,0,0);
  for i := 0 to Copy.Count-1 do
  begin
    GrObPtr := Copy.GetObject(i)^.CreateCopy;
    AddObject(GrObPtr);
    Pool.AddObject(GrObPtr);
  end;
  if not ScriptPlay then
  begin
    CalcFrame;
    Pool.CalcFrame;
    GetFrame(X1, Y1, X2, Y2);
    Redraw(X1, Y1, X2, Y2);
  end;
end;

procedure WorldClass.AlignSelected;
 var X1, Y1, X2, Y2: Real;
begin
  if GridInfo.SnapX or GridInfo.SnapY then
  begin
    GetFrame(X1, Y1, X2, Y2);
    if ScriptRecord then
      Script.AddScript(scr_Move, SnapX(zx(X1))-X1, SnapY(zy(Y2))-Y2, 0,0,0,0);
    MoveSelected(SnapX(zx(X1))-X1, SnapY(zy(Y2))-Y2);
  end;
end;


procedure WorldClass.MoveSelected(dX, dY: Real);
 var
    X1, Y1, X2, Y2: Real;
begin
  if ScriptRecord then
    Script.AddScript(scr_Move, SnapX(dX)-SnapX(0), SnapY(dY)-SnapY(0), 0,0,0,0);
  Move(SnapX(dx)-SnapX(0), SnapY(dy)-SnapY(0));
  Pool.CalcFrame;
end;

procedure WorldClass.StretchSelected(XOld, YOld, XNew, YNew, XS, YS: Real); 
 var
    i: Integer;
    GrObPtr: PGraphicObject;
    X1, Y1, X2, Y2: Real;
begin
  if ScriptRecord then
    Script.AddScript(scr_Stretch, InvX(XOld), InvY(YOld), SnapX(XNew), SnapY(YNew), XS, YS);
  XOld := InvX(XOld);
  YOld := InvY(YOld);
  XNew := SnapX(XNew);
  YNew := SnapY(YNew);
  Move(-XOld,-YOld);
  Stretch(XS, YS);
  Move(XNew, YNew);
  Pool.CalcFrame;
end;

procedure WorldClass.Rot(NX1, NY1, NX2, NY2, MX, MY: Real);
begin
  if ScriptRecord then
    Script.AddScript(scr_Rot, NX1, NY1, NX2, NY2, MX, MY);
  GroupClass.Rot(NX1, NY1, NX2, NY2, MX, MY);
end;

procedure WorldClass.MakeLine;
 var
    i: Integer;
begin
  if ScriptRecord then
    Script.AddScript(scr_ToLine, 0,0,0,0,0,0);
  for i := 0 to Count-1 do
    Pool.RemoveObject(GetObject(i));
  GroupClass.MakeLine;
  for i := 0 to Count-1 do
    Pool.AddObject(GetObject(i));
end;

procedure WorldClass.MakeBezier;
 var
    i: Integer;
begin
  if ScriptRecord then
    Script.AddScript(scr_ToBez, 0,0,0,0,0,0);
  for i := 0 to Count-1 do
    Pool.RemoveObject(GetObject(i));
  GroupClass.MakeBezier;
  for i := 0 to Count-1 do
    Pool.AddObject(GetObject(i));
end;

procedure WorldClass.SetSelectedPosition(Pos: Word);
 var
    i, j: Integer;
    GrObPtr: PGraphicObject;
    X1, Y1, X2, Y2: Real;
begin
  if ScriptRecord then
    Script.AddScript(scr_SetPos, Pos, 0,0,0,0,0);
  if Count>0 then
  begin
    GetFrame(X1, Y1, X2, Y2);
    case Pos of
      0 : begin                        { nach hinten }
            i := 0;
            j := 0;
            while (i<Pool.Count) do
            begin
              GrObPtr := Pool.GetObject(i);
              if GObjects.IndexOf(GrObPtr)<>-1 then
              begin
                Pool.GObjects.AtDelete(i);
                Pool.GObjects.AtInsert(j, GrObPtr);
                j := j+1;
              end;
              i := i+1;
            end;
          end;
      1 : begin                        { nach vorne }
            i := 0;
            j := Pool.Count;
            while (i<j) do
            begin
              GrObPtr := Pool.GetObject(i);
              if GObjects.IndexOf(GrObPtr)<>-1 then
              begin
                Pool.GObjects.AtDelete(i);
                Pool.GObjects.Insert(GrObPtr);
                j := j-1;
              end
              else
                i := i+1;
            end;
          end;
      2 : begin                        { eins zurck }
            i := 0;
            j := 0;
            while (i<Pool.Count) do
            begin
              GrObPtr := Pool.GetObject(i);
              if GObjects.IndexOf(GrObPtr)<>-1 then
              begin
                if j<>0 then
                begin
                  Pool.GObjects.AtDelete(i);
                  Pool.GObjects.AtInsert(i-1, GrObPtr);
                end;
              end
              else
                j := 1;
              i := i+1;
            end;
          end;
      3 : begin                        { eins vor }
            i := Pool.Count-1;
            j := 0;
            while (i>=0) do
            begin
              GrObPtr := Pool.GetObject(i);
              if GObjects.IndexOf(GrObPtr)<>-1 then
              begin
                if j<>0 then
                begin
                  Pool.GObjects.AtDelete(i);
                  Pool.GObjects.AtInsert(i+1, GrObPtr);
                end;
              end
              else
                j := 1;
              i := i-1;
            end;
          end;
    end;
    Redraw(X1, Y1, X2, Y2);
  end;
end;

procedure WorldClass.WriteOut;
begin
  ClearSelected;
  Pool.WriteOut;
end;

procedure WorldClass.ReplaySelected;
 var
    i: Integer;
    Command: ScriptRec;
    X1, Y1, X2, Y2: Real;
begin
  i := 0;
  GiveWindowTime;
  While (i<Script.Count) and not BreakConv do
  begin
    Script.GetData(i, Command);
    with Command do
    begin
      case Aktion of
        scr_Move         : Move(Data[0],Data[1]);
        scr_Stretch      : begin
                             Move(-Data[0],-Data[1]);
                             Stretch(Data[4], Data[5]);
                             Move(Data[2], Data[3]);
                           end;
        scr_Rot          : Rot(Data[0],Data[1],Data[2],Data[3],Data[4],Data[5]);
        scr_ToLine       : MakeLine;
        scr_ToBez        : MakeBezier;
        scr_SelectAll    : SelectAll;
        scr_DeselectAll  : ClearSelected;
        scr_Cut          : DeleteSelected;
        scr_Copy         : CopySelected;
        scr_Paste        : PasteSelected;
        scr_Group        : GroupSelected(trunc(Data[0]), trunc(Data[1]));
        scr_Ungroup      : UngroupSelected;
        scr_SetPos       : SetSelectedPosition(trunc(Data[0]));
      end;
    end;
    i := i+1;
  end;
end;

procedure WorldClass.Replay;
begin
  ReplaySelected;
  CalcFrame;
  Pool.CalcFrame;
  GrClear;
  DrawAll;
end;

procedure WorldClass.IterateStep(n, FirstCount, LastCount: Integer; SelfIterate: Boolean);
 var
    i: Integer;
    X1, Y1, X2, Y2: Real; 
begin
  if (n>0) and not BreakConv then
  begin
    for i := FirstCount to LastCount - 1 do
    if not BreakConv then
    begin
      if (i<Pool.Count) then
      begin
        ClearSelected;
        AddObject(Pool.GetObject(i));
        CalcFrame;
        GetFirstSelectedLinePoints(X1, Y1, X2, Y2);
        Script.SetPos(X1, Y1, X2, Y2);
        ReplaySelected;
      end;
    end;
    if SelfIterate then
      IterateStep(n-1, FirstCount, Pool.Count, SelfIterate)
    else
      IterateStep(n-1, LastCount, Pool.Count, SelfIterate);
  end;
end;

procedure WorldClass.Iterate(n: Integer; SelfIterate: Boolean);
 var
    LastCount: Integer;
    X1, Y1, X2, Y2: Real;
begin
  SetSelectedPosition(1);
  IterateStep(n, Pool.Count-Count, Pool.Count, SelfIterate);
  CalcFrame;
  Pool.CalcFrame;
  GrClear;
  DrawAll;
end;


function CreateBlendLine(Line1Ptr, Line2Ptr: PLine; t: Real): PLine;
 var
    NewLinePtr: PLine;
    Bez1Ptr, Bez2Ptr: PLine;
    BP1, BP2, BP3, BP4: Punkt;
begin
  if (Line1Ptr^.GetType = gt_LineClass) and (Line2Ptr^.GetType = gt_LineClass) then
  begin
    BlendPoint(BP1, Line1Ptr^.P1, Line2Ptr^.P1, t);
    BlendPoint(BP4, Line1Ptr^.P4, Line2Ptr^.P4, t);
    NewLinePtr := New(PLine, Init(BP1.X, BP1.Y, BP4.X, BP4.Y));
  end
  else 
  begin
    Bez1Ptr := Line1Ptr;
    Bez2Ptr := Line2Ptr;
    if Line1Ptr^.GetType <> gt_LineClass then
      Bez1Ptr := Line1Ptr
    else
      Bez1Ptr := LineToBezier(Line1Ptr);
    if Line2Ptr^.GetType <> gt_LineClass then
      Bez2Ptr := Line2Ptr
    else
      Bez2Ptr := LineToBezier(Line2Ptr);
    BlendPoint(BP1, Bez1Ptr^.P1, Bez2Ptr^.P1, t);
    BlendPoint(BP2, Bez1Ptr^.P2, Bez2Ptr^.P2, t);
    BlendPoint(BP3, Bez1Ptr^.P3, Bez2Ptr^.P3, t);
    BlendPoint(BP4, Bez1Ptr^.P4, Bez2Ptr^.P4, t);
    if Bez1Ptr<>Line1Ptr then
      Dispose(Bez1Ptr, Done);
    if Bez2Ptr<>Line2Ptr then
      Dispose(Bez2Ptr, Done);
    NewLinePtr := New(PBezier, Init( BP1.X, BP1.Y,
                                     BP2.X, BP2.Y,
                                     BP3.X, BP3.Y,
                                     BP4.X, BP4.Y));
  end;
  NewLinePtr^.Color := rgb(0,0,0);
  CreateBlendLine := NewLinePtr;
end;

function CreateBlendGroup(GrOb1Ptr, GrOb2Ptr: PGraphicObject; t: Real): PGroup;
 var
    i, j, iMax, jMax: Integer;
    NewGroupPtr: PGroup;
    Line1Ptr, Line2Ptr, NewLinePtr: PLine;
    G1P, G2P: Punkt;
    fc1, fc2: LongInt;
begin
  fc1 := -1;
  fc2 := -1;
  if GrOb1Ptr^.GetType and gt_GroupClass = gt_GroupClass then
    fc1 := PGroup(GrOb1Ptr)^.FillColor;
  if GrOb2Ptr^.GetType and gt_GroupClass = gt_GroupClass then
    fc2 := PGroup(GrOb2Ptr)^.FillColor;
  new(NewGroupPtr, Init( BlendColor(GrOb1Ptr^.Color, GrOb2Ptr^.Color, t),
                         BlendColor(fc1, fc2, t)));
  i := -1;
  j := -1;
  if (GrOb1Ptr^.GetType and gt_GroupClass) = gt_GroupClass then
    iMax := PGroup(GrOb1Ptr)^.Count
  else
    iMax := 1;
  if (GrOb2Ptr^.GetType and gt_GroupClass) = gt_GroupClass then
    jMax := PGroup(GrOb2Ptr)^.Count
  else
    jMax := 1;
  while (i<iMax) or (j<jMax) do
  begin
    i := i+1;
    if GrOb1Ptr^.GetType and gt_GroupClass = gt_GroupClass then
      while (i<iMax) and ((PGroup(GrOb1Ptr)^.GetObject(i)^.GetType and gt_LineClass)<>gt_LineClass) do
        i := i+1;
    j := j+1;
    if GrOb2Ptr^.GetType and gt_GroupClass = gt_GroupClass then
      while (j<jMax) and ((PGroup(GrOb2Ptr)^.GetObject(j)^.GetType and gt_LineClass)<>gt_LineClass) do
        j := j+1;
    if (i<iMax) or (j<jMax) then
    begin
      if GrOb1Ptr^.GetType and gt_GroupClass = gt_GroupClass then
      begin
        if i<iMax then
          Line1Ptr := PLine(PGroup(GrOb1Ptr)^.GetObject(i))
        else
        begin
          NewLinePtr := PLine(PGroup(GrOb1Ptr)^.GetObject(0));
          Line1Ptr := New(PLine, Init( NewLinePtr^.P1.X, NewLinePtr^.P1.Y,
                                       NewLinePtr^.P1.X, NewLinePtr^.P1.Y));
        end;
      end
      else
      begin
        if i<iMax then
          Line1Ptr := PLine(GrOb1Ptr)
        else
          Line1Ptr := New(PLine, Init( GrOb1Ptr^.P1.X, GrOb1Ptr^.P1.Y,
                                       GrOb1Ptr^.P1.X, GrOb1Ptr^.P1.Y));
      end;
      if GrOb2Ptr^.GetType and gt_GroupClass = gt_GroupClass then
      begin
        if j<jMax then
          Line2Ptr := PLine(PGroup(GrOb2Ptr)^.GetObject(j))
        else
        begin
          NewLinePtr := PLine(PGroup(GrOb2Ptr)^.GetObject(0));
          Line2Ptr := New(PLine, Init( NewLinePtr^.P1.X, NewLinePtr^.P1.Y,
                                       NewLinePtr^.P1.X, NewLinePtr^.P1.Y));
        end;
      end
      else
      begin
        if j<jMax then
          Line2Ptr := PLine(GrOb2Ptr)
        else
          Line2Ptr := New(PLine, Init( GrOb2Ptr^.P1.X, GrOb2Ptr^.P1.Y,
                                       GrOb2Ptr^.P1.X, GrOb2Ptr^.P1.Y));
      end;
      NewLinePtr := CreateBlendLine(Line1Ptr, Line2Ptr, t);
      NewGroupPtr^.AddObject(NewLinePtr);
      if i>=iMax then
        Dispose(Line1Ptr, Done);
      if j>=jMax then
        Dispose(Line2Ptr, Done);
    end;
  end;
  NewGroupPtr^.Sort;
  NewGroupPtr^.CalcFrame;
  CreateBlendGroup := NewGroupPtr;
end;


procedure WorldClass.Blend(Steps: Integer);
 var
    i: Integer;
    GrOb1Ptr, GrOb2Ptr: PGraphicObject;
    GroupPtr, BlendGroupPtr: PGroup;
    X1, Y1, X2, Y2: Real;
begin
  if (Count=2) then
  begin
    BlendGroupPtr := new(PGroup, Init(rgb(0,0,0), rgb(255,0,0)));
    GrOb1Ptr := GetObject(0);
    GrOb2Ptr := GetObject(1);
    for i := 1 to Steps do
    begin
      GroupPtr := CreateBlendGroup(GrOb1Ptr, GrOb2Ptr, 1.0*i/(Steps+1));
      if GroupPtr^.Count=1 then
      begin
        BlendGroupPtr^.AddObject(GroupPtr^.GetObject(0));
        GroupPtr^.RemoveAll;
        Dispose(GroupPtr, Done);
      end
      else
      begin
        GroupPtr^.CalcFrame;
        BlendGroupPtr^.AddObject(GroupPtr);
      end;
    end;
    BlendGroupPtr^.CalcFrame;
    Pool.AddObject(BlendGroupPtr);
    Pool.RemoveObject(GrOb2Ptr);
    Pool.AddObject(GrOb2Ptr);
    CalcFrame;
    Pool.CalcFrame;
    GetFrame(X1, Y1, X2, Y2);
    RedrawSelected(X1, Y1, X2, Y2);
  end;
end;

procedure WorldClass.GetFirstSelectedLinePoints(var X1, Y1, X2, Y2: Real);
 var
    GroupPtr: PGroup;
    GrObPtr: PGraphicObject;
begin
  GroupPtr := @Self;
  while (GroupPtr^.Count>0) and (GroupPtr^.GetObject(0)^.GetType and gt_GroupClass = gt_GroupClass) do
    GroupPtr := PGroup(GroupPtr^.GetObject(0));
  if GroupPtr^.Count=0 then
    GrObPtr := GroupPtr
  else
    GrObPtr := GroupPtr^.GetObject(0);
  X1 := GrObPtr^.P1.X;
  Y1 := GrObPtr^.P1.Y;
  X2 := GrObPtr^.P4.X;
  Y2 := GrObPtr^.P4.Y;
end;

destructor WorldClass.Done;
begin
  Temp.Done;
  Pool.Done;
  Copy.Done;
  Script.Done;
  RemoveAll;
  SelectionClass.Done;
end;                                            

{---------------------------------------------------------}
{-------------------- ConvertClass  ----------------------}
{---------------------------------------------------------}

Constructor ConvertClass.Init;
begin
  WorldClass.Init;
  Punkte.Init(50,10);
  new(PolyPtr);
end;

function ConvertClass.GetType: Word;
begin
  GetType := gt_ConvertClass;
end;

function ConvertClass.AddTest(GrObPtr: PGraphicObject): Boolean;
begin
    if (GrObPtr^.PMin.X<=ClipX2) and (GrObPtr^.PMax.X>=ClipX1) and
       (GrObPtr^.PMin.Y<=ClipY2) and (GrObPtr^.PMax.Y>=ClipY1) then
        AddTest := True
    else
        AddTest := False;
end;

procedure ConvertClass.Flatten;
 var
    i,j: Integer;
    GrObPtr: PGraphicObject;
begin
  CurrentState := 6;
  State3 := Pool.Count;
  State2 := 0;
  ClearSelected;

  i := 0;
  while (i<Pool.Count) and not BreakConv do
  begin
    State2 := State2+1;
    GiveWindowTime;
    GrObPtr := Pool.GetObject(i);
    if AddTest(GrObPtr) and not BreakConv then
    begin
      if (GrObPtr^.GetType and gt_GroupClass) = gt_GroupClass then
      begin
        State3 := State3+PGroup(GrObPtr)^.Count;
        for j := 0 to PGroup(GrObPtr)^.Count-1 do
          Pool.AddObject(PGroup(GrObPtr)^.GetObject(j));
        PGroup(GrObPtr)^.RemoveAll;
        Pool.RemoveObject(GrObPtr);
        i := i-1;
        Dispose(GrObPtr, Done);
      end
      else
      if TestUnique(GrObPtr) then
      begin
        AddObject(GrObPtr);
      end;
    end;
    i := i+1;
  end;
end;


procedure ConvertClass.Split;
 var
    LinePtr, Line2Ptr: PLine;
    LocalLine: PLine;
    i, j, Touch: Integer;
    t1, t2, F, r,
    dummy1, dummy2: Real;

begin
    SkipMode := 0;
    CurrentState := 1;
    i := 0;
    LocalLine := nil;
    while (i<Count) and not BreakConv and (SkipMode<3) do
    begin
        SkipMode := 0;
        State2 := i;
{        setcolor(0);
        setfillstyle(solidLn,0);
        bar(0,0,200,10);
        setcolor(32);
        outtextxy(1,1,NToS(i,7)+'/'+NToS(Linien.Count,7)+'   Mem:'+NToS(MemAvail,6));}
        LinePtr := PLine(GetObject(i));

        LocalLine := PLine(LinePtr^.CreateCopy);

        if AddTest(LinePtr) then
        begin

            F := abs(LocalLine^.P1.X-LocalLine^.P4.X)+
                 abs(LocalLine^.P1.Y-LocalLine^.P4.Y);

            j := i+1;
            while (j<Count) and (F>MinF) and not BreakConv and (SkipMode<2) do
            begin
                SkipMode := 0;
                State3 := Count;
                if AddTest(GetObject(j)) then
                begin
                    Touch := LocalLine^.GetTouches(PLine(GetObject(J)));
                    if (Touch<>0) then
                    begin
                      case Touch of
                        1 : begin
                              r := LocalLine^.ShrinkL;
                              if r<0.5 then
                              begin
                                if LocalLine^.GetCut(PLine(GetObject(j)),t1,t2) then
                                begin
                                  LocalLine^.Transform(0,t1);
                                  LocalLine^.ShrinkR;
                                  if not LocalLine^.GetCut(PLine(GetObject(j)),dummy1,dummy2) then
                                  begin
                                    t1 := r+(1-r)*t1;
                                    if LinePtr^.Matters(t1) then
                                    begin
                                      Line2Ptr := PLine(LinePtr^.CreateCopy);
                                      Line2Ptr^.Transform(t1,1);
                                      AddObject(Line2Ptr);
                                      Pool.AddObject(Line2Ptr);
                                      LinePtr^.Transform(0,t1);
                                    end;
                                    if PLine(GetObject(j))^.Matters(t2) then
                                    begin
                                      Line2Ptr := PLine(PLine(GetObject(j))^.CreateCopy);
                                      Line2Ptr^.Transform(t2,1);
                                      AddObject(Line2Ptr);
                                      Pool.AddObject(Line2Ptr);
                                      PLine(GetObject(j))^.Transform(0,t2);
                                    end;
                                  end;
                                end;
                              end;
                            end;
                        2 : begin
                              r := LocalLine^.ShrinkR;
                              if r>0.5 then
                              begin
                                if LocalLine^.GetCut(PLine(GetObject(j)),t1,t2) then
                                begin
                                  LocalLine^.Transform(t1,1);
                                  LocalLine^.ShrinkL;
                                  if not LocalLine^.GetCut(PLine(GetObject(j)),dummy1,dummy2) then
                                  begin
                                    t1 := r*t1;
                                    if LinePtr^.Matters(t1) then
                                    begin
                                      Line2Ptr := PLine(LinePtr^.CreateCopy);
                                      Line2Ptr^.Transform(t1,1);
                                      AddObject(Line2Ptr);
                                      Pool.AddObject(Line2Ptr);
                                      LinePtr^.Transform(0,t1);
                                    end;
                                    if PLine(GetObject(j))^.Matters(t2) then
                                    begin
                                      Line2Ptr := PLine(PLine(GetObject(j))^.CreateCopy);
                                      Line2Ptr^.Transform(t2,1);
                                      AddObject(Line2Ptr);
                                      Pool.AddObject(Line2Ptr);
                                      PLine(GetObject(j))^.Transform(0,t2);
                                    end;
                                  end;
                                end;
                              end;
                            end;
                        3 : begin
                              LocalLine^.Transform(0.4,0.6);
                              if LocalLine^.GetCut(PLine(GetObject(J)),t1,t2) then
                              begin
                                Line2Ptr := PLine(GetObject(j));
                                RemoveObject(Line2Ptr);
                                Pool.RemoveObject(Line2Ptr);
                                Dispose(Line2Ptr, Done);
                              end;
                            end;
                      end;

                      Dispose(LocalLine,Done);

                      LocalLine := PLine(LinePtr^.CreateCopy);
                      F := abs(LocalLine^.P1.X-LocalLine^.P4.X)+
                           abs(LocalLine^.P1.Y-LocalLine^.P4.Y);
                    end
                    else
                    begin
                        if LocalLine^.GetCut(PLine(GetObject(J)), t1, t2) then
                        begin
                            if LinePtr^.Matters(t1) then
                            begin
                              Line2Ptr := PLine(LinePtr^.CreateCopy);
                              Line2Ptr^.Transform(t1,1);
                              AddObject(Line2Ptr);
                              Pool.AddObject(Line2Ptr);
                              LinePtr^.Transform(0,t1);
                            end;
                            if PLine(GetObject(j))^.Matters(t2) then
                            begin
                                Line2Ptr := PLine(PLine(GetObject(j))^.CreateCopy);
                                Line2Ptr^.Transform(t2,1);
                                AddObject(Line2Ptr);
                                Pool.AddObject(Line2Ptr);
                                PLine(GetObject(j))^.Transform(0,t2);
                                j := j-1;
                            end;

                            Dispose(LocalLine,Done);

                            LocalLine := PLine(LinePtr^.CreateCopy);
                            F := abs(LocalLine^.P1.X-LocalLine^.P4.X)+
                                 abs(LocalLine^.P1.Y-LocalLine^.P4.Y);
                        end;
                    end;
                end;
                j := j+1;
            end;
        end;
        i := i+1;

        Dispose(LocalLine,Done);
    end;
end;

function ConvertClass.FindNode(P: Punkt): PNode;
 var
    i: Integer;
    NodePtr: PNode;
begin
    for i := 0 to Punkte.Count-1 do
    begin
        NodePtr := Punkte.at(I);
        if Abs(NodePtr^.P.X-P.X)+Abs(NodePtr^.P.Y-P.Y)<2*MinF then { vorher <0.1}
        begin
            FindNode := NodePtr;
            exit;
        end;
    end;
    NodePtr := New(PNode, Init(P));
    if Punkte.Count>=Punkte.Limit then
        Punkte.SetLimit(Punkte.Count+10);
    Punkte.Insert(NodePtr);
    FindNode := NodePtr;
end;

procedure ConvertClass.MakeGraph;
 var
    Node1Ptr, Node2Ptr: PNode;
    SonPtr: PSon;
    LinePtr: PLine;
    Anz, i, j: Integer;
begin
    CurrentState := 2;
    State3 := Count;

    if ClipX1<>-99999 then
        GrRectangle(ClipX1, ClipY1, ClipX2, ClipY2,
                    GetStockObject(LtGray_Brush));

{    SelectObject(vhDC,GetStockObject(Black_Pen));}
    for i := 0 to Count-1 do   { Graphen aufbauen }
    if not BreakConv then begin
        LinePtr := PLine(GetObject(i));
        State2 := i+1;
        LinePtr^.Draw;

        if AddTest(LinePtr) then
        begin

            Node1Ptr := FindNode(LinePtr^.P1);
            Node2Ptr := FindNode(LinePtr^.P4);

            LinePtr^.P1 := Node1Ptr^.P;
            LinePtr^.P4 := Node2Ptr^.P;
                
            if Node1Ptr<>Node2Ptr then
            begin
                Node1Ptr^.AddSon(New(PSon,Init(Node2Ptr,True,  LinePtr)));
                Node2Ptr^.AddSon(New(PSon,Init(Node1Ptr,False, LinePtr)));
            end;
        end;
    end;

{    if not BreakConv then
        GrClear;}

    State2 := 0;
    i := 0;
    while (i<Punkte.Count) and not BreakConv do
    begin
        State3 := Punkte.Count;
        State2 := trunc(max(State2,i));
        GiveWindowTime;
        Node1Ptr := Punkte.At(I);
        if Node1Ptr^.Sons.Count=0 then
            Punkte.AtFree(I)
        else if Node1Ptr^.Sons.Count=1 then
        begin
            SonPtr := Node1Ptr^.Sons.At(0);
            Node2Ptr := SonPtr^.NodePtr;
            Anz := Node2Ptr^.Sons.Count;
            j := 0;
            while (j<Anz) and (PSon(Node2Ptr^.Sons.At(J))^.NodePtr<>Node1Ptr) do
                J := J+1;
            Punkte.AtFree(i);
            if J<Anz then
            begin
                if Anz<=2 then
                begin
                    Node2Ptr^.Sons.AtFree(j);
                    i := 0;                    
                end
                else
                    Node2Ptr^.Sons.AtFree(J);
            end;
        end
        else
            i := i+1;
    end;


    CurrentState := 3;
    State3 := Punkte.Count;
    for i := 0 to Punkte.Count-1 do if not BreakConv then
    begin
        State2 := i+1;
        GiveWindowTime;
        GetPolys(Punkte.At(i));
    end;
end;


function ConvertClass.TestPoly(NodePtr: PNode; SonPtr: PSon): Boolean;
 var
     LocalNodePtr: PNode;
     LocalSonPtr: PSon;
     i: Integer;
     NodeList: TCollection; { of PNode }
begin
   if (SonPtr^.Marked) then
       TestPoly := False
   else
   begin
       LocalSonPtr := SonPtr;

       LocalNodePtr := LocalSonPtr^.NodePtr;
       NodeList.Init(1001,0);
       NodeList.Insert(LocalNodePtr);
       while (LocalNodePtr<>NodePtr) and (NodeList.Count<1000) do
       begin
           LocalSonPtr := LocalNodePtr^.NextSon(LocalSonPtr^.Ang2);
           if LocalSonPtr=nil then
           Begin
               NodeList.DeleteAll;
               NodeList.Done;
               TestPoly := False;
               exit;
           end;

           LocalNodePtr := LocalSonPtr^.NodePtr;
           if NodeList.IndexOf(LocalNodePtr)<>-1 then
           begin
               NodeList.DeleteAll;
               NodeList.Done;
               TestPoly := False;
               exit;
           end;

           NodeList.Insert(LocalNodePtr);
       end;
       if NodeList.Count<1000 then
           TestPoly := True
       else
           TestPoly := False;
       NodeList.DeleteAll;
       NodeList.Done;
   end;
end;

procedure ConvertClass.ShowPoly(NodePtr: PNode; SonPtr: PSon);
 var
     LocalNodePtr: PNode;
     LocalSonPtr: PSon;
     PolyPointsPtr: PPolyPoint;
     Anz: Word;
     OldPen: HPen;
     OldBrush: HBrush;
begin
   New(PolyPointsPtr);
   Anz := 0;
   LocalSonPtr := SonPtr;
   LocalSonPtr^.LinePtr^.GetPoints(PolyPointsPtr, Anz, not LocalSonPtr^.IsStart);
   LocalNodePtr := LocalSonPtr^.NodePtr;
   while (LocalNodePtr<>NodePtr) and not BreakConv do
   begin
       LocalSonPtr := LocalNodePtr^.NextSon(LocalSonPtr^.Ang2);
       LocalSonPtr^.LinePtr^.GetPoints(PolyPointsPtr, Anz, not LocalSonPtr^.IsStart);
       LocalNodePtr := LocalSonPtr^.NodePtr;
   end;

   OldPen := SelectObject(vhDC,TheRedPen);
   OldBrush := selectObject(vhDC,GetStockObject(LtGray_Brush));
   Polygon(vhDC,PolyPointsPtr^,Anz);
   SelectObject(vhDC, OldPen);
   SelectObject(vhDC, OldBrush);

   Dispose(PolyPointsPtr);
end;

procedure ConvertClass.GetPolys(NodePtr: PNode);
 var
    i, j: Integer;
    LocalSonPtr: PSon;
    LocalNodePtr: PNode;
    c: char;
begin
    if NodePtr^.Sons.Count>1 then
    begin
        for i := 0 to NodePtr^.Sons.Count - 1 do
        if not BreakConv then begin
            LocalNodePtr := NodePtr;
            LocalSonPtr := NodePtr^.Sons.At(i);
            if TestPoly(LocalNodePtr, LocalSonPtr) then
            begin
                showPoly(LocalNodePtr, LocalSonPtr);
                AusgabeM(LocalSonPtr^.LinePtr, LocalSonPtr^.IsStart);
                repeat
                    LocalSonPtr^.Marked := True;
                    AusgabeC(LocalSonPtr^.LinePtr,LocalSonPtr^.IsStart);
                    LocalNodePtr := LocalSonPtr^.NodePtr;
                    if LocalNodePtr<>NodePtr then
                        LocalSonPtr := LocalNodePtr^.NextSon(LocalSonPtr^.Ang2);
                until (LocalNodePtr = NodePtr) or (LocalSonPtr = nil) or BreakConv;
                Ausgabe('b')
            end;
        end;
    end;
end;


destructor ConvertClass.Done;
begin
  Punkte.Done;
  Dispose(PolyPtr);
  WorldClass.Done;
end; 




{---------------------------------------------------------------------}
{--------------- Graphikfunktionen -----------------------------------}
{---------------------------------------------------------------------}

var
  GraphDriver : Integer;
  GraphMode   : Integer;
  MidX, MidY  : Integer;

Procedure GrInit;
var
  I: Integer;
  S: String;
begin

  GrClear;
  ResetZoom;
  SendMessage(TheWindow, wm_Command, cm_SetScrollbars, 0)

end;

Procedure GrExit;
begin
end;




{---------------------------------------------------------}
{-------------------- Hauptprogramm ----------------------}
{---------------------------------------------------------}

function GetState(Nr: Integer): LongInt;
begin
  case Nr of
    0 : GetState := World.CurrentState;
    1 : GetState := World.State2;
    2 : GetState := World.State3;
  end;  
end;

procedure Skip(m: Integer);
begin
    SkipMode := m;
end;

procedure BreakMain;
begin
    BreakConv := True;
end;

procedure ShowProgress;
begin
    ShowConv := True;
end;

procedure HideProgress;
begin
    ShowConv := False;    
end;


procedure ShowAI(AiFileName: String);
 var
    i: Integer;
    SelectionPtr: PSelection;
begin

    BreakConv := False;

    GrInit;
{    ClearSelected;}

    if World.Copy.Count=0 then
      SelectionPtr := nil
    else
      SelectionPtr := PSelection(World.Copy.CreateCopy);

    World.Done;
    World.init;

    if not BreakConv then
        ReadFile(AiFileName);

    if not BreakConv then
        World.DrawAll;

    SendMessage(TheWindow, wm_Command, cm_SaveBitmap, 0);

    World.CurrentState := -1;

    GrExit;
    BreakConv := False;

    if SelectionPtr<>nil then
    begin
      for i := 0 to SelectionPtr^.Count-1 do
        World.Copy.AddObject(SelectionPtr^.GetObject(i));
      SelectionPtr^.RemoveAll;
      Dispose(SelectionPtr, Done);
    end;

end;


procedure Main;
 var
    OpenOk: Boolean;
    ErrText: array[0..255] of Char;
begin

    BreakConv := False;

    SendMessage(TheWindow, wm_Command, cm_SaveBitmap, 0);

    if not BreakConv then
      World.Flatten;

    if not BreakConv then
        World.Split;

    if not BreakConv then
        GrClear;
    
    if not BreakConv then
    begin
      PASNEWNAME := CDRDIR+'PASNEW.IA';
      Assign(OutFile, PASNEWNAME);
      repeat
        StrPCopy(ErrText,'Fehler beim Anlegen der Flchendatei: <'+PASNEWNAME+'>!');
        OpenOk := True;
        {$I-}
        Rewrite(OutFile);
        {$I+}
        if IOResult<>0 then
        begin
          OpenOk := False;
          if MessageBox(TheWindow,ErrText,'AI -> CDR',mb_RetryCancel+mb_iconExclamation)=idCancel then
          begin
            BreakConv := True;
            PASNEWNAME := '';
          end;            
        end;
      until OpenOK or BreakConv;
    end;

    if not BreakConv then
        World.MakeGraph;

    {$I-}
    Close(OutFile);
    {$I+}

    World.Punkte.Done;
    World.Punkte.Init(50,10);

    World.ClearSelected;

    World.CurrentState := -2;

    GrExit;
    BreakConv := False;
end;



procedure UnitInit(vWindow: HWnd);
begin
  TheWindow := vWindow;
  ShowConv := True;
  FrameMode := 0;
  GridInfo.GridX := 1;
  GridInfo.GridY := 1;
  GridInfo.SnapX := False;
  GridInfo.SnapY := False;
  GridInfo.Show  := False;
  MinX := 0;
  MinY := 0;
  MaxX := xw-10;
  MaxY := yw-10;
  DeltaTest := 0.001;
  MinF := 0.001;
  resetClip;
  resetZoom;
  SendMessage(TheWindow, wm_Command, cm_SetScrollbars, 0);
  MaxGroup := 0;
end;


{ Deal with deinitialization of unit. }
var
  SaveExit: Pointer;

procedure Bezier2Exit;
far;
begin
  ExitProc := SaveExit;

  World.Done;

  DeletePasNew;
end;

{ initialization }
begin
  PASNEWNAME := '';
  World.Init;

  SaveExit := ExitProc;
  ExitProc := @Bezier2Exit;
end.
