
unit Drawing;

{ This unit defines a tool bar window for the paint program.
  The toolbar is responsible for the management of the available tools
  and displays the icons for the available tools. Selection of the current
  tool is handled here.
}  

interface

uses WinTypes, WinProcs, {WObjects}Objects, Win31, Defs, OWindows;

type
  PDrawing = ^TDrawing;
  TDrawing = object(TWindow)

    MyCursor: HCursor;
    MsgWindow: HWnd;
    MouseX, MouseY: Integer;
    Deactivated,
    LeftButtonPressed: Boolean;
    Tasten: Word;
    OldXPos, OldYPos: Integer;
    OldXw, OldYw: Integer;

    { Creation and destruction }
    constructor Init(AParent: PWindowsObject);
    procedure   InitDCs; virtual;
    procedure   ExitDCs; virtual;
    procedure   InitMsgWindow(AMsgWindow: HWnd); virtual;
    procedure   GetWindowClass(var AWndClass: TWndClass); virtual;    
    function    GetClassName: PChar; virtual;
    destructor  Done; virtual;

    { Display }
    procedure Paint(PaintDC: HDC; var PaintInfo: TPaintStruct); virtual;


    procedure Activate; virtual;
    procedure Deactivate; virtual;

    { Commands }
    procedure CMSaveBitmap(VAR MSG: TMessage); virtual cm_First + cm_SaveBitmap;
    procedure CMClearBitmap(VAR MSG: TMessage); virtual cm_First + cm_ClearBitmap;
    procedure CMSetScrollBars(VAR MSG: TMessage); virtual cm_First + cm_SetScrollBars;

    { Window manager responses }
    Procedure wmSize(VAR MSG: TMessage); virtual wm_First + wm_Size;
    Procedure wmSetCursor(VAR MSG: TMessage); virtual wm_First + wm_SetCursor;
    Procedure wmRButtonDown(VAR MSG: TMessage); virtual wm_First + wm_RButtonDown;
    Procedure wmLButtonDown(VAR MSG: TMessage); virtual wm_First + wm_LButtonDown;
    Procedure wmMouseMove(VAR MSG: TMessage); virtual wm_First + wm_MouseMove;
    Procedure wmLButtonUp(VAR MSG: TMessage); virtual wm_First + wm_LButtonUp;
  end;

implementation


uses Bezier2, BezDefs, Strings,GrTools;


{ Create the actual toolbar and a new instance of each tool it is to contain.
}
constructor TDrawing.Init(AParent: PWindowsObject);
begin
  TWindow.Init(AParent, nil);
  Attr.Style := ws_Child or ws_Visible or ws_HScroll or ws_VScroll;
  MyCursor := LoadCursor(0, idc_Arrow);
  Deactivate;
  LeftButtonPressed := False;

  OldXPos := 0;
  OldYPos := 0;
  OldXw := xw;
  OldYw := yw;
  Scroller := New(PScroller, Init(@Self, 1, 1, 640, 480));
  Scroller^.XLine := 20;
  Scroller^.YLine := 15;
  Scroller^.XPage := 120;
  Scroller^.YPage := 90;
end;


procedure TDrawing.InitDCs;
 var
    r: TRect;
begin
  vHDC := GetDC(HWindow);

  BufferDC := CreateCompatibleDC(vhDC);
  BufferBitmap := SelectObject(BufferDC,
                    CreateCompatibleBitmap(vhDC, xw,yw));
  SetRect(r, 0, 0, xw, yw);
  FillRect(BufferDC, r, TheBKBrush);

  CreateCaret(HWindow, 0, 1, 1);
  HideCaret(HWindow);
  CaretVis := False;
end;

procedure TDrawing.ExitDCs;
begin
  DeleteObject(SelectObject(BufferDC, BufferBitmap));
  DeleteDC(BufferDC);
  ReleaseDC(HWindow, vHDC);
end;


procedure TDrawing.InitMsgWindow(AMsgWindow: HWnd);
begin
  MsgWindow := AMsgWindow;
end;

{ Destroy each tool instance befory dying.
}

procedure TDrawing.GetWindowClass(var AWndClass: TWndClass);
begin
  TWindow.GetWindowClass(AWndClass);
  AWndClass.hbrBackground := GetStockObject(White_Brush);
end;

function TDrawing.GetClassName: PChar;
begin
  GetClassName := 'MyWhiteWindow';
end;

destructor TDrawing.Done;
begin
  DestroyCaret;
  ExitDCs;
  TWindow.Done;
end;


procedure TDrawing.Paint(PaintDC: HDC; var PaintInfo: TPaintStruct);
 var
    X1, Y1: Integer;
    a: array[0..255] of char;
    r: TRect;
    P1, P2: TPoint;
    Dummy: TMessage;
    TempRgn: HRgn;
begin
{  TWindow.Paint(PaintDC, PaintInfo);}
  if CaretVis then
    HideCaret(HWindow);
  if (OldXPos=Scroller^.XPos) and (OldYPos=Scroller^.YPos) then
  begin
    if (TW1.X <> OldTW1.X) or (TW1.Y <> OldTW1.Y) then
    begin
      OldTW1 := TW1;
      OldXPos := -2;
    end;
    if (TP1.X <> OldTP1.X) or (TP1.Y <> OldTP1.Y) then
    begin
      OldTW1 := TW1;
      OldXPos := -2;
    end;
  end;

  if (OldXPos=-1) then
  begin
    OldXPos := Scroller^.XPos;
    OldYPos := Scroller^.YPos;
    SetRect(r, 0, 0, xw, yw);
    FillRect(BufferDC, r, TheBkBrush);
    FillRect(vhDC, r, TheBkBrush);
    FillRect(PaintDC, r, TheBkBrush);
  end;

  if (OldXPos<>Scroller^.XPos) or (OldYPos<>Scroller^.YPos) then
  begin
     with PaintInfo.rcPaint do
     begin
       if OldXPos<>-2 then
       begin
         X1 := Scroller^.XPos-OldXPos;
         Y1 := Scroller^.YPos-OldYPos;
         SetZoom(X1, Y1, xw+X1, yw+Y1);
         SetRect(PaintInfo.rcPaint, 0, 0, xw, yw);
         if Y1=0 then
         begin
           if X1>0 then
             Left := xw-x1
           else
             Right := -x1;
         end
         else
         begin
           if Y1>0 then
             Top := yw-y1
           else
             Bottom := -y1;  
         end;

         P1 := TW1;
         P2 := TW2;
         ScreenToClient(HWindow, P1);
         ScreenToClient(HWindow, P2);
         if (P2.X>=0) and (P2.Y>=0) and (P1.X<=xw) and (P1.Y<=yw) then
         begin
           P1.X := P1.X-X1;
           P1.Y := P1.Y-Y1;
           P2.X := P2.X-X1;
           P2.Y := P2.Y-Y1;
           if (P2.X>=0) and (P2.Y>=0) and (P1.X<=xw) and (P1.Y<=yw) then
           begin
             DisableWindowTime;
             World.Redraw( InvX(P1.X),
                           InvY(P2.Y),
                           InvX(P2.X),
                           InvY(P1.Y));
             TempRgn := CreateRectRgn(P1.X, P1.Y, P2.X+1, P2.Y+1);
             SelectClipRgn(vhDC, TempRgn);
             DeleteObject(TempRgn);
             World.ShowFrame;
           end;
         end;

         P1 := TP1;
         P2 := TP2;
         ScreenToClient(HWindow, P1);
         ScreenToClient(HWindow, P2);
         if (P2.X>=0) and (P2.Y>=0) and (P1.X<=xw) and (P1.Y<=yw) then
         begin
           P1.X := P1.X-X1;
           P1.Y := P1.Y-Y1;
           P2.X := P2.X-X1;
           P2.Y := P2.Y-Y1;
           if (P2.X>=0) and (P2.Y>=0) and (P1.X<=xw) and (P1.Y<=yw) then
           begin
             DisableWindowTime;
             World.Redraw( InvX(P1.X),
                           InvY(P2.Y),
                           InvX(P2.X),
                           InvY(P1.Y));
             TempRgn := CreateRectRgn(P1.X, P1.Y, P2.X+1, P2.Y+1);
             SelectClipRgn(vhDC, TempRgn);
             DeleteObject(TempRgn);
             World.ShowFrame;
           end;
         end;
       end;

       OldXPos := Scroller^.XPos;
       OldYPos := Scroller^.YPos;
       DisableWindowTime;
       World.Redraw( InvX(Left),
                     InvY(Bottom),
                     InvX(Right),
                     InvY(Top));
       TempRgn := CreateRectRgn(Left, Top, Right+1, Bottom+1);
       SelectClipRgn(vhDC, TempRgn);
       DeleteObject(TempRgn);
       World.ShowFrame;
       EnableWindowTime;
       TempRgn := CreateRectRgn(0, 0, xw, yw);
       SelectClipRgn(vhDC, TempRgn);
       DeleteObject(TempRgn);
{
       MessageString(HWindow, 'ru=('+NToS(Right)+','+NToS(Bottom)+')',
                              'lo=('+NToS(Left)+','+NToS(Top)+')', mb_OK);
}
     end;
     cmSaveBitmap(Dummy);
  end
  else
    BitBlt(vHDC,0,0,xw,yw,BufferDC,0,0,SrcCopy);
  if CaretVis then
    ShowCaret(HWindow);
end;

procedure TDrawing.Activate;
 var
    MSG: TMessage;
begin
  Deactivated := False;
  WMSetCursor(MSG);
end;

procedure TDrawing.Deactivate;
 var
    MSG: TMessage;
begin
  Deactivated := True;
  WMSetCursor(MSG);
end;

procedure TDrawing.CMSaveBitmap(VAR MSG: TMessage);
begin
  if CaretVis then
    HideCaret(HWindow);
  BitBlt(BufferDC, 0, 0, xw, yw, vhDC, 0,0, SrcCopy);
  if CaretVis then
    ShowCaret(HWindow);
end;

procedure TDrawing.CMClearBitmap(VAR MSG: TMessage);
 var
    r: TRect;
begin
  SetRect(r, 0, 0, xw, yw);
  if CaretVis then
    HideCaret(HWindow);
  FillRect(vhDC, r, TheBkBrush);
  FillRect(BufferDC, r, TheBkBrush);
  SendMessage(HWindow, wm_Paint, 0, 0);
  if CaretVis then
    ShowCaret(HWindow);
end;

procedure TDrawing.CMSetScrollBars(VAR MSG: TMessage);
 var
    SizeX, SizeY, X, Y: LongInt;
    Buffer: TSizeInfo;
begin
  GetMaxSize(@Buffer);
  SizeX := zx(Buffer.X2)-zx(Buffer.X1);
  SizeY := zy(Buffer.Y1)-zy(Buffer.Y2);
  if SizeX<xw then
    SizeX := xw;
  if SizeY<yw then
    SizeY :=yw;
  X := -zx(Buffer.X1)+(xw div 2);
  Y := -zy(Buffer.Y2)+(yw div 2);
  if X<0 then
    X := 0;
  if Y<0 then
    Y := 0;
  if SizeX<=X then
    SizeX := X+1;
  if SizeY<=Y then
    SizeY := Y+1;
  OldXPos := -1;
  Scroller^.SetRange(SizeX, SizeY);
  OldXPos := -1;
  Scroller^.ScrollTo(X,Y);
  cmClearBitmap(Msg);
end;


Procedure TDrawing.wmSize(VAR MSG: TMessage);
begin
  if vhDC<>0 then
  begin
    ExitDCs;
    TWindow.wmSize(Msg);
    InitDCs;
    SetZoom(0, 0, OldXw, OldYw);
    OldXw := xw;
    OldYw := yw;
    cmSetScrollBars(Msg);
    DisableWindowTime;
    World.DrawAll;
    World.ShowFrame;
    cmSaveBitmap(Msg);
    EnableWindowTime;
  end;
end;

Procedure TDrawing.wmSetCursor(VAR MSG: TMessage);
var
  Pt: TPoint;			{ Cursor position }
  R: TRect;			{ Window client area }
begin
  GetCursorPos(Pt);		{ In global coordinates }
  ScreenToClient(HWindow, Pt);  { In window client local coordinates }
  GetClientRect(HWindow, R);
  if Deactivated or not(PtInRect(R, Pt)) then
    SetCursor(LoadCursor(0, idc_Arrow))
  else
    SetCursor(MyCursor);
end;


 
procedure TDrawing.WMRButtonDown(var Msg: TMessage);
begin
  if not Deactivated then
  begin
    Deactivate;
    MouseX := Integer(Msg.LParamLo);
    MouseY := Integer(Msg.LParamHi);
    Tasten := Msg.WParam;
    Msg.Result := SendMessage(MsgWindow, wm_command, cm_DrawingRButtonDown, 0);
    Activate;
  end;
end;

procedure TDrawing.WMLButtonDown(var Msg: TMessage);
begin
  if not Deactivated then
  begin
    SetCapture(HWindow);
    LeftButtonPressed := True;
    MouseX := Integer(Msg.LParamLo);
    MouseY := Integer(Msg.LParamHi);
    Tasten := Msg.WParam;
    Msg.Result := SendMessage(MsgWindow, wm_command, cm_DrawingLButtonDown, 0);
  end;
end;

procedure TDrawing.WMMouseMove(var Msg: TMessage);
begin
  if not Deactivated then
  begin
    MouseX := Integer(Msg.LParamLo);
    MouseY := Integer(Msg.LParamHi);
    Tasten := Msg.WParam;
    Msg.Result := SendMessage(MsgWindow, wm_command, cm_DrawingMouseMove, 0);
  end;
end;

procedure TDrawing.WMLButtonUp(var Msg: TMessage);
begin
  if not Deactivated then
  begin
    Deactivate;
    LeftButtonPressed := False;
    MouseX := Integer(Msg.LParamLo);
    MouseY := Integer(Msg.LParamHi);
    Tasten := Msg.WParam;
    Msg.Result := SendMessage(MsgWindow, wm_command, cm_DrawingLButtonUp, 0);
    ReleaseCapture;
    Activate;
  end;
end;


end.
