unit Fonts;

interface

Uses {WObjects}Objects, WinTypes, WinProcs, Strings, Win31, CommDlg,
     GrTools, Bezier1;



type
    PFont = ^TFontClass;
    TFontClass = object(TObject)
      CurrentFont: HFont;
      FontFaktor: Real;
      LogFont: TLogFont;
      Constructor Init;
      Procedure   SelectFont;                       virtual;
      Function    GetLetter(Nr: Integer; var W: Real): PMultiGroup; virtual;
      Function    GetHeight: Real;                  virtual;
      Destructor  Done;                             virtual;
    end;  

var
    Font: TFontClass;

implementation

uses Defs, Dialogs;

Constructor TFontClass.Init;
BEGIN
  TObject.Init;
  FillChar(LogFont, SizeOf(TLogFont), #0);
  with LogFont do
  begin
    lfHeight        := -96;
    lfWeight        := FW_NORMAL;
    lfOutPrecision  := Out_TT_Only_Precis; {Out_Stroke_Precise;}
    lfClipPrecision := Clip_TT_Always; {Clip_Stroke_Precis;}
    lfQuality       := Default_Quality;
    lfPitchAndFamily:= {tmpf_TrueType;} Variable_Pitch;
    StrCopy(lfFaceName, 'Arial');
  end;
  CurrentFont := CreateFontIndirect(LogFont);
  FontFaktor := 0.25;
END;

Procedure TFontClass.SelectFont;
begin
  LogFont.lfHeight := trunc(-96.0*FontFaktor);
  if FontDialog(0, LogFont) then
  begin
    FontFaktor := LogFont.lfHeight/-96;
    LogFont.lfHeight := -96;
    DeleteObject(CurrentFont);
    CurrentFont := CreateFontIndirect(LogFont);
  end;
end;
                       
Function TFontClass.GetLetter(Nr: Integer; var W: Real): PMultiGroup;
 const
    DFaktor = 1.5;
 type
    DataArray= array[0..8000] of Integer;
 var
    IDMat: TMat2;
    MetricBuffer: TGlyphMetrics;
    DataPtr: ^DataArray;
    Anz: LongInt;
    i, Pos, n, MaxI: Integer;
    StartX, StartY, LastX, LastY, MX, MY, NewX, NewY,
    DX, DY, OldMX, OldMY, OldDX, OldDY: Real;
    OldLastX, OldLastY: Real;
    OldFont: HFont;
    SubGroupPtr: PGroup;
    MGroupPtr: PMultiGroup;
begin
  W := 0;
  MGroupPtr := nil;

  IDMat.eM11.Value := 1;
  IDMat.eM11.Fract := 0;
  IDMat.eM12.Value := 0;
  IDMat.eM12.Fract := 0;
  IDMat.eM21.Value := 0;
  IDMat.eM21.Fract := 0;
  IDMat.eM22.Value := 1;
  IDMat.eM22.Fract := 0;

  OldFont := SelectObject(vhDC, CurrentFont);

  Anz := GetGlyphOutline( vhDC,
                          Nr,
                          GGO_NATIVE,
                          MetricBuffer,
                          0,
                          nil,
                          IDMat);

  W := MetricBuffer.gmCellIncX*FontFaktor;
{  Message(NToS(MetricBuffer.gmptGlyphOrigin.X)+','+NToS(MetricBuffer.gmptGlyphOrigin.X));}

  if (Anz<=0) or (Anz>16000) then
  begin
    GetLetter := nil;
    exit;
  end;

  GetMem(DataPtr, Anz);

  Anz := GetGlyphOutline( vhDC,
                          Nr,
                          GGO_NATIVE,
                          MetricBuffer,
                          Anz,
                          PChar(DataPtr),
                          IDMat);

{  TextOut(vhDC, 100, 100, 'ABC', 3);}

  SelectObject(vhDC, OldFont);

  if Anz>0 then
  begin
    MGroupPtr := New(PMultiGroup, Init(Colr[CurrentColor], Colr[CurrentFillColor]));
    Pos := 0;
    while Pos<(Anz div 2) do
    begin
      SubGroupPtr := New(PGroup, Init(Colr[CurrentColor], Colr[CurrentFillColor]));
      n := Pos+(DataPtr^[Pos] div 2);
      Pos := Pos+4;
      StartX := 1.0*DataPtr^[Pos+1]+word(DataPtr^[Pos])/65536.0;
      StartY := 1.0*DataPtr^[Pos+3]+word(DataPtr^[Pos+2])/65536.0;
      LastX := StartX;
      LastY := StartY;
      Pos := Pos+4;
      while Pos<n do
      begin
        if DataPtr^[Pos] = TT_Prim_Line then
        begin
          Pos := Pos+1;
          for i := 1 to DataPtr^[Pos] do
          begin
            NewX := 1.0*DataPtr^[Pos+2]+word(DataPtr^[Pos+1])/65536.0;
            NewY := 1.0*DataPtr^[Pos+4]+word(DataPtr^[Pos+3])/65536.0;
            SubGroupPtr^.AddObject(New(PLine, Init(LastX, LastY, NewX, NewY)));
            SubGroupPtr^.GetObject(SubGroupPtr^.Count-1)^.Color := Colr[CurrentColor];
            LastX := NewX;
            LastY := NewY;
            Pos := Pos+4;
          end;
        end
        else
{NEW (Catmull-Rom)
        if DataPtr^[Pos] = TT_Prim_QSpline then
        begin
          Pos := Pos+1;
          MaxI := DataPtr^[Pos];
          for i := 1 to MaxI do
          begin
            NewX := 1.0*DataPtr^[Pos+2]+Word(DataPtr^[Pos+1])/65536.0;
            NewY := 1.0*DataPtr^[Pos+4]+Word(DataPtr^[Pos+3])/65536.0;
            if i=MaxI then
            begin
              if i=1 then
              begin
                SubGroupPtr^.AddObject(New(PLine, Init(LastX, LastY, NewX, NewY)));
                SubGroupPtr^.GetObject(SubGroupPtr^.Count-1)^.Color := rgb(0,0,0);
              end
              else
              begin
                DX := (NewX-OldLastX)/DFaktor;
                DY := (NewY-OldLastY)/DFaktor;
                SubGroupPtr^.AddObject(New(PBezier,
                             Init( OldLastX,       OldLastY,
                                   OldLastX+OldDX, OldLastY+OldDY,
                                   LastX-DX,       LastY-DY,
                                   LastX,          LastY)));
                SubGroupPtr^.GetObject(SubGroupPtr^.Count-1)^.Color := rgb(0,0,0);
                OldDX := DX;
                OldDY := DY;
                DX := (NewX-LastX)/DFaktor;
                DY := (NewY-LastY)/DFaktor;
                SubGroupPtr^.AddObject(New(PBezier,
                             Init( LastX,          LastY,
                                   LastX+OldDX,    LastY+OldDY,
                                   NewX-DX,        NewY-DY,
                                   NewX,           NewY)));
                SubGroupPtr^.GetObject(SubGroupPtr^.Count-1)^.Color := rgb(0,0,0);
              end;
            end
            else
            if i=1 then
            begin
              DX := (NewX-LastX)/DFaktor;
              DY := (NewY-LastY)/DFaktor;
            end
            else
            begin
              DX := (NewX-OldLastX)/DFaktor;
              DY := (NewY-OldLastY)/DFaktor;
              SubGroupPtr^.AddObject(New(PBezier,
                           Init( OldLastX,       OldLastY,
                                 OldLastX+OldDX, OldLastY+OldDY,
                                 LastX-DX,       LastY-Dy,
                                 LastX,          LastY)));
              SubGroupPtr^.GetObject(SubGroupPtr^.Count-1)^.Color := rgb(0,0,0);
            end;
            OldDX := DX;
            OldDY := DY;
            OldLastX := LastX;
            OldLastY := LastY;
            LastX := NewX;
            LastY := NewY;
            Pos := Pos+4;
          end;
        end
        else
{OLD QSpline?}
        if DataPtr^[Pos] = TT_Prim_QSpline then
        begin
          Pos := Pos+1;
          MaxI := DataPtr^[Pos];
          for i := 1 to MaxI do
          begin
            NewX := 1.0*DataPtr^[Pos+2]+word(DataPtr^[Pos+1])/65536.0;
            NewY := 1.0*DataPtr^[Pos+4]+word(DataPtr^[Pos+3])/65536.0;
            if i=MaxI then
            begin
              if i=1 then
              begin
                SubGroupPtr^.AddObject(New(PLine, Init(LastX, LastY, NewX, NewY)));
                SubGroupPtr^.GetObject(SubGroupPtr^.Count-1)^.Color := Colr[CurrentColor]
              end
              else
              begin
                MX := NewX;
                MY := NewY;
                DX := (NewX-LastX)/DFaktor;
                DY := (NewY-LastY)/DFaktor;
                SubGroupPtr^.AddObject(New(PBezier,
                      Init( OldMX,       OldMY,
                            OldMX+OldDX, OldMY+OldDY,
                            MX-DX,       MY-DY,
                            MX,          MY)));
                SubGroupPtr^.GetObject(SubGroupPtr^.Count-1)^.Color := Colr[CurrentColor]
              end;
            end
            else if i=1 then
            begin
              MX := LastX;
              MY := LastY;
              DX := (NewX-MX)/DFaktor;
              DY := (NewY-MY)/DFaktor;
            end
            else
            begin
              MX := (NewX+LastX)/2;
              MY := (NewY+LastY)/2;
              DX := (NewX-MX)/DFaktor;
              DY := (NewY-MY)/DFaktor;
              SubGroupPtr^.AddObject(New(PBezier,
                    Init( OldMX,       OldMY,
                          OldMX+OldDX, OldMY+OldDY,
                          MX-DX,       MY-DY,
                          MX,          MY)));
              SubGroupPtr^.GetObject(SubGroupPtr^.Count-1)^.Color := Colr[CurrentColor]
            end;
            Pos := Pos+4;
            OldMX := MX;
            OldMY := MY;
            OldDX := DX;
            OldDY := DY;
            LastX := NewX;
            LastY := NewY;
          end;
        end
        else
          Message('Fehlerhafte Daten: '+NToS(DataPtr^[Pos]));
        Pos := Pos+1;
      end;
      if (LastX<>StartX) or (LastY<>StartY) then
      begin
        SubGroupPtr^.AddObject(New(PLine, Init(LastX, LastY, StartX, StartY)));
        SubGroupPtr^.GetObject(SubGroupPtr^.Count-1)^.Color := Colr[CurrentColor]
      end;
      if SubGroupPtr^.Count>0 then
      begin
        SubGroupPtr^.CalcFrame;
        SubGroupPtr^.Sort;
        MGroupPtr^.AddObject(SubGroupPtr);
      end;
    end;
    if MGroupPtr^.Count = 0 then
    begin
      Dispose(MGroupPtr, Done);
      MGroupPtr := nil;
    end
    else
    if MGroupPtr^.Count = 1 then
    begin
      SubGroupPtr := PGroup(MGroupPtr^.GetObject(0));
      MGroupPtr^.RemoveAll;
      Dispose(MGroupPtr, Done);
      MGroupPtr := PMultiGroup(SubGroupPtr);
    end
    else
    begin
      MGroupPtr^.CalcFrame;
    end;
    FreeMem(DataPtr, Anz);
  end;
  if MGroupPtr<>nil then
  begin
    MGroupPtr^.Stretch(FontFaktor, FontFaktor);
  end;
  GetLetter := MGroupPtr;
end;

Function TFontClass.GetHeight: Real;
begin
  GetHeight := 96.0*FontFaktor;
end;

Destructor TFontClass.Done;
BEGIN
  DeleteObject(CurrentFont);
  TObject.Done;
END;

END.


