{************************************************}
{                                                }
{   Turbo Pascal for Windows: Paint Demo         }
{   ToolBar unit                                 }
{   Copyright (c) 1992 by Borland International  }
{                                                }
{************************************************}

unit MyToolBar;

{ This unit defines a tool bar window for the paint program.
  The toolbar is responsible for the management of the available tools
  and displays the icons for the available tools. Selection of the current
  tool is handled here.
}  

interface

uses Tools, Drawing, Status, Defs,
     WinTypes, WinProcs, {WObjects}Objects, OWindows;

type
  { All available tools }
  ToolName = ( SnapTool, BezierTool, ZoomTool, SelectTool,
               LineTool, RectTool, CircleTool, TextTool);
{               RotateTool, HMirrorTool, VMirrorTool);}

const
  MinTool = SnapTool;
  MaxTool = TextTool;

type
  PToolBar = ^TToolBar;
  TToolBar = object(TWindow)
    Drawing: PDrawing;
    StateLine: PStateLine;
    CurrentTool: ToolName;
    Tools: array[ToolName] of PTool;

    { Creation and destruction }
    constructor Init(AParent: PWindowsObject; ADrawing: PDrawing; AStateLine: PStateLine);
    procedure   InitTools; virtual;
    destructor Done; virtual;

    { Utility }
    procedure ToolSelect(Tool: ToolName);

    { Display } 
    procedure Paint(PaintDC: HDC; var PaintInfo: TPaintStruct); virtual;

    { Window manager responses }
    procedure WMLButtonDown(var Msg: TMessage);
      virtual wm_First + wm_LButtonDown;

    { commands from Drawing}
    procedure CMDrawingRButtonDown(var Msg: TMessage);
      virtual cm_First + cm_DrawingRButtonDown;
    procedure CMDrawingLButtonDown(var Msg: TMessage);
      virtual cm_First + cm_DrawingLButtonDown;
    procedure CMDrawingMouseMove(var Msg: TMessage);
      virtual cm_First + cm_DrawingMouseMove;
    procedure CMDrawingLButtonUp(var Msg: TMessage);
      virtual cm_First + cm_DrawingLButtonUp;

    { commands from MyWindow}
    procedure CMMyWindowChar(var Msg: TMessage);
      virtual cm_First + cm_MyWindowChar;

  end;

implementation

uses Bezier2, GrTools;

{ Create the actual toolbar and a new instance of each tool it is to contain.
}
constructor TToolBar.Init(AParent: PWindowsObject; ADrawing: PDrawing; AStateLine: PStateLine);
begin
  TWindow.Init(AParent, nil);
  Attr.Style := ws_Child or ws_Visible;
  Drawing := ADrawing;
  StateLine := AStateLine;
  ToolsActive := False;
end;

procedure TToolBar.InitTools;
begin
  Tools[SnapTool] := new(PSnap, init(Drawing^.HWindow));
  Tools[BezierTool] := new(PBezierTool, init(Drawing^.HWindow));
  Tools[ZoomTool] := new(PZoom, init(Drawing^.HWindow));
  Tools[SelectTool] := new(PSelect, init(Drawing^.HWindow));
  Tools[LineTool] := new(PLineTool, init(Drawing^.HWindow));
  Tools[RectTool] := new(PRectTool, init(Drawing^.HWindow));
  Tools[CircleTool] := new(PCircleTool, init(Drawing^.HWindow));
  Tools[TextTool] := new(PTextTool, init(Drawing^.HWindow));
{
  Tools[RotateTool] := new(PRotateTool, init(Drawing^.HWindow));
  Tools[HMirrorTool] := new(PHMirrorTool, init(Drawing^.HWindow));
  Tools[VMirrorTool] := new(PVMirrorTool, init(Drawing^.HWindow));
}
  CurrentTool := MinTool;
  ToolSelect(MinTool);
  ToolsActive := False;
end;

{ Destroy each tool instance befory dying.
}
destructor TToolBar.Done;
var
  Tool: ToolName;
begin
  for Tool := MinTool to MaxTool do Dispose(Tools[Tool], Done);
  TWindow.Done;
end;

{ Deselect the current tool and select the new tool and update the display.
}
procedure TToolBar.ToolSelect(Tool: ToolName);
 var
    OldCurrentTool: ToolName;
begin
  if not ToolsActive then
  begin
    ToolsActive := True;
    ToolsBusy := True;
    OldCurrentTool := CurrentTool;
    Tools[CurrentTool]^.DeselectTool;
    CurrentTool := Tool;
    if Tools[CurrentTool]^.Cursor <> 0 then
      Drawing^.MyCursor := Tools[CurrentTool]^.Cursor;
    InvalidateRect(HWindow, nil, False);
    if not Tools[CurrentTool]^.SelectTool then
    begin
      ToolsActive := False;
      ToolSelect(OldCurrentTool);
    end;
    ToolsActive := False;
  end;
end;

{ Paint the toolbar by painting the icon for each tool horizontally. The icon
  for the currently selected tool is highlighted. Note that icons are 32x32,
  but overlap by one pixel.
}
procedure TToolBar.Paint(PaintDC: HDC; var PaintInfo: TPaintStruct);
var
  I: Integer;		{ Position of tool in row of icons }
  Tool: ToolName;	{ Current tool being drawn }
  R: TRect;		{ Coordinates of icon }
begin
  for Tool := MinTool to MaxTool do
  begin
    I := Ord(Tool);
    DrawIcon(PaintDC, I * 31, 0, Tools[Tool]^.Icon);
    
    { Highlight currently selected tool }
    if Tool = CurrentTool then
    begin
      R.top := 1;
      R.left := I * 31 + 1;
      R.bottom := R.top + 30;
      R.right := r.left + 30;
      InvertRect(PaintDC, R);
    end;
  end;
end;

{ Select the tool whose icon is pressed.
}
procedure TToolBar.WMLButtonDown(var Msg: TMessage);
var
  Tool: ToolName;
begin
  Tool := ToolName(Msg.LParamLo div 31);
  if Tool <= MaxTool then ToolSelect(Tool);
end;

procedure TToolBar.CMDrawingRButtonDown(var MSG: TMessage);
 var
    r: TRect;
begin
  if not ToolsActive then
  begin
    ToolsActive := True;
    ToolsBusy := True;
    World.ShowFrame;
    ResetZoom;
    SendMessage(Drawing^.HWindow, wm_Command, cm_SetScrollbars, 0);
    World.DrawAll;
    World.ShowFrame;
    SendMessage(Drawing^.HWindow, wm_Command, cm_SaveBitmap, 0);
    ToolsActive := False;
  end;
end;

procedure TToolBar.CMDrawingLButtonDown(var MSG: TMessage);
begin
  if not ToolsActive then
  begin
    ToolsActive := True;
    ToolsBusy := True;
    World.ShowFrame;
    SendMessage(Drawing^.HWindow, wm_Command, cm_SaveBitmap, 0);
    Tools[CurrentTool]^.MouseDown( Drawing^.MouseX,
                                   Drawing^.MouseY,
                                   Drawing^.Tasten);
    ToolsBusy := False;
  end;
end;

procedure TToolBar.CMDrawingMouseMove(var MSG: TMessage);
 var S1, S2: String;
begin
  str(SnapX(Drawing^.MouseX):7:3,S1);
  str(SnapY(Drawing^.MouseY):7:3,S2);
  StateLine^.SetText('X: '+S1+'    Y:'+S2);
  if ToolsActive and not ToolsBusy then
  begin
    ToolsBusy := True;
    Tools[CurrentTool]^.MouseMove( Drawing^.MouseX,
                                   Drawing^.MouseY,
                                   Drawing^.Tasten);
    ToolsBusy := False;
  end;
end;

procedure TToolBar.CMDrawingLButtonUp(var MSG: TMessage);
 var
    r: TRect;
begin
  if ToolsActive then
  begin
    if ToolsBusy then
      PostMessage(HWindow, wm_Command, cm_DrawingLButtonUp, 0)
    else
    begin
      ToolsBusy := True;
      Tools[CurrentTool]^.MouseUp( Drawing^.MouseX,
                                   Drawing^.MouseY,
                                   Drawing^.Tasten);
      World.ShowFrame;
      SendMessage(Drawing^.HWindow, wm_Command, cm_SaveBitmap, 0);
      ToolsActive := False;
      sendMessage(Parent^.HWindow, wm_Command, cm_SetMenu, 0);
    end;
  end;
end;

procedure TToolBar.CMMyWindowChar(var Msg: TMessage);
begin
  if (not ToolsActive) then
  begin
    ToolsActive := True;
    ToolsBusy := True;
    World.ShowFrame;
    Tools[CurrentTool]^.Char( Msg.LParamHi);
    World.ShowFrame;
    SendMessage(Drawing^.HWindow, wm_Command, cm_SaveBitmap, 0);
    ToolsActive := False;
    sendMessage(Parent^.HWindow, wm_Command, cm_SetMenu, 0);
  end;
end;

end.

