unit poly;

{ $N+}

interface

uses {wobjects}objects;


type
    koeff= array[0..3] of real;

    PPolynom = ^PolynomClass;
    PolynomClass = Object(TObject)
      Grad: Integer;
      a: Koeff;
      Constructor Init(r1, r2, r3, r4: Real);
      Constructor CreateCopy(PolyPtr: PPolynom);
      Procedure   Assign(r1, r2, r3, r4: Real);                        virtual;
      Procedure   CalcGrad;                                            virtual;
      Function    f(x: Real): Real;                                    virtual;
      Procedure   Ableiten;                                            virtual;
      Procedure   Neg;                                                 virtual;
      Procedure   Add(PolyPtr: PPolynom);                              virtual;
      Procedure   Sub(PolyPtr: PPolynom);                              virtual;
      Procedure   Combine1(PolyPtr: PPolynom);                          virtual;
      Procedure   Invert1;                                              virtual;
      Function    SucheNull(var ug, og: real): boolean;                virtual;
      Function    NullStellen(var n: Koeff; ug, og: Real): Integer;    virtual;
      Destructor  Done;                                                virtual;
    end;

implementation


{------------- HILFSFUNKTIONEN --------------------------------------}

function sgn(x: Real): Integer;
begin
    if x=0 then
        sgn := 0
    else if x<0 then
        sgn := -1
    else
        sgn := 1;
end;

{------------- POLYNOMCLASS -----------------------------------------}

Constructor PolynomClass.Init(r1, r2, r3, r4: Real);
begin
    Assign(r1, r2, r3, r4);
end;

Constructor PolynomClass.CreateCopy(PolyPtr: PPolynom);
begin
    Assign(PolyPtr^.a[3],PolyPtr^.a[2],PolyPtr^.a[1],PolyPtr^.a[0]);
end;

Procedure PolynomClass.Assign(r1, r2, r3, r4: Real);
begin
    a[3] := r1;
    a[2] := r2;
    a[1] := r3;
    a[0] := r4;
    CalcGrad;
end;

Procedure PolynomClass.CalcGrad;
begin
    Grad := 3;
    while (Grad>0) and (a[Grad]=0) do
        Grad := Grad-1;
end;
procedure PolynomClass.Ableiten;
begin
  assign(0, 3*a[3], 2*a[2], a[1]);
end;

Function PolynomClass.NullStellen(var n: Koeff; ug, og: Real): Integer;
 var
    Anz, AnzEx: Integer;
    p, q, w,h : real;
    ex: Koeff;
    Poly: PolynomClass;

  procedure AddN(r: Real);
  begin
      if (r>=ug) and (r<=og) then
      begin
          n[Anz] := r;
          Anz := Anz+1;
      end;
  end;

begin
    Anz := 0;
    Case Grad of
      0 : if a[0] = 0 then
              AddN((ug+og)/2);           { beliebig }
      1 : AddN(-a[0]/a[1]);
      2 : begin
              p := a[1]/a[2];
              q := a[0]/a[2];
              w := sqr(p/2)-q;
              if w=0 then
                  AddN(-p/2)
              else if w>0 then
              begin
                  w := sqrt(w);
                  AddN(-p/2-w);
                  AddN(-p/2+w);
              end
          end;
      3 : begin
              Poly.CreateCopy(@Self);
              Poly.Ableiten;
              AnzEx := Poly.NullStellen(ex, ug, og);
              Poly.Done;
              case AnzEx of
                0 : if SucheNull(ug, og) then
                        AddN((ug+og)/2);
                1 : begin
                        h := ex[0];
                        if SucheNull(ug, h) then
                            AddN((ug+h)/2);
                        if SucheNull(ex[0], og) then
                            AddN((ex[0]+og)/2);
                    end;
                2 : begin
                        h := ex[0];
                        if SucheNull(ug, h) then
                            AddN((ug+h)/2);
                        h := ex[1];
                        if SucheNull(ex[0], h) then
                            AddN((ex[0]+h)/2);
                        if SucheNull(ex[1], og) then
                            AddN((ex[1]+og)/2);
                    end;
              end;
          end;
    end;
    NullStellen := Anz;
end;

Function PolynomClass.f(x: Real): Real;
 var
    y: real;
    i: integer;
begin
    y := a[Grad];
    for i := Grad-1 downto 0 do
    begin
        y := y*x;
        y := y+a[i];
    end;
    f := y;
end;


Function PolynomClass.SucheNull(var ug, og: Real): Boolean;
 var yu, yo, ym, Mitte: Real;
     su, so: Integer;
     Quit: Boolean;
begin
    yu := f(ug);
    yo := f(og);
    su := sgn(yu);
    so := sgn(yo);
    if yu =0 then
    begin
        SucheNull := True;
        og := ug;
    end
    else if yo =0 then
    begin
        SucheNull := True;
        ug := og;
    end
    else if su=so then
        SucheNull := False
    else
    begin
        SucheNull := True;
        Quit := False;
        repeat
            Mitte := (ug+og)/2;
            ym := f(Mitte);
            if (ug=Mitte) or (og=Mitte) then
                Quit := True;
            if ym=0 then
            begin
                ug := Mitte;
                og := Mitte;
                Quit := True;
            end
            else
            begin
                if sgn(ym)=su then
                begin
                    yu := ym;
                    ug := Mitte;
                end
                else
                begin
                    yo := ym;
                    og := Mitte;
                end;
                if (ug=og) then
                    Quit := True;
            end;
        until Quit;
    end;
end;

Procedure PolynomClass.Neg;
 var
    i: Integer;
begin
    for i := 0 to Grad do
        a[i] := -a[i];
end;

Procedure PolynomClass.Add(PolyPtr: PPolynom);
 var
    i: Integer;
begin
    if PolyPtr^.Grad>Grad then
        Grad := PolyPtr^.Grad;
    for i := 0 to Grad do
        a[i] := a[i]+PolyPtr^.a[i];
    CalcGrad;
end;

Procedure PolynomClass.Sub(PolyPtr: PPolynom);
begin
    Neg;
    Add(PolyPtr);
    Neg;
end;

Procedure PolynomClass.Combine1(PolyPtr: PPolynom);
 var
    i: integer;
begin
    if PolyPtr^.Grad=1 then
    begin
        for i := 0 to Grad do
            a[i] := a[i]*PolyPtr^.a[1];
        a[0] := a[0]+PolyPtr^.a[0];
    end;
    CalcGrad;
end;

Procedure PolynomClass.Invert1;
 var
    r1, r0: Real;
begin
    if Grad=1 then
    begin
        r1 := a[1];
        r0 := a[0];
        a[1] := 1/r1;
        a[0] := -r0/r1;
    end;
end;

Destructor PolynomClass.Done;
begin
end;

{--------------------------------------------------------------------}


begin
end.

{-- BEISPIEL --}

procedure RandPoly(PolyPtr: PPolynom; Grad: Integer);
 var
    a: Koeff;
    i: Integer;
begin
    for i := 0 to 3 do
      if i<=Grad then
          a[i] := (2.0*random-1)*50
      else
          a[i] := 0;
    while a[Grad]=0 do
      a[Grad] := (2.0*random-1)*50;
    PolyPtr^.Assign(a[3],a[2],a[1],a[0]);
end;



var PolyX, PolyY, LineX, LineY,
    HelpPoly1, HelpPoly2: PolynomClass;
    anz, i: Integer;
    n: Koeff;
    a, b, c, d,
    px, py, lx, ly, t1, t2: Real;

begin
  writeln;
  RandSeed := 0;

  PolyX.Init(0,0,0,0);
  PolyY.Init(0,0,0,0);
  LineX.Init(0,0,0,0);
  LineY.Init(0,0,0,0);
  HelpPoly1.Init(0,0,0,0);
  HelpPoly2.Init(0,0,0,0);

  repeat

    repeat
      RandPoly(@PolyX,3);
      RandPoly(@PolyY,3);
      RandPoly(@LineX,1);
      RandPoly(@LineY,1);

      HelpPoly1.CreateCopy(@PolyX);
      HelpPoly2.CreateCopy(@LineX);
      HelpPoly2.Invert1;
      HelpPoly1.Combine1(@HelpPoly2);
      HelpPoly1.Combine1(@LineY);
      HelpPoly1.Sub(@PolyY);

      Anz := HelpPoly1.NullStellen(n, 0, 1);
    until Anz<>0;

    a := HelpPoly1.a[3];
    b := HelpPoly1.a[2];
    c := HelpPoly1.a[1];
    d := HelpPoly1.a[0];

    writeln;
    writeln('Nullstellen von ',a:8:5,'*x^3+',b:8:5,'*x^2+',c:8:5,'*x+',d:8:5);
    writeln('Anzahl = ',Anz);
    for i := 0 to anz - 1 do
    begin
      t1 := n[i];
      px := PolyX.f(t1);
      py := PolyY.f(t1);
      t2 := HelpPoly2.f(px);
      lx := LineX.f(t2);
      ly := LineY.f(t2);

      writeln(i+1:10,'. bei ',n[i]:8:5,'    f = ',HelpPoly1.f(n[i]));
      writeln('  t1 = ',t1:8:5,':    px = ',px:8:5,',   py = ',py:8:5);
      writeln('  t2 = ',t2:8:5,':    lx = ',lx:8:5,',   ly = ',ly:8:5);
      writeln('  dx = ',px-lx,',   dy = ',py-ly);

    end;
  until readkey=#27;

  PolyX.Done;
  PolyY.Done;
  LineX.Done;
  LineY.Done;
  HelpPoly1.Done;
  HelpPoly2.Done;
end.