unit Polygone;

interface

uses WinTypes, WinProcs, {WObjects}Objects, BezDefs;

type
    PPolygon = ^TPolygonClass;
    TPolygonClass = object(TCollection)  {of PPunkt}
      MaxZoom: Real;
      Constructor Init;
      Procedure   AddPoint(X, Y: Real);               virtual;
      Procedure   AddAtPoint(n: Integer; X, Y: Real); virtual;
      Procedure   RemoveAll;                          virtual;
      Function    GetPoints(var PolyPtr: PPolyPoint): Integer; virtual;
      Procedure   GotPoint(i: Integer);               virtual;
      Function    GetRgn: HRgn;                       virtual;
      Procedure   Move(dx, dy: Real);                 virtual;
      Procedure   Stretch(xs, ys: Real);              virtual;
      procedure   Rot(NX1, NY1, NX2, NY2, MX, MY: Real); virtual;
      Destructor  Done;                               virtual;
    end;


    PPolyPolygon = ^TPolyPolygonClass;
    TPolyPolygonClass = object(TPolygonClass) 
      PolyAnzPtr, TempAnzPtr: PPolyAnz;
      LastP, MaxAnz, PAnz, TempAnz: Integer;
      Constructor Init(Anz: integer);
      Procedure   RemoveAll;                          virtual;
      Procedure   NewPoly;                            virtual;
      Function    GetPolyPoints(PolyPtr: PPolyPoint; var AnzPtr: PPolyAnz): Integer; virtual;
      Procedure   GotPoint(i: Integer);               virtual;
      Function    GetRgn: HRgn;                       virtual;
      Destructor  Done;                               virtual;
    end;


implementation

{uses Bezier2;}

{------------- TPOLYGONCLASS ------------------}

Constructor TPolygonClass.Init;
begin
  TCollection.Init(4,0);
  MaxZoom := 9999;
end;

Procedure TPolygonClass.AddPoint(X, Y: Real);
 var
    PktPtr: PPunkt;
begin
  if Count=Limit then
    SetLimit(Count+4);
{
  if Count>0 then
  begin
    PktPtr := At(Count-1);
    if (PktPtr^.X=X) and (PktPtr^.Y=Y) then
      exit;
  end;
}
  New(PktPtr);
  PktPtr^.X := X;
  PktPtr^.Y := Y;
  Insert(PktPtr);
end;

Procedure TPolygonClass.AddAtPoint(n: Integer; X, Y: Real);
 var
    PktPtr: PPunkt;
begin
  if Count=Limit then
    SetLimit(Count+4);
  New(PktPtr);
  PktPtr^.X := X;
  PktPtr^.Y := Y;
  AtInsert(n,PktPtr);
end;

Procedure TPolygonClass.RemoveAll;
 var
    i: Integer;
begin
  for i := 0 to Count-1 do
    Dispose(PPunkt(At(i)));
  DeleteAll;
end;

Function TPolygonClass.GetPoints(var PolyPtr: PPolyPoint): Integer;
 var
    i, j, LastI: Integer;
    Vis, Vis2: Word;
    PktPtr, Pkt2ptr: PPunkt;
begin
  Vis := 0;
  j := 0;
  LastI := -1;
  for i := 0 to Count-1 do
  if j<1000 then begin
    PktPtr := at(i);
    Vis2 := 0;
    if VisMin.X>PktPtr^.X then
      Vis2 := Vis2+1
    else if VisMax.X<PktPtr^.X then
      Vis2 := Vis2+2;
    if VisMin.Y>PktPtr^.Y then
      Vis2 := Vis2+4
    else if VisMax.Y<PktPtr^.Y then
      Vis2 := Vis2+8;
Vis2 := 0;
    if (Vis2=0) or (Vis2<>Vis) then
    begin
      if LastI<>i-1 then
      begin
        GotPoint(i-1);
        Pkt2Ptr := at(i-1);
        PolyPtr^[j].X := zx(Pkt2Ptr^.X);
        PolyPtr^[j].Y := zy(Pkt2Ptr^.Y);
        j := j+1;
      end;
      GotPoint(i);
      PolyPtr^[j].X := zx(PktPtr^.X);
      PolyPtr^[j].Y := zy(PktPtr^.Y);
      j := j+1;
      LastI := i;
      Vis := Vis2;
    end;
  end;
  GetPoints := j;
end;

Procedure TPolygonClass.GotPoint(i: Integer);
begin
end;               

Function TPolygonClass.GetRgn: HRgn;
 var
    Anz: Integer;
    PolyPtr: PPolyPoint;
begin
  new(PolyPtr);
  Anz := Getpoints(PolyPtr);
  if Anz>1 then
    GetRgn := CreatePolygonRgn(PolyPtr^, Anz, alternate)
  else
    GetRgn := 0;
  Dispose(PolyPtr);
end;


Procedure TPolygonClass.Move(dx, dy: Real);
 var
    i: Integer;
    PktPtr: PPunkt;
begin
  for i := 0 to Count-1 do
  begin
    PktPtr := At(i);
    PktPtr^.X := PktPtr^.X + dx;
    PktPtr^.Y := PktPtr^.Y + dy;
  end;
end;

Procedure TPolygonClass.Stretch(xs, ys: Real);
 var
    i: Integer;
    PktPtr: PPunkt;
begin
  for i := 0 to Count-1 do
  begin
    PktPtr := At(i);
    PktPtr^.X := PktPtr^.X * xs;
    PktPtr^.Y := PktPtr^.Y * ys;
  end;
end;

procedure TPolygonClass.Rot(NX1, NY1, NX2, NY2, MX, MY: Real);
 var
    i: Integer;
    PktPtr: PPunkt;
begin
  for i := 0 to Count-1 do
  begin
    PktPtr := at(i); 
    Trans(PktPtr^.X, PktPtr^.Y, NX1, NY1, NX2, NY2, MX, MY);
  end;
end;

Destructor TPolygonClass.Done;
begin
  RemoveAll;
  TCollection.Done;
end;

{------------ TPOLYPOLYGONCLASS ------------------}

Constructor TPolyPolygonClass.Init(Anz: integer);
begin
  TPolygonClass.Init;
  if Anz<4 then
    Anz := 4;
  MaxAnz := Anz;
  GetMem(PolyAnzPtr, MaxAnz*SizeOf(Integer));
  PAnz := 0;
  PolyAnzPtr^[0] := 0;
  TempAnzPtr := nil;
end;


Procedure TPolyPolygonClass.RemoveAll;
begin
  TPolygonClass.RemoveAll;
  PAnz := 0;
  PolyAnzPtr^[0] := 0;
end;
                          
Procedure TPolyPolygonClass.NewPoly;
 var
    i: Integer;
    NewAnzPtr: PPolyAnz;
begin
  if PAnz=MaxAnz-1 then
  begin
    GetMem(NewAnzPtr, (MaxAnz+4)*SizeOf(Integer));
    for i := 0 to MaxAnz-1 do
      NewAnzPtr^[i] := PolyAnzPtr^[i];
    FreeMem(PolyAnzPtr, MaxAnz*SizeOf(Integer));
    PolyAnzPtr := NewAnzPtr;
    MaxAnz := MaxAnz+4;
  end;
  PolyAnzPtr^[PAnz] := Count;
  PAnz := PAnz+1;
  PolyAnzPtr^[PAnz] := 0;
end;

                            
Function TPolyPolygonClass.GetPolyPoints(PolyPtr: PPolyPoint; var AnzPtr: PPolyAnz): Integer;
begin
  if TempAnzPtr<>nil then
    FreeMem(TempAnzPtr, TempAnz*SizeOf(Integer));
  TempAnz := PAnz+1;
  GetMem(TempAnzPtr, TempAnz*SizeOf(Integer));
  FillChar(TempAnzPtr^, TempAnz*SizeOf(Integer), #0);
  LastP := 0; 
  TPolygonClass.GetPoints(PolyPtr);
  GetPolyPoints := LastP+1;
  AnzPtr := TempAnzPtr;
end;

procedure TPolyPolygonClass.GotPoint(i: Integer);
begin
  while PolyAnzPtr^[LastP]<=i do
    LastP := LastP+1;
  TempAnzPtr^[LastP] := TempAnzPtr^[LastP]+1;
end;
 
Function TPolyPolygonClass.GetRgn: HRgn;                       
 var
    Anz: Integer;
    AnzPtr: PPolyAnz;
    PolyPtr: PPolyPoint;
begin
  new(PolyPtr);
  Anz := GetPolyPoints(PolyPtr, AnzPtr);
  if Anz>1 then
    GetRgn := CreatePolyPolygonRgn(PolyPtr^, AnzPtr^, Anz, alternate)
  else
    GetRgn := 0;
  Dispose(PolyPtr);
end;

Destructor TPolyPolygonClass.Done;                               
begin
  if TempAnzPtr<>nil then
    FreeMem(TempAnzPtr, TempAnz*SizeOf(Integer));
  FreeMem(PolyAnzPtr, MaxAnz*SizeOf(Integer));
  TPolygonClass.Done;
end;

end.
