unit Script;

interface

uses {WObjects}Objects;

const
  scr_Move         =  1;
  scr_Stretch      =  2;
  scr_Rot          =  3;
  scr_ToLine       =  4;
  scr_ToBez        =  5;
  scr_SelectAll    =  6;
  scr_DeselectAll  =  7;
  scr_Cut          =  8;
  scr_Copy         =  9;
  scr_Paste        = 10;
  scr_Group        = 11;
  scr_Ungroup      = 12;
  scr_SetPos       = 13;

var
  ScriptRecord,
  ScriptPlay: Boolean; 

type PScriptRec = ^ScriptRec;
     ScriptRec = Record
                   Aktion: Word;
                   Data: array[0..5] of real;
                 end;

     PScript = ^ScriptClass;
     ScriptClass = Object(TCollection) {of PScriptRec}
       CX1, CY1, CX2, CY2,
       FX1, FY1, FX2, FY2: Real;
       Faktor,
       XTx, XTy, YTx, YTy: Real;
       Constructor Init;
       Procedure   AddScript(Akt: Word; X1, X2, X3, X4, X5, X6: Real); virtual;
       Procedure   ClearScript;                              virtual;
       Procedure   GetData(i: Integer; var Data: ScriptRec); virtual;
       Procedure   SetPos(SX1, SY1, SX2, SY2: Real);         virtual;
       Procedure   StartRecord(SX1, SY1, SX2, SY2: Real);    virtual;
       Destructor  Done;                                     virtual;
     end;

implementation

uses BezDefs;

{---------- SCRIPTCLASS ------------}

Constructor ScriptClass.Init;
begin
  TCollection.Init(20,0);
end;

Procedure ScriptClass.AddScript(Akt: Word; X1, X2, X3, X4, X5, X6: Real);
 var
    DataPtr: PScriptRec;
begin
  if Count>=Limit then
    SetLimit(Count+5);
  new(DataPtr);
  DataPtr^.Aktion := Akt;
  DataPtr^.Data[0] := X1;
  DataPtr^.Data[1] := X2;
  DataPtr^.Data[2] := X3;
  DataPtr^.Data[3] := X4;
  DataPtr^.Data[4] := X5;
  DataPtr^.Data[5] := X6;
  Insert(DataPtr);
end;
 
Procedure ScriptClass.ClearScript;
 var
    i: Integer;
begin
  for i := 0 to Count-1 do
    Dispose(PScriptRec(At(i)));
  DeleteAll;
end;
                          
Procedure ScriptClass.GetData(i: Integer; var Data: ScriptRec);
 var
    DataPtr: PScriptRec;
    X, Y: Real;
begin
  DataPtr := At(i);
  Data := DataPtr^;
  with Data do
  begin
    case Aktion of
      scr_Move    : begin
                      X := Data[0];
                      Y := Data[1];
                      Data[0] := (X*XTx+Y*XTy)*Faktor;
                      Data[1] := (X*YTx+Y*YTy)*Faktor;
                    end;
      scr_Stretch : begin
                      X := Data[0]-FX1;
                      Y := Data[1]-FY1;
                      Data[0] := CX1+(X*XTx+Y*XTy)*Faktor;
                      Data[1] := CY1+(X*YTx+Y*YTy)*Faktor;
                      X := Data[2]-FX1;
                      Y := Data[3]-FY1;
                      Data[2] := CX1+(X*XTx+Y*XTy)*Faktor;
                      Data[3] := CY1+(X*YTx+Y*YTy)*Faktor;
                    end;
      scr_Rot     : begin
                      X := Data[4]-FX1;
                      Y := Data[5]-FY1;
                      Data[4] := CX1+(X*XTx+Y*XTy)*Faktor;
                      Data[5] := CY1+(X*YTx+Y*YTy)*Faktor;
                    end;
    end;
  end;
end;

Procedure ScriptClass.SetPos(SX1, SY1, SX2, SY2: Real);
 var Alpha: Real;
begin
  Faktor := sqrt(Sqr(FX2-FX1)+Sqr(FY2-FY1));
  if Faktor=0 then
    Faktor := 1
  else
    Faktor := sqrt(Sqr(SX2-SX1)+Sqr(SY2-SY1))/Faktor;
  Alpha := ATan2(FX2-FX1, FY2-FY1)-ATan2(SX2-SX1, SY2-SY1);
  XTx := cos(Alpha);
  XTy := -sin(Alpha);
  YTx := sin(Alpha);
  YTy := cos(Alpha);
  CX1 := SX1;
  CY1 := SY1;
  CX2 := SX2;
  CY2 := SY2;
end;

Procedure ScriptClass.StartRecord(SX1, SY1, SX2, SY2: Real);
begin
  CX1 := SX1;
  CY1 := SY1;
  CX2 := SX2;
  CY2 := SY2;
  FX1 := SX1;
  FY1 := SY1;
  FX2 := SX2;
  FY2 := SY2;
  Faktor := 1;
  XTx := 1;
  XTy := 0;
  YTx := 0;
  YTy := 1;
end;

Destructor ScriptClass.Done;
begin
  ClearScript;
  TCollection.Done;
end;                                 


end.