unit Tools;

interface

uses WinTypes, WinProcs, GrTools, Bezier2, Bezier1, BezDefs;

type
    PTool = ^ToolClass;
    ToolClass = Object
      Cursor: HCursor;
      Icon: HIcon;
      Wnd: HWND;
      StartX, StartY,
      OldX, OldY: Integer;
      StartXSnap, StartYSnap,
      OldXSnap, OldYSnap: Integer;
      Constructor Init(Cur: HCursor; Icn: HIcon; AWnd: HWND);
      Procedure   MouseDown(X, Y: Integer; Tasten: Word); virtual;
      Procedure   MouseMove(X, Y: Integer; Tasten: Word); virtual;
      Procedure   MouseUp(X, Y: Integer; Tasten: Word); virtual;
      Procedure   Char(n: Integer); virtual;
      Procedure   SetShift; virtual;
      Procedure   DrawOutline; virtual;
      Procedure   DoIt; virtual;
      Function    SelectTool: Boolean; virtual;
      Procedure   DeselectTool; virtual;
      Destructor  Done; virtual;
    end;

    PZoom = ^ZoomClass;
    ZoomClass = Object(ToolClass)
      Constructor Init(AWnd: HWND);
      Procedure   DrawOutline; virtual;
      Procedure   DoIt; virtual;
    end;

    PSelect = ^SelectClass;
    SelectClass = Object(ZoomClass)
      Constructor Init(AWnd: HWND);
      Procedure   DoIt; virtual;
    end;

    PLineTool = ^LineToolClass;
    LineToolClass = Object(ToolClass)
      Constructor Init(AWnd: HWND);
      Procedure   SetShift; virtual;
      Procedure   DrawOutline; virtual;
      Procedure   DoIt; virtual;
    end;

    PRectTool = ^RectToolClass;
    RectToolClass = Object(LineToolClass)
      Constructor Init(AWnd: HWND);
      Procedure   SetShift; virtual;
      Procedure   DrawOutline; virtual;
      Procedure   DoIt; virtual;
    end;

    PCircleTool = ^CircleToolClass;
    CircleToolClass = Object(RectToolClass)
      Constructor Init(AWnd: HWND);
      Procedure   DrawOutline; virtual;
      Procedure   DoIt; virtual;
    end;

    PSnap = ^SnapToolClass;
    SnapToolClass = Object(ToolClass)
      Selected: Integer;
      Mode: Word;
      Constructor Init(AWnd: HWND);
      Procedure   MouseDown(X, Y: Integer; Tasten: Word); virtual;
      Procedure   MouseMove(X, Y: Integer; Tasten: Word); virtual;
      Procedure   MouseUp(X, Y: Integer; Tasten: Word); virtual;
    end;


    PBezierTool = ^BezierToolClass;
    BezierToolClass = Object(SnapToolClass)
      Constructor Init(AWnd: HWND);
      Procedure   MouseMove(X, Y: Integer; Tasten: Word); virtual;
      Procedure   MouseUp(X, Y: Integer; Tasten: Word); virtual;
      Function    SelectTool: Boolean; virtual;
      Procedure   DeselectTool; virtual;
    end;

    PTextTool = ^TextToolClass;
    TextToolClass = Object(ToolClass)
      CaretPos,
      StartPos: TPoint;
      Constructor Init(AWnd: HWND);
      Procedure   MouseDown(X, Y: Integer; Tasten: Word); virtual;
      Procedure   Char(n: Integer); virtual;
      Function    SelectTool: Boolean; virtual;
      Procedure   DeselectTool; virtual;
    end;


implementation

uses {WObjects}Objects, Defs, Dialogs, Fonts;

{---------------- Globale Variablen -------------}

var
    TheSelectBrush: HBrush;

{---------------- HilfsFunktionen ---------------}

procedure Sort(var X,Y: Integer);
 var h: Integer;
begin
  if X>Y then
  begin
    h := X;
    X := Y;
    Y := h;
  end;
end;

function Sgn(X: Integer): Integer;
begin
  if X=0 then
    Sgn := 0
  else if X<0 then
    Sgn := -1
  else
    Sgn := 1;
end;

 
{---------------- ToolClass ---------------------}

Constructor ToolClass.Init(Cur: HCursor; Icn: HIcon; AWnd: HWND);
begin
    Cursor := Cur;
    Icon := Icn;
    Wnd := AWnd;
end;

Procedure ToolClass.MouseDown(X, Y: Integer; Tasten: Word);
begin
  StartX := X;
  StartY := Y;
  OldX := X;
  OldY := Y;
  StartXSnap := zx(SnapX(StartX));
  StartYSnap := zy(SnapY(StartY));
  OldXSnap := StartXSnap;
  OldYSnap := StartYSnap;
end;

Procedure ToolClass.MouseMove(X, Y: Integer; Tasten: Word);
begin
  OldX := X;
  OldY := Y;
  OldXSnap := zx(SnapX(OldX));
  OldYSnap := zy(SnapY(OldY));
  if (Tasten and mk_Shift)<>0 then
    SetShift;
  if CaretVis then
    HideCaret(Wnd);
  BitBlt(vhDC, 0,0, xw, yw, BufferDC, 0, 0, SrcCopy);
  if CaretVis then
    ShowCaret(Wnd);
  DrawOutline;
end;

Procedure ToolClass.MouseUp(X, Y: Integer; Tasten: Word);
begin
  OldX := X;
  OldY := Y;
  OldXSnap := zx(SnapX(OldX));
  OldYSnap := zy(SnapY(OldY));
  if (Tasten and mk_Shift)<>0 then
    SetShift;
  if CaretVis then
    HideCaret(Wnd);
  BitBlt(vhDC, 0,0, xw, yw, BufferDC, 0, 0, SrcCopy);
  if CaretVis then
    ShowCaret(Wnd);
  DoIt;
end;

procedure ToolClass.Char;
begin
end;

procedure ToolClass.SetShift;
begin
end;

procedure ToolClass.DrawOutline;
begin
end;

procedure ToolClass.DoIt;
begin
end;

function ToolClass.SelectTool: Boolean;
begin
  SelectTool := True;
end;

procedure ToolClass.DeselectTool;
begin
end;


Destructor ToolClass.Done;
begin
  if Cursor <> 0 then
    DeleteObject(Cursor);
  if Icon <> 0 then
    DeleteObject(Icon);
end;



{---------------- ZoomClass ---------------------}

Constructor ZoomClass.Init(AWnd: HWND);
begin
  ToolClass.Init(LoadCursor(HInstance, 'ZoomCursor'),
                 LoadIcon(HInstance, 'ZoomIcon'),
                 AWnd);
end;

Procedure ZoomClass.DrawOutline;
 var
    X, Y: Integer;
    r: TRect;
begin
  X := StartX;
  Y := StartY;
  Sort(X, OldX);
  Sort(Y, OldY); 
  SetRect(r, X, Y, OldX+1, OldY+1);
  DrawFocusRect(vhDC, r);
end;


Procedure ZoomClass.DoIt;
 var
    r: TRect;
begin
  Sort(StartX, OldX);
  Sort(StartY, OldY); 
  if (OldX>StartX+1) or (OldY>StartY+1) then
  begin
    SetZoom( StartX, StartY, OldX, OldY);
    SendMessage(Wnd, wm_Command, cm_SetScrollbars, 0);
    World.DrawAll;
  end;
end;

{---------------- SelectClass ---------------------}

Constructor SelectClass.Init(AWnd: HWND);
begin
  ToolClass.Init(LoadCursor(HInstance, 'SelectCursor'),
                 LoadIcon(HInstance, 'SelectIcon'),
                 AWnd);
end;

Procedure SelectClass.DoIt;
 var
    r: TRect;
begin
  Sort(StartX, OldX);
  Sort(StartY, OldY); 
  if (OldX>StartX+1) or (OldY>StartY+1) then
    SetClip( StartX, StartY, OldX, OldY)
  else
    ResetClip;
end;

{---------------- LineToolClass ---------------------}



Constructor LineToolClass.Init(AWnd: HWND);
begin
  ToolClass.Init(LoadCursor(HInstance, 'LineToolCursor'),
                 LoadIcon(HInstance, 'LineToolIcon'),
                 AWnd);
end;


Procedure LineToolClass.SetShift;
 var
    DX, DY: Integer;
begin
  DX := abs(OldXSnap-StartXSnap);
  DY := abs(OldYSnap-StartYSnap);
  if 2*DX<DY then
    OldXSnap := StartXSnap
  else if 2*DY<DX then
    OldYSnap := StartYSnap
  else if DY<DX then
    OldXSnap := StartXSnap+sgn(OldXSnap-StartXSnap)*DY
  else if DX<DY then
    OldYSnap := StartYSnap+sgn(OldYSnap-StartYSnap)*DX;
end;

Procedure LineToolClass.DrawOutline;
begin
  MoveTo(vhDC, StartXSnap, StartYSnap);
  LineTo(vhDC, OldXSnap, OldYSnap);
end;

Procedure LineToolClass.DoIt;
begin
  World.ClearSelected;
  World.AddLine(SnapX(StartXSnap), SnapY(StartYSnap), SnapX(OldXSnap), SnapY(OldYSnap), Colr[CurrentColor]);
  World.CalcFrame;
  World.Pool.CalcFrame;
  World.Draw;
end;


{---------------- RectToolClass ---------------------}

Constructor RectToolClass.Init(AWnd: HWND);
begin
  ToolClass.Init(LoadCursor(HInstance, 'RectCursor'),
                 LoadIcon(HInstance, 'RectIcon'),
                 AWnd);
end;

Procedure RectToolClass.SetShift;
 var
    DX, DY: Integer;
begin
  DX := abs(OldXSnap-StartXSnap);
  DY := abs(OldYSnap-StartYSnap);
  if DY<DX then
    OldXSnap := StartXSnap+sgn(OldXSnap-StartXSnap)*DY
  else if DX<DY then
    OldYSnap := StartYSnap+sgn(OldYSnap-StartYSnap)*DX;
end;

Procedure RectToolClass.DrawOutline;
 var
    r: TRect;
    X1, Y1, X2, Y2: Integer;
begin
  X1 := StartXSnap;
  Y1 := StartYSnap;
  X2 := OldXSnap;
  Y2 := OldYSnap;
  Sort(X1, X2);
  Sort(Y1, Y2);
  SetRect(r, X1, Y1, X2+1, Y2+1);
  FrameRect(vhDC, r, TheSelectBrush);
end;
  

Procedure RectToolClass.DoIt;
 var
    r: TRect;
    X1, Y1, X2, Y2: Real;
begin
  X1 := SnapX(StartXSnap);
  Y1 := SnapY(StartYSnap);
  X2 := SnapX(OldXSnap);
  Y2 := SnapY(OldYSnap);
  World.ClearSelected;
  World.AddLine( X2, Y2, X2, Y1, Colr[CurrentColor]);
  World.AddLine( X2, Y1, X1, Y1, Colr[CurrentColor]);
  World.AddLine( X1, Y1, X1, Y2, Colr[CurrentColor]);
  World.AddLine( X1, Y2, X2, Y2, Colr[CurrentColor]);
  World.GroupSelected(Colr[CurrentColor], Colr[CurrentFillColor]);
  World.CalcFrame;
  World.Pool.CalcFrame;
  World.draw;
end;





{---------------- CircleToolClass ---------------------}

Constructor CircleToolClass.Init(AWnd: HWND);
begin
  ToolClass.Init(LoadCursor(HInstance, 'CircleCursor'),
                 LoadIcon(HInstance, 'CircleIcon'),
                 AWnd);
end;

Procedure CircleToolClass.DrawOutline;
 var
    X1, Y1, X2, Y2: Integer;
    OldBrush: HBrush;
begin
  OldBrush := SelectObject(vhDC, GetStockObject(Null_Brush));
  X1 := StartXSnap;
  Y1 := StartYSnap;
  X2 := OldXSnap;
  Y2 := OldYSnap;
  Sort(X1, X2);
  Sort(Y1, Y2);
  Ellipse(vhDC, X1, Y1, X2, Y2);
  SelectObject(vhDC, OldBrush);
end;
  


Procedure CircleToolClass.DoIt;
 var
    r: TRect;
    sx, sy, ox, oy,
    DX, DY, XM, YM: Real;
begin
  BitBlt(vhDC, 0,0, xw, yw, BufferDC, 0, 0, SrcCopy);
  World.ClearSelected;
  sx := SnapX(StartXSnap);
  sy := SnapY(StartYSnap);
  ox := SnapX(OldXSnap);
  oy := SnapY(OldYSnap);

  World.AddEllipse(ox, oy, sx, sy, Colr[CurrentColor], Colr[CurrentFillColor]);

  World.CalcFrame;
  World.Pool.CalcFrame;

  World.Draw;
end;





{---------------- SnapToolClass ---------------------}



Constructor SnapToolClass.Init(AWnd: HWND);
begin
  ToolClass.Init(LoadCursor(HInstance, 'SnapCursor'),
                 LoadIcon(HInstance, 'SnapIcon'),
                 AWnd);
end;

Procedure SnapToolClass.MouseDown(X, Y: Integer; Tasten: Word);
 var
    i : Integer;
begin
  ToolClass.MouseDown(X, Y, Tasten);
  i := World.HitsFrameControlPoints(X, Y);
  if i<>-1 then
  begin
    Selected := -3;
    Mode := i;
  end
  else
  begin 
    Selected := World.GetSnap(X-2, Y-2, X+2, Y+2);
    if (Selected<>1) and ((Tasten and mk_shift) = 0) then
      World.ClearSelected;
    if (Selected<>-1) and ((Tasten and mk_shift) <> 0) then
    begin
      World.ToggleHitObject;
      Selected := -2;
    end
    else
    if (Selected=0) then
    begin
      World.ToggleHitObject;
  {    Selected := -2;}
    end;
    World.CalcFrame;
{    SendMessage(Wnd, wm_Command, cm_SaveBitmap, 0);}
  end;
end;

Procedure SnapToolClass.MouseMove(X, Y: Integer; Tasten: Word);
 var
    r: TRect;
    X1, Y1, X2, Y2: Integer;
    dX, dY: Real;
begin
  ToolClass.MouseMove(X, Y, Tasten);
  if Selected=-3 then
  begin
    World.GetSelectedFrame(X1, Y1, X2, Y2);
    if ((Tasten and mk_Shift)<>0) and not (Mode in [1,2,4,8])
       and (X2>X1) and (Y2>Y1) then
    begin
      if (Mode and 1) <> 0 then
        dY := Y2-OldYSnap;
      if (Mode and 2) <> 0 then
        dX := X2-OldXSnap;
      if (Mode and 4) <> 0 then
        dX := OldXSnap-X1;
      if (Mode and 8) <> 0 then
        dY := OldYSnap-Y1;
      dX := dX/(X2-X1);
      dY := dY/(Y2-Y1);
      if dX>dY then
      begin
        if (Mode and 2) <> 0 then
          OldXSnap := trunc(X2-dy*(X2-X1));
        if (Mode and 4) <> 0 then
          OldXSnap := trunc(X1+dy*(X2-X1));
      end
      else
      begin
        if (Mode and 1) <> 0 then
          OldYSnap := trunc(Y2-dx*(Y2-Y1));
        if (Mode and 8) <> 0 then
          OldYSnap := trunc(Y1+dx*(Y2-Y1));
      end;
    end;
    if (Mode and 1) <> 0 then
      Y1 := OldYSnap;
    if (Mode and 2) <> 0 then
      X1 := OldXSnap;
    if (Mode and 4) <> 0 then
      X2 := OldXSnap;
    if (Mode and 8) <> 0 then
      Y2 := OldYSnap;
    SetRect(r, X1, Y1, X2+1, Y2+1);
    if (Y2>Y1) and (X2>X1) then
      DrawFocusRect(vhDC, r);
  end
  else if Selected>-2 then
  begin
    if Selected=-1 then
    begin
      X := StartX;
      Y := StartY;
      Sort(X, OldX);
      Sort(Y, OldY);
      SetRect(r, X, Y, OldX+1, OldY+1);
      DrawFocusRect(vhDC, r);
    end
    else
    begin
      World.GetSelectedFrame(X1, Y1, X2, Y2);
      X := zx(SnapX(X1));
      Y := zy(SnapY(Y1));
      X2 := X2+X-X1;
      Y2 := Y2+Y-Y1;
      if X2<=X1 then
        X2 := X1+1;
      if Y2<=Y1 then
        Y2 :=Y2+1;
      X1 := X;
      Y1 := Y; 
      X := OldXSnap - StartXSnap;
      Y := OldYSnap - StartYSnap;
      SetRect(r, X1+X, Y1+Y, X2+X+1, Y2+Y+1);
      DrawFocusRect(vhDC, r);
    end;
  end;
end;


Procedure SnapToolClass.MouseUp(X, Y: Integer; Tasten: Word);
 var
    i: Integer;
    X1, Y1, X2, Y2, 
    X1New, Y1New, X2New, Y2New: Integer;
    SX1, SY1, SX2, SY2,
    dX, dY: Real;
begin
  ToolClass.MouseUp(X, Y, Tasten);
  if Selected=-3 then
  begin
    World.GetSelectedFrame(X1, Y1, X2, Y2);
    X1New := X1;
    Y1New := Y1;
    X2New := X2;
    Y2New := Y2;    
    if ((Tasten and mk_Shift)<>0) and not (Mode in [1,2,4,8])
       and (X2>X1) and (Y2>Y1) then
    begin
      if (Mode and 1) <> 0 then
        dY := Y2-OldYSnap;
      if (Mode and 2) <> 0 then
        dX := X2-OldXSnap;
      if (Mode and 4) <> 0 then
        dX := OldXSnap-X1;
      if (Mode and 8) <> 0 then
        dY := OldYSnap-Y1;
      dX := dX/(X2-X1);
      dY := dY/(Y2-Y1);
      if dX>dY then
      begin
        if (Mode and 2) <> 0 then
          OldXSnap := trunc(X2-dy*(X2-X1));
        if (Mode and 4) <> 0 then
          OldXSnap := trunc(X1+dy*(X2-X1));
      end
      else
      begin
        if (Mode and 1) <> 0 then
          OldYSnap := trunc(Y2-dx*(Y2-Y1));
        if (Mode and 8) <> 0 then
          OldYSnap := trunc(Y1+dx*(Y2-Y1));
      end;
    end;
    if (Mode and 1) <> 0 then
      Y1New := OldYSnap;
    if (Mode and 2) <> 0 then
      X1New := OldXSnap;
    if (Mode and 4) <> 0 then
      X2New := OldXSnap;
    if (Mode and 8) <> 0 then
      Y2New := OldYSnap;

    if (X2New>X1New) and (Y2New>Y1New) then
    begin
      World.GetFrame(SX1, SY1, SX2, SY2);
{      World.Erase;}
      if (Y2=Y1) then
        Y2 := Y1+1;
      if (X2=X1) then
        X2 := X1+1;
      World.StretchSelected(X1, Y1, X1New, Y1New, 1.0*(X2New-X1New)/(X2-X1),1.0*(Y2New-Y1New)/(Y2-Y1));
      World.RedrawSelected(SX1, SY1, SX2, SY2);
    end;
  end
  else if Selected>-2 then
  begin
    if Selected=-1 then
    begin
      X := StartX;
      Y := StartY;
      Sort(X, OldX);
      Sort(Y, OldY);
      World.SnapAll(X, Y, OldX, OldY);
      World.CalcFrame;
    end
    else
    begin
      if (abs(OldXSnap-StartXSnap)>1) or (abs(OldYSnap-StartYSnap)>1) then
      begin
        World.GetFrame(SX1, SY1, SX2, SY2);
{        World.Erase;}
        World.AlignSelected;
        World.MoveSelected(OldXSnap-StartXSnap, OldYSnap-StartYSnap);
        World.RedrawSelected(SX1, SY1, SX2, SY2);
      end;
    end;
  end;
end;


{---------------- BezierToolClass ---------------------}



Constructor BezierToolClass.Init(AWnd: HWND);
begin
  ToolClass.Init(LoadCursor(HInstance, 'BezierCursor'),
                 LoadIcon(HInstance, 'BezierIcon'),
                 AWnd);
end;

Procedure BezierToolClass.MouseMove(X, Y: Integer; Tasten: Word);
 var
    LocalBezPtr: PBezier;
    LinePtr: PLine;
    Nr, P: Integer;
    OldPen: HPen;
begin
  if Selected<>-3 then
    SnapToolClass.MouseMove(X, Y, Tasten)
  else
  begin
    OldPen := SelectObject(vhDC, TheRedPen);
    ToolClass.MouseMove(X, Y, Tasten);
    Nr := Mode div 4;
    P := Mode mod 4;
    LinePtr := PLine(HitObjectPtr);
    if LinePtr^.GetType=gt_LineClass then
    begin
      case P of
        0 : begin
              MoveTo(vhDC, zx(LinePtr^.P4.X), zy(LinePtr^.P4.Y));
              LineTo(vhDC, OldXSnap, OldYSnap);
            end;
        3 : begin
              MoveTo(vhDC, zx(LinePtr^.P1.X), zy(LinePtr^.P1.Y));
              LineTo(vhDC, OldXSnap, OldYSnap);
            end;
      end;
    end
    else
    begin
      LocalBezPtr := PBezier(LinePtr^.CreateCopy);
      case P of
        0 : begin
              LocalBezPtr^.P1.X := SnapX(OldX);
              LocalBezPtr^.P1.Y := SnapY(OldY);
              LocalBezPtr^.P2.X := LocalBezPtr^.P2.X+LocalBezPtr^.P1.X-LinePtr^.P1.X;
              LocalBezPtr^.P2.Y := LocalBezPtr^.P2.Y+LocalBezPtr^.P1.Y-LinePtr^.P1.Y;
            end;
        1 : begin
              MoveTo(vhDC, zx(LocalBezPtr^.P1.X), zy(LocalBezPtr^.P1.Y));
              LineTo(vhDC, OldX, OldY);
              LocalBezPtr^.P2.X := InvX(OldX);
              LocalBezPtr^.P2.Y := InvY(OldY);
            end;
        2 : begin
              MoveTo(vhDC, zx(LocalBezPtr^.P4.X), zy(LocalBezPtr^.P4.Y));
              LineTo(vhDC, OldX, OldY);
              LocalBezPtr^.P3.X := InvX(OldX);
              LocalBezPtr^.P3.Y := InvY(OldY);
            end;
        3 : begin
              LocalBezPtr^.P4.X := SnapX(OldX);
              LocalBezPtr^.P4.Y := SnapY(OldY);
              LocalBezPtr^.P3.X := LocalBezPtr^.P3.X+LocalBezPtr^.P4.X-LinePtr^.P4.X;
              LocalBezPtr^.P3.Y := LocalBezPtr^.P3.Y+LocalBezPtr^.P4.Y-LinePtr^.P4.Y;
            end;
      end;
      LocalBezPtr^.Draw;
      Dispose(LocalBezPtr,Done);
    end;
    SelectObject(vhDC, OldPen);
  end;
end;

Procedure BezierToolClass.MouseUp(X, Y: Integer; Tasten: Word);
 var
    LocalBezPtr: PBezier;
    LinePtr: PLine;
    GroupPtr: PGroup;
    Nr, P: Integer;
    OldPen: HPen;
    X1, Y1, X2, Y2: Real;
begin
  if Selected<>-3 then
    SnapToolClass.MouseUp(X, Y, Tasten)
  else
  begin
    ToolClass.MouseUp(X, Y, Tasten);
    Nr := Mode div 4;
    P := Mode mod 4;
    LinePtr := PLine(HitObjectPtr);

    X1 := LinePtr^.PMin.X;
    Y1 := LinePtr^.PMin.Y;
    X2 := LinePtr^.PMax.X;
    Y2 := LinePtr^.PMax.Y;

{    LinePtr^.Erase;}
    if LinePtr^.GetType = gt_LineClass then
    begin
      case P of
        0 : begin
              LinePtr^.assign(   SnapX(OldX),   SnapY(OldY),
                               LinePtr^.P4.X, LinePtr^.P4.Y,
                                 SnapX(OldX),   SnapY(OldY),
                               LinePtr^.P4.X, LinePtr^.P4.Y);
            end;
        3 : begin
              LinePtr^.assign( LinePtr^.P1.X, LinePtr^.P1.Y,
                                 SnapX(OldX),   SnapY(OldY),
                               LinePtr^.P1.X, LinePtr^.P1.Y,
                                 SnapX(OldX),   SnapY(OldY));
            end;
      end;
    end
    else
    begin
      LocalBezPtr := PBezier(LinePtr^.CreateCopy);
      case P of
        0 : begin
              LocalBezPtr^.P1.X := SnapX(OldX);
              LocalBezPtr^.P1.Y := SnapY(OldY);
              LocalBezPtr^.P2.X := LocalBezPtr^.P2.X+LocalBezPtr^.P1.X-LinePtr^.P1.X;
              LocalBezPtr^.P2.Y := LocalBezPtr^.P2.Y+LocalBezPtr^.P1.Y-LinePtr^.P1.Y;
            end;
        1 : begin
              LocalBezPtr^.P2.X := InvX(OldX);
              LocalBezPtr^.P2.Y := InvY(OldY);
            end;
        2 : begin
              LocalBezPtr^.P3.X := InvX(OldX);
              LocalBezPtr^.P3.Y := InvY(OldY);
            end;
        3 : begin
              LocalBezPtr^.P4.X := SnapX(OldX);
              LocalBezPtr^.P4.Y := SnapY(OldY);
              LocalBezPtr^.P3.X := LocalBezPtr^.P3.X+LocalBezPtr^.P4.X-LinePtr^.P4.X;
              LocalBezPtr^.P3.Y := LocalBezPtr^.P3.Y+LocalBezPtr^.P4.Y-LinePtr^.P4.Y;
            end;
      end;
      with LocalBezPtr^ do
        LinePtr^.Assign( P1.X, P1.Y, P2.X, P2.Y,
                         P3.X, P3.Y, P4.X, P4.Y);
      Dispose(LocalBezPtr,Done);
    end;
    X1 := min(X1, LinePtr^.PMin.X);
    Y1 := min(Y1, LinePtr^.PMin.Y);
    X2 := max(X2, LinePtr^.PMax.X);
    Y2 := max(Y2, LinePtr^.PMax.Y);

    LinePtr^.Draw;
    GroupPtr := LinePtr^.Group;
    if (GroupPtr^.PolygonPtr<>nil) then
      GroupPtr^.PolygonPtr^.MaxZoom := 0;
    while GroupPtr<>nil do
    begin
      GroupPtr^.CalcFrame;
      GroupPtr := GroupPtr^.Group;
    end;
    World.Redraw(X1, Y1, X2, Y2);
  end;
end;


function BezierToolClass.SelectTool: Boolean;
begin
  World.ShowFrame;
  FrameMode := FrameMode or fm_Bezier;
  World.ShowFrame;
  SendMessage(Wnd, WM_Command, cm_SaveBitmap, 0);
  SelectTool := True;
end;

procedure BezierToolClass.DeselectTool;
begin
  World.ShowFrame;
  FrameMode := FrameMode and not fm_Bezier;
  World.ShowFrame;
  SendMessage(Wnd, WM_Command, cm_SaveBitmap, 0);
end;

{---------------- TextToolClass ---------------------}

Constructor TextToolClass.Init(AWnd: HWND);
begin
  ToolClass.Init(LoadCursor(0, idc_IBeam),
                 LoadIcon(HInstance, 'TextIcon'),
                 AWnd);
end;

Procedure TextToolClass.MouseDown(X, Y: Integer; Tasten: Word);
begin
  ToolClass.MouseDown(X, Y, Tasten);
  CaretPos.X := StartXSnap;
  CaretPos.Y := StartYSnap+8;
  StartPos := CaretPos;
  HideCaret(Wnd);
  SetCaretPos(CaretPos.X, CaretPos.Y);
  ShowCaret(Wnd);
end;

Procedure TextToolClass.Char(n: Integer);
 var
    GroupPtr: PGroup;
    Width,
    X1, Y1, X2, Y2: Real;
begin
  HideCaret(Wnd);
  case lo(n) of
   13      : begin
               CaretPos.X := StartPos.X;
               CaretPos.Y := CaretPos.Y+zy(0)-zy(abs(Font.GetHeight));
             end;
   32..255 : begin
               GroupPtr := Font.GetLetter(lo(n), Width);
               if GroupPtr<>nil then
               begin
                 World.ClearSelected;
                 GroupPtr^.Move(InvX(CaretPos.X), InvY(CaretPos.Y-1));
                 World.AddObject(GroupPtr);
                 World.Pool.AddObject(GroupPtr);
                 World.CalcFrame;
                 World.Pool.CalcFrame;
                 World.GetFrame(X1, Y1, X2, Y2);
                 World.RedrawSelected(X1, Y1, X2, Y2);
                 World.ClearSelected;
                 World.CalcFrame;
               end;
               CaretPos.X := CaretPos.X+zx(Width)-zx(0);
             end;
  end;  
  SetCaretPos(CaretPos.X, CaretPos.Y);
  ShowCaret(Wnd);
end;

function TextToolClass.SelectTool: Boolean;
 var
    P: TPoint;
begin
  DestroyCaret;
  CreateCaret(Wnd, 0, 2, zy(Font.GetHeight)-zy(0));
  ShowCaret(Wnd);
  CaretVis := True;
  GetCaretPos(CaretPos);
  StartPos := CaretPos;
  SelectTool := True;
end;

Procedure TextToolClass.DeselectTool;
begin
  HideCaret(Wnd);
  CaretVis := False;
end;


{---------------- Unitstuff ---------------------}

var
  SaveExit: Pointer;

procedure ToolsExit;
far;
begin
  ExitProc := SaveExit;

  DeleteObject(TheSelectBrush);
end;

{ initialization }
begin
  TheSelectBrush := CreateSolidBrush(Colr[CurrentColor]);

  SaveExit := ExitProc;
  ExitProc := @ToolsExit;
end.

