unit ToolWindow;

interface

uses {WObjects}Objects, WinTypes, WinProcs, Defs, Dialogs,
     GrTools, Script, OWindows;


Const
    TWSize = 6;

type
  PCloseBox = ^TCloseBox;
  PPaletteWindow = ^TPaletteWindow;
  PToolWindow = ^TToolWindow;

  TCloseBox = object(TWindow)
    constructor Init(AParent: PWindowsObject);
    procedure   Paint(PaintDC: HDC; var PaintInfo: TPaintStruct); virtual;
    procedure WMLButtonDown(var Msg: TMessage);
      virtual wm_First + wm_LButtonDown;
  end;

  TPaletteWindow = object(TWindow)
    Minimized: Boolean;
    CloseBox: PCloseBox;
    ShowColor: PWindowsObject;
    constructor Init(AParent, AShowColor: PWindowsObject);
    procedure GetWindowClass(var AWndClass: TWndClass); virtual;
    function  GetClassName: PChar; virtual;
    procedure Paint(PaintDC: HDC; var PaintInfo: TPaintStruct); virtual;
    procedure WMLButtonDblClk(var Msg: TMessage);
      virtual wm_First + wm_LButtonDblClk;
    procedure WMRButtonDblClk(var Msg: TMessage);
      virtual wm_First + wm_RButtonDblClk;
    procedure WMLButtonDown(var Msg: TMessage);
      virtual wm_First + wm_LButtonDown;
    procedure WMRButtonDown(var Msg: TMessage);
      virtual wm_First + wm_RButtonDown;
    procedure WMMove(var Msg: TMessage);
      virtual wm_First + wm_Move;
    destructor Done; virtual;
  end;

  TToolWindow = object(TWindow)
    Minimized: Boolean;
    CloseBox: PCloseBox;
    Bmps: array[0..2*TWSize-1] of HBitmap;

    { Creation and destruction }
    constructor Init(AParent: PWindowsObject);
    destructor Done; virtual;

    { Display }
    procedure Paint(PaintDC: HDC; var PaintInfo: TPaintStruct); virtual;

    { Window manager responses }

    procedure WMLButtonDown(var Msg: TMessage);
      virtual wm_First + wm_LButtonDown;
    procedure WMMove(var Msg: TMessage);
      virtual wm_First + wm_Move;

    procedure ToolSelect(n: Integer); virtual;
  end;


implementation

uses Bezier2, BezDefs;

{-------------- TPALETTEWINDOW -----------------}

constructor TPaletteWindow.Init(AParent, AShowColor: PWindowsObject);
begin
  TWindow.Init(AParent, nil);
  ShowColor := AShowColor;
  Attr.Style := ws_PopUp or ws_Visible or ws_caption or ws_VScroll;
  Attr.W := 4*15+1+20;
  Attr.H :=20+4*15;
  Minimized := False;
  CloseBox := New(PCloseBox, Init(@Self));
  TPActive := True;
  Scroller := New(PScroller, Init(@Self, 0, 1, 0, 59*15));
  Scroller^.YLine := 15;
  Scroller^.YPage := 60;

end;

procedure TPaletteWindow.GetWindowClass(var AWndClass: TWndClass);
begin
  TWindow.GetWindowClass(AWndClass);
  AWndClass.Style := AWndClass.Style or cs_DblClks;
end;


function TPaletteWindow.GetClassName: PChar;
begin
  GetClassName := 'PalettenWindow';
end;


procedure TPaletteWindow.Paint(PaintDC: HDC; var PaintInfo: TPaintStruct);
var
  i, j: Integer;		{ Position of tool in row of icons }
  r: TRect;		{ Coordinates of icon }
  Br: HBrush;
  OldPen: HPen;
begin
  MoveWindow(CloseBox^.HWindow, Attr.X+3, Attr.y+3, 6, 6, True);
  if not Minimized then
  begin
    for j := IMax(0, Scroller^.YPos div 15) to IMin(63, 4+(Scroller^.YPos div 15)) do
    begin
      for i := 0 to 3 do
      begin
        SetRect(r, i*15-1, j*15-1, i*15+15, j*15+15);
        if Colr[j*4+i]=-1 then
        begin
          FillRect(PaintDC, r, GetStockObject(White_Brush));
          OldPen := SelectObject(PaintDC, GetStockObject(Black_Pen));
          MoveTo(PaintDC, i*15,  j*15);
          LineTo(PaintDC, i*15+14, j*15+14);
          MoveTo(PaintDC, i*15-1,  j*15+14);
          LineTo(PaintDC, i*15+14, j*15-1);
          SelectObject(PaintDC, OldPen);
        end
        else
        begin
          Br := CreateSolidBrush(Colr[j*4+i]);
          FillRect(PaintDC, r, Br);
          DeleteObject(Br);
        end;
        FrameRect(PaintDC, r, GetStockObject(Black_Brush));
        if (4*j+i=CurrentColor) or (4*j+i=CurrentFillColor) then
        begin
          if 4*j+i=CurrentColor then
          begin
            SetRect(r, i*15, j*15, i*15+14, j*15+14);
            InvertRect(PaintDC, r);
            SetRect(r, i*15+1, j*15+1, i*15+13, j*15+13);
            InvertRect(PaintDC, r);
          end;
          if 4*j+i=CurrentFillColor then
          begin
            SetRect(r, i*15+1, j*15+1, i*15+13, j*15+13);
            InvertRect(PaintDC, r);
            SetRect(r, i*15+2, j*15+2, i*15+12, j*15+12);
            InvertRect(PaintDC, r);
          end;
        end;
      end;
    end;
  end;
  SetActiveWindow(Parent^.HWindow);
end;

procedure TPaletteWindow.WMLButtonDblClk(var Msg: TMessage);
 var
    NewColor: LongInt;
begin
  NewColor := Colr[CurrentColor];
  if ChooseColorDialog(HWindow, NewColor) then
  begin
    Colr[CurrentColor] := NewColor;
    InvalidateRect(ShowColor^.HWindow, nil, False);
    InvalidateRect(HWindow, nil, False);
    SendMessage(Parent^.HWindow, WM_Command, cm_SetMenu, 0);
  end;
end;

procedure TPaletteWindow.WMRButtonDblClk(var Msg: TMessage);
 var
    NewColor: LongInt;
begin
  NewColor := Colr[CurrentFillColor];
  if ChooseColorDialog(HWindow, NewColor) then
  begin
    Colr[CurrentFillColor] := NewColor;
    InvalidateRect(ShowColor^.HWindow, nil, False);
    InvalidateRect(HWindow, nil, False);
    SendMessage(Parent^.HWindow, WM_Command, cm_SetMenu, 0);
  end;
end;


procedure TPaletteWindow.WMLButtonDown(var Msg: TMessage);
var
  i, j: Integer;
  r: TRect;
  ADC: HDC;
begin
    i := Integer(Msg.LParamLo) div 15;
    j := (Scroller^.YPos + Integer(Msg.LParamHi)) div 15;
    if (i>=0) and (i<=3) and (j>=0) and (j<=63) then
    begin
      CurrentColor := 4*j+i;
      InvalidateRect(ShowColor^.HWindow, nil, False);
      InvalidateRect(HWindow, nil, False);
      SendMessage(Parent^.HWindow, WM_Command, cm_SetMenu, 0);
    end;
end;

procedure TPaletteWindow.WMRButtonDown(var Msg: TMessage);
var
  i, j: Integer;
  r: TRect;
  ADC: HDC;
begin
    i := Integer(Msg.LParamLo) div 15;
    j := (Scroller^.YPos + Integer(Msg.LParamHi)) div 15;
    if (i>=0) and (i<=3) and (j>=0) and (j<=63) then
    begin
      CurrentFillColor := 4*j+i;
      InvalidateRect(ShowColor^.HWindow, nil, False);
      InvalidateRect(HWindow, nil, False);
      SendMessage(Parent^.HWindow, WM_Command, cm_SetMenu, 0);
    end;
end;

procedure TPaletteWindow.WMMove(var Msg: TMessage);
 var
    x, y: Integer;
begin
  x := Integer(Msg.LParamLo);
  y := Integer(Msg.LParamHi);

  TWindow.WMMove(Msg);

  OldTP1 := TP1;
  TP1.X := Attr.x;
  TP1.Y := Attr.y;
  TP2.X := Attr.x+Attr.w;
  TP2.Y := Attr.y+Attr.h;
  MoveWindow(CloseBox^.HWindow, Attr.X+3, Attr.y+3, 6, 6, True);
  SetActiveWindow(Parent^.HWindow);
end;



destructor TPaletteWindow.Done;
var
  i: Integer;
begin
  TPActive := False;
  TP1.X := OldTP1.X+1;
  PostMessage(Parent^.HWindow, WM_Command, cm_SetMenu, 0);
  Dispose(CloseBox, Done);
  TWindow.Done;
end;

{-------------- TTOOLWINDOW -----------------}

constructor TToolWindow.Init(AParent: PWindowsObject);
begin
  TWindow.Init(AParent, nil);
  Attr.Style := ws_PopUp or ws_Visible or ws_caption;
  Attr.W := 31;
  Attr.H :=20+TWSize*15;
  Minimized := False;
  BMPs[0] := LoadBitmap(HInstance, 'PfeilLinks');
  BMPs[1] := LoadBitmap(HInstance, 'PfeilRunter');
  BMPs[2] := LoadBitmap(HInstance, 'PfeilRechts');
  BMPs[3] := LoadBitmap(HInstance, 'PfeilHoch');
  BMPs[4] := LoadBitmap(HInstance, 'HMirror');
  BMPs[5] := LoadBitmap(HInstance, 'VMirror');
  BMPs[6] := LoadBitmap(HInstance, 'Rotieren');
  BMPs[7] := LoadBitmap(HInstance, 'Move');
  BMPs[8] := LoadBitmap(HInstance, 'InKurveUmwandeln');
  BMPs[9] := LoadBitmap(HInstance, 'InLinieUmwandeln');
  BMPs[10] := LoadBitmap(HInstance, 'LinienAnpassen');
  BMPs[11] := LoadBitmap(HInstance, 'Farben');
  CloseBox := New(PCloseBox, Init(@Self));
  TWActive := True;
end;


procedure TToolWindow.Paint(PaintDC: HDC; var PaintInfo: TPaintStruct);
var
  i, j: Integer;		{ Position of tool in row of icons }
  r: TRect;		{ Coordinates of icon }
  MemDC: HDC;
  MemBitmap: HBitmap;
begin
  MoveWindow(CloseBox^.HWindow, Attr.X+3, Attr.y+3, 6, 6, True);
  if not Minimized then
  begin
    MemDC := CreateCompatibleDC(PaintDC);
    MemBitmap := SelectObject(MemDC, BMPs[0]);
    for I := 0 to TWSize-1 do
      for j := 0 to 1 do
      begin
        SelectObject(MemDC, BMPs[i*2+j]);
        BitBlt(PaintDC,15*j-1 ,15*i-1, 16, 16,MemDC, 0, 0, SrcCopy);
    end;
    SelectObject(MemDC, MemBitmap);
    DeleteDC(MemDC);
  end;
end;

procedure TToolWindow.WMLButtonDown(var Msg: TMessage);
var
  i, j: Integer;
  r: TRect;
  ADC: HDC;
begin
  if not ToolsActive then
  begin
    ToolsActive := True;
    ToolsBusy := True;
    j := Integer(Msg.LParamLo) div 15;
    i := Integer(Msg.LParamHi) div 15;
    if (i>=0) and (i<TWSize) and (j>=0) and (j<=1) then
    begin
      ADC := GetDC(HWindow);
      SetRect(r, j*15, i*15, j*15+14, i*15+14);
      InvertRect(ADC, r);
      ToolSelect(2*i+j);
      InvertRect(ADC, r);
      ReleaseDC(HWindow, ADC);
    end;
    ToolsActive := False;
  end;  
  SetActiveWindow(Parent^.HWindow);
end;

procedure TToolWindow.WMMove(var Msg: TMessage);
 var
    x, y: Integer;
begin
  x := Integer(Msg.LParamLo);
  y := Integer(Msg.LParamHi);

  TWindow.WMMove(Msg);

  OldTW1 := TW1;
  TW1.X := Attr.x;
  TW1.Y := Attr.y;
  TW2.X := Attr.x+Attr.w;
  TW2.Y := Attr.y+Attr.h;
  MoveWindow(CloseBox^.HWindow, Attr.X+3, Attr.y+3, 6, 6, True);
  SetActiveWindow(Parent^.HWindow);
end;


procedure TToolWindow.ToolSelect(n: Integer);
 var
    i: Integer;
    SX1, SY1, SX2, SY2: Real;
    Buffer: TRotateInfo;
    MBuffer: TMoveInfo;
begin
  case n of
    0..3 : begin
             i := 1 shl n;
             World.ShowFrame;
             World.GetFrame(SX1, SY1, SX2, SY2);
             World.Adjust(i);
             World.CalcFrame;
             World.Pool.CalcFrame;
             World.Clear(SX1, SY1, SX2, SY2);
             World.RedrawSelected(SX1, SY1, SX2, SY2);
             World.ShowFrame;
             SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0)
           end;
    4    : begin
             World.ShowFrame;
             World.GetFrame(SX1, SY1, SX2, SY2);
             World.HMirror((SX1+SX2)/2);
             World.CalcFrame;
             World.Pool.CalcFrame;
             World.Clear(SX1, SY1, SX2, SY2);
             World.RedrawSelected(SX1, SY1, SX2, SY2);
             World.ShowFrame;
             SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0);
           end;
    5    : begin
             World.ShowFrame;
             World.GetFrame(SX1, SY1, SX2, SY2);
             World.VMirror((SY1+SY2)/2);
             World.CalcFrame;
             World.Pool.CalcFrame;
             World.Clear(SX1, SY1, SX2, SY2);
             World.RedrawSelected(SX1, SY1, SX2, SY2);
             World.ShowFrame;
             SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0);
           end;
    6    : begin
             SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0);
             Buffer.Alpha := 90;
             if Application^.ExecDialog(new(PRotateDialog, Init(nil, @Buffer)))= id_ok then
             begin
               DisableWindowTime;
               BitBlt(vhDC, 0, 0, xw, yw, BufferDC, 0,0, SrcCopy);
               World.ShowFrame;
               World.GetFrame(SX1, SY1, SX2, SY2);
               World.Rotate(Buffer.Alpha, (SX1+SX2)/2, (SY1+SY2)/2);
               World.CalcFrame;
               World.Pool.CalcFrame;
               World.Clear(SX1, SY1, SX2, SY2);
               World.RedrawSelected(SX1, SY1, SX2, SY2);
               World.ShowFrame;
               SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0);
               EnableWindowTime;
             end;
           end;
    7    : begin
             SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0);
             MBuffer.X1 := 0;
             MBuffer.Y1 := 0;
             MBuffer.X2 := 1;
             MBuffer.Y2 := 1;
             if Application^.ExecDialog(new(PMoveDialog, Init(nil, @MBuffer)))= id_ok then
              if (MBuffer.X1<>0) or (MBuffer.Y1<>0) or
                 (MBuffer.X2<>1) or (MBuffer.Y2<>1) then
              begin
               DisableWindowTime;
               BitBlt(vhDC, 0, 0, xw, yw, BufferDC, 0,0, SrcCopy);
               World.ShowFrame;
               World.GetFrame(SX1, SY1, SX2, SY2);
               if (MBuffer.X2<>1) or (MBuffer.Y2<>1) then
               begin
                 World.Move(-(SX1+SX2)/2, -(SY1+SY2)/2);
                 World.Stretch(MBuffer.X2, MBuffer.Y2);
                 World.Move((SX1+SX2)/2, (SY1+SY2)/2);
                 if ScriptRecord then
                   World.Script.AddScript(scr_Stretch, (SX1+SX2)/2, (SY1+SY2)/2,
                                                       (SX1+SX2)/2, (SY1+SY2)/2,
                                                        MBuffer.X2,  MBuffer.Y2);
               end;
               if (MBuffer.X1<>0) or (MBuffer.Y1<>0) then
               begin
                 World.Move(MBuffer.X1, MBuffer.Y1);
                 if ScriptRecord then
                   World.Script.AddScript(scr_Move,MBuffer.X1, MBuffer.Y1, 0,0,0,0);
               end;
               World.CalcFrame;
               World.Pool.CalcFrame;
               World.Clear(SX1, SY1, SX2, SY2);
               World.RedrawSelected(SX1, SY1, SX2, SY2);
               World.ShowFrame;
               SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0);
               EnableWindowTime;
             end;
           end;
    8    : begin
             World.ShowFrame;
             World.GetFrame(SX1, SY1, SX2, SY2);
             World.MakeBezier;
             World.Clear(SX1, SY1, SX2, SY2);
             World.RedrawSelected(SX1, SY1, SX2, SY2);
             World.ShowFrame;
             SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0)
           end;
    9    : begin
             World.ShowFrame;
             World.GetFrame(SX1, SY1, SX2, SY2);
             World.MakeLine;
             World.Clear(SX1, SY1, SX2, SY2);
             World.RedrawSelected(SX1, SY1, SX2, SY2);
             World.ShowFrame;
             SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0)
           end;
    10   : begin
             World.ShowFrame;
             World.GetFrame(SX1, SY1, SX2, SY2);
             World.SetLineColor;
             World.Clear(SX1, SY1, SX2, SY2);
             World.RedrawSelected(SX1, SY1, SX2, SY2);
             World.ShowFrame;
             SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0)
           end;
    11    : begin
             World.ShowFrame;
             World.GetFrame(SX1, SY1, SX2, SY2);
             World.SetSelectedColor(Colr[CurrentColor], Colr[CurrentFillColor]);
             World.Clear(SX1, SY1, SX2, SY2);
             World.RedrawSelected(SX1, SY1, SX2, SY2);
             World.ShowFrame;
             SendMessage(TheWindow, WM_Command, cm_SaveBitmap, 0)
           end;
  end;
end;


destructor TToolWindow.Done;
var
  i: Integer;
begin
  TWActive := False;
  TW1.X := OldTW1.X+1;
  PostMessage(Parent^.HWindow, WM_Command, cm_SetMenu, 0);
  for i := 0 to 2*TWSize-1 do
    DeleteObject(BMPs[i]);
  Dispose(CloseBox, Done);
  TWindow.Done;
end;

{----------------------- TCloseBox ---------------------}

constructor TCloseBox.Init(AParent: PWindowsObject);
begin
  TWindow.Init(AParent, nil);
  Attr.Style := ws_PopUp or ws_Visible;
end;


procedure TCloseBox.Paint(PaintDC: HDC; var PaintInfo: TPaintStruct);
var
  r: TRect;		
begin
  SetRect(r, 0, 0, 6, 6);
  FrameRect(PaintDC, r, GetStockObject(Black_Brush));
  SetRect(r, 1, 1, 5, 5);
  FillRect(PaintDC, r, GetStockObject(White_Brush));
end;

procedure TCloseBox.WMLButtonDown(var Msg: TMessage);
begin
  PostMessage(Parent^.HWindow, wm_Close, 0, 0);
end;

end.